C+    detect_hopf.f
C This file was written by Jonathan A. Sherratt
C Copyright (C) 2011 Heriot-Watt University, Edinburgh, UK
C  
C THIS FILE IS PART OF WAVETRAIN.
C WAVETRAIN SOFTWARE IS INTENDED FOR FREE PERSONAL PRIVATE 
C AND ACADEMIC USE ONLY AND WAS DEVELOPED BY JONATHAN SHERRATT AT 
C HERIOT-WATT UNIVERSITY, EDINBURGH, UK, AND IS PROTECTED BY COPYRIGHT. 
C THE COPYRIGHT OWNER IS HERIOT-WATT UNIVERSITY, WITH THE EXCEPTION OF 
C THE ROUTINES FROM THE PACKAGES AUTO, BLAS, EISPACK, LAPACK AND MINPACK, 
C WHICH ARE INCLUDED WITH PERMISSION, WHICH ARE CLEARLY IDENTIFIED IN 
C THE SOURCE CODE, AND THE USE OF THESE PACKAGES IS UNDER THE CONDITIONS 
C OF THE ORIGINAL OWNERS WHICH ARE AVAILABLE AT 
C http://www.netlib.org/blas/faq.html#2;
C http://www.netlib.org/lapack/#_licensing; 
C http://www.netlib.org/minpack/disclaimer.
C 
C PERSONS OR ORGANISATIONS WISHING TO USE THIS WAVETRAIN SOFTWARE FOR 
C COMMERCIAL PURPOSES OR PURPOSES OTHER THAN ITS ORIGINAL INTENTION 
C SHOULD CONTACT THE OWNERS TO REQUEST SPECIFIC WRITTEN PERMISSIONS.
C 
C DISTRIBUTION: DISTRIBUTION OF WAVETRAIN SOFTWARE, IN PART OR AS 
C A WHOLE, BY ANY MEANS (ELECTRONIC, PRINTED COPY OR OTHERWISE), IS 
C PROHIBITED WITHOUT THE PRIOR WRITTEN PERMISSION OF EITHER HERIOT-WATT 
C UNIVERSITY OR THE AUTHOR JONATHAN A. SHERRATT. ALL REQUESTS FOR SUCH 
C PERMISSION SHOULD BE SENT BY EMAIL TO J.A.SHERRATT@HW.AC.UK; PLEASE 
C INCLUDE THE WORD WAVETRAIN IN THE SUBJECT HEADING OF THE MESSAGE.  
C 
C WARRANTY LIMITATIONS: THE PRODUCT AND SERVICE ARE PROVIDED "AS IS". 
C THE OWNER DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING BUT NOT LIMITED 
C TO, ALL EXPRESSED OR IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
C FOR A PARTICULAR PURPOSE. UNDER NO CIRCUMSTANCES WILL THE OWNER BE 
C LIABLE TO YOU FOR ANY LOSS OF USE, INTERRUPTION OF BUSINESS OR ANY 
C DIRECT, INDIRECT, SPECIAL, INCIDENTAL, AND/OR CONSEQUENTIAL DAMAGES 
C OF ANY KIND (INCLUDING LOSS OF PROFITS) EVEN IF THE OWNER HAS BEEN 
C ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. THE OWNER ACCEPTS NO LEGAL 
C LIABILITY FOR ANY OF THE PRODUCT AND/OR SERVICE SOLD, OFFERED FOR SALE, 
C OR OTHERWISE MADE AVAILABLE FOR PUBLIC OR PRIVATE CONSUMPTION IN 
C RELATION TO THIS SOFTWARE. THE OWNER DOES NOT WARRANT THAT THIS SOFTWARE 
C WILL BE ERROR-FREE.
C  
C+    THIS PROGRAM CHECKS WHETHER AUTO HAS DETECTED A CHANGE IN STABILITY
C+    OF HOPF TYPE FOR THE PERIODIC TRAVELLING WAVE SOLUTIONS.
C+    IF A HOPF POINT IS FOUND, THE CORRESPONDING AUTO LABEL FOR THE 
C+      POINT IS WRITTEN TO THE ONE LINE FILE data_files/hopf.label
C+    IF NOT, idtect IS GIVEN A VALUE AS FOLLOWS:
C+               idtect=12: AUTO COMPLETED CONTINUATION ACROSS THE RANGE
C+                          OF PARAMETERS VALUES SPECIFIED IN THE CALL
C+                          STATEMENT, BUT WITHOUT DETECTING AN HOPF 
C+                          POINT.
C+               idtect=10: CONVERGENCE PROBLEM IN AUTO.
C+               idtect=11: AUTO FINISHED CONTINUATION WITHOUT COVERING
C+                           THE WHOLE RANGE OF PARAMETER VALUES  
C+                           SPECIFIED IN THE CALL STATEMENT. THE MOST 
C+                           LIKELY REMEDY FOR THIS WOULD BE TO EITHER 
C+                           ADD MORE CONTINUATION STEPS OR MAKE THE ds
C+                           VALUES LARGER. (OR TO SPECIFY A SMALLER
C+                           RANGE OVER WHICH TO SEARCH FOR THE HOPF
C+                           POINT).
C+    NOTE THAT THE PRINCIPAL CONTINUATION PARAMETER CAN BE 
C+      EITHER pp OR c.
C+    A NOTE ABOUT THE METHODOLOGY: IN MANY OTHER AUTO RUNS THE OUTPUT
C+      LABEL HAS BEEN SPECIFIED AS NEGATIVE SO THAT AUTO OUTPUTS WHEN
C+      IT REACHES THE REQUIRED VALUE. THEN IT IS NECESSARY TO ASSESS
C+      WHETHER OR NOT THE REQUIRED OUTPUT POINT HAS BEEN REACHED. FOR 
C+      INSTANCE, THE PARAMETERS periodatol AND periodrtol ARE USED FOR
C+      THIS PURPOSE WHEN SEARCHING FOR A SOLUTION OF GIVEN PERIOD IN
C+      THE periodcontour PROGRAMS. HOWEVER I COULD NOT SEE ANY WAY OF
C+      ASSESSING A PRIORI A SUITABLE THRESHOLD FOR PAR(5) AND SO I HAVE
C+      LEFT AUTO TO ASSESS WHEN IT BECOMES ZERO. THE DOWNSIDE OF THIS
C+      IS THE EXTRA COMPUTATION TIME WHILE AUTO CONTINUES THE SOLUTION
C+      AFTER PAR(5) HAS CHANGED SIGN, BUT THERE SEEMS NO SIMPLE WAY OF 
C+      AVOIDING THIS.
C++
      IMPLICIT REAL*8 (A-H,K,O-Z)
      CHARACTER*1 ADTECT
      CALL GINFO(INFO)
      OPEN(12,FILE='fort.7'
     &,STATUS='OLD')
      REWIND(12)
      ADTECT='n'
      IFINAL=0
      DO 600 I=1,12
         READ(12,*,END=101)
 600  CONTINUE
      DO 100 I=1,100000
         IFINAL=I
         READ(12,*,END=101)IDUMMY,IPT,ITY,ILAB,PPC,RNORM
         IF (ITY.EQ.(-4)) THEN
            OPEN(19,FILE='data_files/hopf.label'
     &,STATUS='NEW')
            WRITE(19,'(I8)')ILAB
            CLOSE(19)
            ADTECT='y'
            GOTO 102
         ENDIF
         IF (ITY.EQ.(-9)) THEN
            IDTECT=10
            GOTO 102
         ENDIF
 100  CONTINUE
      OPEN(66,FILE='../temp_files/fortran.error'
     &,STATUS='NEW')
      WRITE(66,'(''ERROR: END OF LOOP 100 IN detect_hopf.f'')')
      CLOSE(66)
      CLOSE(12)
      ADTECT='n'
      WRITE(6,9000)ADTECT
      CALL ISTOP(INFO)
 101  CONTINUE
C     auto converged but didn't find an Hopf point
      IF (IFINAL.EQ.1) THEN
         OPEN(66,FILE='../temp_files/fortran.error'
     &,STATUS='NEW')
         WRITE(66,'(''ERROR: NO REAL OUTPUT LINES IN FILE 12'',
     &        '' in detect_hopf.f'')')
         CLOSE(66)
         CLOSE(12)
         ADTECT='n'
         WRITE(6,9000)ADTECT
         CALL ISTOP(INFO)
      ENDIF
      OPEN(17,FILE='../temp_files/sbdy.startdata'
     &,STATUS='OLD')
      REWIND(17)
      READ(17,*)PP1,C1,PP2,C2
      CLOSE(17)
      OPEN(16,FILE='../temp_files/defaults.input'
     &,STATUS='OLD')
      REWIND(16)
      READ(16,*)PPATOL,PPRTOL
      READ(16,*)CATOL,CRTOL
      CLOSE(16)
      IF (C1.EQ.C2) THEN
         IF (ABS(PPATOL).GT.ABS(0.01D0*PP2)) THEN
            OPEN(69,FILE='../temp_files/detect.warning1'
     &,STATUS='NEW')
            WRITE(69,'(/,
     &''WARNING: THE VALUE OF controlatol IS RATHER LARGE'',/,
     &''         COMPARED TO THE CONTROL PARAMETER VALUE'',/,
     &''         AT THE END OF THE SEARCH RANGE.'',/,
     &''         THIS MAY BE APPROPRIATE, BUT IT MIGHT'',/,
     &''         ALSO INDICATE THAT controlatol SHOULD'',/,
     &''         BE REDUCED IN defaults.input''
     &,/)')
            CLOSE(69)
         ENDIF
         TOL=PPATOL+ABS(PPRTOL*PP2)
         ENDPT=PP2
         IF (PP2.GT.PP1) THEN
            SIGN=1.0D0
         ELSE
            SIGN=-1.0D0
         ENDIF
      ELSE
         IF (ABS(CATOL).GT.ABS(0.01D0*C2)) THEN
            OPEN(69,FILE='../temp_files/detect.warning1'
     &,STATUS='NEW')
            WRITE(69,'(/,
     &''WARNING: THE VALUE OF speedatol IS RATHER LARGE'',/,
     &''         COMPARED TO THE WAVE SPEED VALUE'',/,
     &''         AT THE END OF THE SEARCH RANGE.'',/,
     &''         THIS MAY BE APPROPRIATE, BUT IT MIGHT'',/,
     &''         ALSO INDICATE THAT speedatol SHOULD'',/,
     &''         BE REDUCED IN defaults.input''
     &,/)')
            CLOSE(69)
         ENDIF
         TOL=CATOL+ABS(CRTOL*C2)
         ENDPT=C2
         IF (C2.GT.C1) THEN
            SIGN=1.0D0
         ELSE
            SIGN=-1.0D0
         ENDIF
      ENDIF
      IF ((SIGN*PPC).GT.((SIGN*ENDPT)-TOL)) THEN
         IDTECT=12
      ELSE
         IDTECT=11
      ENDIF
 102  CLOSE(12)
      CALL DWARN(IPT)
      WRITE(6,9000)ADTECT
C     screen output not "message": feeds into a pipe
      IF (ADTECT.EQ.'n') THEN
         OPEN(14,FILE='../temp_files/detect_hopf.output'
     &,STATUS='NEW')
         WRITE(14,9002)IDTECT
         CLOSE(14)
      ENDIF
      CALL ISTOP(INFO)
 9000 FORMAT(A1)
 9001 FORMAT(E18.10E2)
 9002 FORMAT(I2)
      END

      SUBROUTINE DWARN(IPT)
      IMPLICIT REAL*8 (A-H,K,O-Z)
      LOGICAL DOWARN
      CHARACTER*5 WORD1
      CHARACTER*28 WORD2
      CHARACTER*8 WORD3
      CHARACTER*25 ATYPE
C-----
C      ATYPE='HOPF BIFURCATION POINT.  '
C      ATYPE='PERIODIC TRAVELLING WAVE.'
C      ATYPE='FOLD POINT.              '
C      ATYPE='ECKHAUS POINT.           '
      ATYPE='HOPF POINT.              '
C      ATYPE='SOLN OF THE RIGHT PERIOD.'
C-----
      OPEN(23,FILE='../temp_files/defaults.input'
     &,STATUS='OLD')
      REWIND(23)
      DO 230 I230=1,11
         READ(23,*)
 230  CONTINUE
      READ(23,*)NWARN1,NWARN2
      CLOSE(23)
      IPT=ABS(IPT)
      DOWARN=.FALSE.
      IF (IPT.LT.NWARN1) THEN
         DOWARN=.TRUE.
         WORD1='SMALL'
         WORD2='JUMPING OVER BIFURCATION PTS'
         WORD3='INCREASE'
      ENDIF
      IF (IPT.GT.NWARN2) THEN
         DOWARN=.TRUE.
         WORD1='LARGE'
         WORD2='UNNECESSARILY LONG RUN TIMES'
         WORD3='DECREASE'
      ENDIF
      IF (DOWARN) THEN
         OPEN(66,FILE='../temp_files/detect.warning'
     &,STATUS='NEW')
         WRITE(66,9000)WORD1,IPT,ATYPE,WORD2,WORD3
         CLOSE(66)
      ENDIF
      RETURN
 9000 FORMAT(/,'WARNING: A RATHER ',A5,' NUMBER OF',
     &     ' CONTINUATION STEPS (=',I5,')',/,
     &     'WAS TAKEN LOOKING FOR THE ',
     &     A25,/,'TO AVOID ',A28,' IT MIGHT BE',
     &     ' HELPFUL TO',/,A8,' THE RELEVANT',
     &     ' ds, dsmin AND dsmax',
     &     ' IN constants.input.',/)
      END
