#+ spectrum.gp
# This file was written by Jonathan A. Sherratt
# Copyright (C) 2011 Heriot-Watt University, Edinburgh, UK
#  
# THIS FILE IS PART OF WAVETRAIN.
# WAVETRAIN SOFTWARE IS INTENDED FOR FREE PERSONAL PRIVATE 
# AND ACADEMIC USE ONLY AND WAS DEVELOPED BY JONATHAN SHERRATT AT 
# HERIOT-WATT UNIVERSITY, EDINBURGH, UK, AND IS PROTECTED BY COPYRIGHT. 
# THE COPYRIGHT OWNER IS HERIOT-WATT UNIVERSITY, WITH THE EXCEPTION OF 
# THE ROUTINES FROM THE PACKAGES AUTO, BLAS, EISPACK, LAPACK AND MINPACK, 
# WHICH ARE INCLUDED WITH PERMISSION, WHICH ARE CLEARLY IDENTIFIED IN 
# THE SOURCE CODE, AND THE USE OF THESE PACKAGES IS UNDER THE CONDITIONS 
# OF THE ORIGINAL OWNERS WHICH ARE AVAILABLE AT 
# http://www.netlib.org/blas/faq.html#2;
# http://www.netlib.org/lapack/#_licensing; 
# http://www.netlib.org/minpack/disclaimer.
# 
# PERSONS OR ORGANISATIONS WISHING TO USE THIS WAVETRAIN SOFTWARE FOR 
# COMMERCIAL PURPOSES OR PURPOSES OTHER THAN ITS ORIGINAL INTENTION 
# SHOULD CONTACT THE OWNERS TO REQUEST SPECIFIC WRITTEN PERMISSIONS.
# 
# DISTRIBUTION: DISTRIBUTION OF WAVETRAIN SOFTWARE, IN PART OR AS 
# A WHOLE, BY ANY MEANS (ELECTRONIC, PRINTED COPY OR OTHERWISE), IS 
# PROHIBITED WITHOUT THE PRIOR WRITTEN PERMISSION OF EITHER HERIOT-WATT 
# UNIVERSITY OR THE AUTHOR JONATHAN A. SHERRATT. ALL REQUESTS FOR SUCH 
# PERMISSION SHOULD BE SENT BY EMAIL TO J.A.SHERRATT@HW.AC.UK; PLEASE 
# INCLUDE THE WORD WAVETRAIN IN THE SUBJECT HEADING OF THE MESSAGE.  
# 
# WARRANTY LIMITATIONS: THE PRODUCT AND SERVICE ARE PROVIDED "AS IS". 
# THE OWNER DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING BUT NOT LIMITED 
# TO, ALL EXPRESSED OR IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE. UNDER NO CIRCUMSTANCES WILL THE OWNER BE 
# LIABLE TO YOU FOR ANY LOSS OF USE, INTERRUPTION OF BUSINESS OR ANY 
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, AND/OR CONSEQUENTIAL DAMAGES 
# OF ANY KIND (INCLUDING LOSS OF PROFITS) EVEN IF THE OWNER HAS BEEN 
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. THE OWNER ACCEPTS NO LEGAL 
# LIABILITY FOR ANY OF THE PRODUCT AND/OR SERVICE SOLD, OFFERED FOR SALE, 
# OR OTHERWISE MADE AVAILABLE FOR PUBLIC OR PRIVATE CONSUMPTION IN 
# RELATION TO THIS SOFTWARE. THE OWNER DOES NOT WARRANT THAT THIS SOFTWARE 
# WILL BE ERROR-FREE.
#  
#+ THIS gnuplot COMMAND FILE PLOTS THE SPECTRUM.
#+ THERE IS AN OPTION TO USE EITHER A SINGLE COLOUR, OR A VARIABLE COLOUR
#+ (USING EITHER RAINBOW OR GREYSCALE COLOUR MAP) BASED ON THE VALUE
#+ OF THE PHASE DIFFERENCE ACROSS ONE PERIOD OF THE WAVE. THIS IS
#+ CHOSEN VIA THE FLAG spectrumcolour, WHICH IS SET IN THE STYLE FILE.
#+ IF A VARIABLE COLOUR IS SET, THEN A SCALEBAR INDICATING
#+ THE INTERPRETATION OF THE COLOURS IS INCLUDED IN THE PLOT.
#+ NOTE THAT WITH A FIXED LINE COLOUR, THE PLOT IS A STANDARD 2D PLOT,
#+ BUT IN THE VARIABLE COLOUR CASES THE PLOT IS REALLY A 3D PLOT,
#+ SEEN "END ON"; ROTATION VIA THE MOUSE IS DISABLE BECAUSE IN THESE
#+ VARIABLE COLOUR CASES THE PLOT IS GENERATED IN THE multiplot ENVIRONMENT.
#++
if ($#==0) print "ERROR: no rcode number was given" ; exit
if ($#>1) print "ERROR: too many arguments: only an rcode number is required" ; exit
unset multiplot ; mpon="no"
unset label
unset xzeroaxis
unset yzeroaxis
system "../plotprepare_spectrum.script ".subdir." ".pdir." $0 ".restrictphasedifference
if (system("cat ../temp_files/plotprepare.error") eq "1") exit
outputdir=pdir."/rcode$0/"
title1=titlesexpand.system("getline.script 1 ../temp_files/plotprepare_spectrum.output").textend
if (system("cat ../temp_files/getline.error") eq "1") exit
title2=titlesexpand.system("getline.script 2 ../temp_files/plotprepare_spectrum.output").textend
if (system("cat ../temp_files/getline.error") eq "1") exit
vrmaxlabeltext=foldtextspexpand.system("getline.script 3 ../temp_files/plotprepare_spectrum.output").textend
if (system("cat ../temp_files/getline.error") eq "1") exit
vrmax=system("getline.script 4 ../temp_files/plotprepare_spectrum.output")
if (system("cat ../temp_files/getline.error") eq "1") exit
vimax=system("getline.script 5 ../temp_files/plotprepare_spectrum.output")
if (system("cat ../temp_files/getline.error") eq "1") exit
xmin=system("getline.script 1 ../temp_files/spectrum_axes_limits.data")
if (system("cat ../temp_files/getline.error") eq "1") exit
xmax=system("getline.script 2 ../temp_files/spectrum_axes_limits.data")
if (system("cat ../temp_files/getline.error") eq "1") exit
ymin=system("getline.script 3 ../temp_files/spectrum_axes_limits.data")
if (system("cat ../temp_files/getline.error") eq "1") exit
ymax=system("getline.script 4 ../temp_files/spectrum_axes_limits.data")
if (system("cat ../temp_files/getline.error") eq "1") exit
set xrange [xmin:xmax] noreverse writeback 
set yrange [ymin:ymax] noreverse writeback
if (showtitles eq "yes") set label title1 at screen xtitle,(1.0-topspace*0.25) center textcolor rgb "black" \
 offset character 0,(titlesfactor-1.0)*0.5
if (showtitles eq "yes") set label title2 at screen xtitle,(1.0-topspace*0.75) center textcolor rgb "black" \
 offset character 0,(titlesfactor-1.0)*0.5
# Now read in data on eigenvalues from the gamma=0 calculation
system "selectdata.script data_files/evalues.data data_files/evg0.dat1 \
 '$$c1 $$c2' '$$c3 -eq 0' "
if (system("cat ../temp_files/selectdata.error") eq "1") exit
system "selectdata.script data_files/evalues.data data_files/evg0.dat2 \
 '$$c1 $$c2' '$$c3 -gt 0'"
if (system("cat ../temp_files/selectdata.error") eq "1") exit
empty1=system("emptytest.script data_files/evg0.dat1")
if (system("cat ../temp_files/emptytest.error") eq "1") exit
empty2=system("emptytest.script data_files/evg0.dat2")
if (system("cat ../temp_files/emptytest.error") eq "1") exit
# Now process spectrum data
system "spectrum_truncate_gamma.script ".xmin." ".xmax." ".ymin." ".ymax
if (system("cat ../temp_files/spectrum_truncate_gamma.error") eq "1") exit
xlabeltext=labelsexpand."Re(eigenvalue)".textend
ylabeltext=labelsexpand."Im(eigenvalue)".textend
if (spectrumcolour eq "rainbow" || spectrumcolour eq "greyscale" || spectrumcolour eq "grayscale" ) \
 call "spectrum_multicolour.gp" ; \
else \
 call "spectrum_fixedcolour.gp"
# Note: don't unset anything: leave all settings as
#       they are so that loci etc can be added via replot.
#       Also no need to use restore on xrange and yrange
#       since these are set explicitly.
#
iline=0 # This initialises the counter
# End of command file.       
