C+    eispack.f
C+    THIS IS THE PROGRAM eispack.f FROM AUTO97
C+    IT CONTAINS THE EISPACK ROUTINES NEEDED BY AUTO.
C+    EISPACK IS A FREE OPEN-SOURCE SOFTWARE PACKAGE FOR THE
C+    NUMERICAL CALCULATION OF THE EIGENVALUES AND EIGENVECTORS 
C+    OF MATRICES. IT HAS LARGELY BEEN SUPERSEDED BY LAPACK, BUT
C+    IS USED BY AUTO97. THIS SUBSET OF EISPACK ROUTINES IS 
C+    DISTRIBUTED AS PART OF AUTO97 AND HAS THEREFORE BEEN
C+    INHERITED BY WAVETRAIN.
C++
C     This code was changed to be entirely upper case by
C     Jonathan Sherratt on 24/09/10. Additionally, the routine
C     DNRM2 was altered by Jonathan Sherratt on 01/07/11 to
C     avoid the use of the assign statement and corresponding
C     goto statements with lists, since these are not recognised by some
C     fortran95 compilers that can otherwise cope with fortran77 code.
C     The lines affected by this latter change are indicated in the code
C     by CJAS appearing at the start of the line. Similarly the routine
C     This does not affect HQR2 was altered on 01/07/11 to remove the
C     use of an arithmetic if statement (also indicated by CJAS at the
C     start of the line). These changes do not affect the
C     interface, functionality, or accuracy of the software, 
C     and therefore routine names have not been changed.
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C          Eigenvalue solver from EISPACK
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE RG(NM,N,A,WR,WI,MATZ,Z,IV1,FV1,IERR)
C
      INTEGER N,NM,IS1,IS2,IERR,MATZ
      DOUBLE PRECISION A(NM,N),WR(N),WI(N),Z(NM,N),FV1(N)
      INTEGER IV1(N)
C
C     THIS SUBROUTINE CALLS THE RECOMMENDED SEQUENCE OF
C     SUBROUTINES FROM THE EIGENSYSTEM SUBROUTINE PACKAGE (EISPACK)
C     TO FIND THE EIGENVALUES AND EIGENVECTORS (IF DESIRED)
C     OF A REAL GENERAL MATRIX.
C
C     ON INPUT
C
C        NM  MUST BE SET TO THE ROW DIMENSION OF THE TWO-DIMENSIONAL
C        ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C        DIMENSION STATEMENT.
C
C        N  IS THE ORDER OF THE MATRIX  A.
C
C        A  CONTAINS THE REAL GENERAL MATRIX.
C
C        MATZ  IS AN INTEGER VARIABLE SET EQUAL TO ZERO IF
C        ONLY EIGENVALUES ARE DESIRED.  OTHERWISE IT IS SET TO
C        ANY NON-ZERO INTEGER FOR BOTH EIGENVALUES AND EIGENVECTORS.
C
C     ON OUTPUT
C
C        WR  AND  WI  CONTAIN THE REAL AND IMAGINARY PARTS,
C        RESPECTIVELY, OF THE EIGENVALUES.  COMPLEX CONJUGATE
C        PAIRS OF EIGENVALUES APPEAR CONSECUTIVELY WITH THE
C        EIGENVALUE HAVING THE POSITIVE IMAGINARY PART FIRST.
C
C        Z  CONTAINS THE REAL AND IMAGINARY PARTS OF THE EIGENVECTORS
C        IF MATZ IS NOT ZERO.  IF THE J-TH EIGENVALUE IS REAL, THE
C        J-TH COLUMN OF  Z  CONTAINS ITS EIGENVECTOR.  IF THE J-TH
C        EIGENVALUE IS COMPLEX WITH POSITIVE IMAGINARY PART, THE
C        J-TH AND (J+1)-TH COLUMNS OF  Z  CONTAIN THE REAL AND
C        IMAGINARY PARTS OF ITS EIGENVECTOR.  THE CONJUGATE OF THIS
C        VECTOR IS THE EIGENVECTOR FOR THE CONJUGATE EIGENVALUE.
C
C        IERR  IS AN INTEGER OUTPUT VARIABLE SET EQUAL TO AN ERROR
C           COMPLETION CODE DESCRIBED IN THE DOCUMENTATION FOR HQR
C           AND HQR2.  THE NORMAL COMPLETION CODE IS ZERO.
C
C        IV1  AND  FV1  ARE TEMPORARY STORAGE ARRAYS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IF (N .LE. NM) GO TO 10
      IERR = 10 * N
      GO TO 50
C
   10 CALL  BALANC(NM,N,A,IS1,IS2,FV1)
      CALL  ELMHES(NM,N,IS1,IS2,A,IV1)
      IF (MATZ .NE. 0) GO TO 20
C     .......... FIND EIGENVALUES ONLY ..........
      CALL  HQR(NM,N,IS1,IS2,A,WR,WI,IERR)
      GO TO 50
C     .......... FIND BOTH EIGENVALUES AND EIGENVECTORS ..........
   20 CALL  ELTRAN(NM,N,IS1,IS2,A,IV1,Z)
      CALL  HQR2(NM,N,IS1,IS2,A,WR,WI,Z,IERR)
      IF (IERR .NE. 0) GO TO 50
      CALL  BALBAK(NM,N,IS1,IS2,FV1,N,Z)
   50 RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE HQR(NM,N,LOW,IGH,H,WR,WI,IERR)
C
      INTEGER I,J,K,L,M,N,EN,LL,MM,NA,NM,IGH,ITN,ITS,LOW,MP2,ENM2,IERR
      DOUBLE PRECISION H(NM,N),WR(N),WI(N)
      DOUBLE PRECISION P,Q,R,S,T,W,X,Y,ZZ,NORM,TST1,TST2
      LOGICAL NOTLAS
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE HQR,
C     NUM. MATH. 14, 219-231(1970) BY MARTIN, PETERS, AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 359-371(1971).
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES OF A REAL
C     UPPER HESSENBERG MATRIX BY THE QR METHOD.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N.
C
C        H CONTAINS THE UPPER HESSENBERG MATRIX.  INFORMATION ABOUT
C          THE TRANSFORMATIONS USED IN THE REDUCTION TO HESSENBERG
C          FORM BY  ELMHES  OR  ORTHES, IF PERFORMED, IS STORED
C          IN THE REMAINING TRIANGLE UNDER THE HESSENBERG MATRIX.
C
C     ON OUTPUT
C
C        H HAS BEEN DESTROYED.  THEREFORE, IT MUST BE SAVED
C          BEFORE CALLING  HQR  IF SUBSEQUENT CALCULATION AND
C          BACK TRANSFORMATION OF EIGENVECTORS IS TO BE PERFORMED.
C
C        WR AND WI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVALUES.  THE EIGENVALUES
C          ARE UNORDERED EXCEPT THAT COMPLEX CONJUGATE PAIRS
C          OF VALUES APPEAR CONSECUTIVELY WITH THE EIGENVALUE
C          HAVING THE POSITIVE IMAGINARY PART FIRST.  IF AN
C          ERROR EXIT IS MADE, THE EIGENVALUES SHOULD BE CORRECT
C          FOR INDICES IERR+1,...,N.
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE LIMIT OF 30*N ITERATIONS IS EXHAUSTED
C                     WHILE THE J-TH EIGENVALUE IS BEING SOUGHT.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IERR = 0
      NORM = 0.0D0
      K = 1
C     .......... STORE ROOTS ISOLATED BY BALANC
C                AND COMPUTE MATRIX NORM ..........
      DO 50 I = 1, N
C
         DO 40 J = K, N
   40    NORM = NORM + DABS(H(I,J))
C
         K = I
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 50
         WR(I) = H(I,I)
         WI(I) = 0.0D0
   50 CONTINUE
C
      EN = IGH
      T = 0.0D0
      ITN = 30*N
C     .......... SEARCH FOR NEXT EIGENVALUES ..........
   60 IF (EN .LT. LOW) GO TO 1001
      ITS = 0
      NA = EN - 1
      ENM2 = NA - 1
C     .......... LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT
C                FOR L=EN STEP -1 UNTIL LOW DO -- ..........
   70 DO 80 LL = LOW, EN
         L = EN + LOW - LL
         IF (L .EQ. LOW) GO TO 100
         S = DABS(H(L-1,L-1)) + DABS(H(L,L))
         IF (S .EQ. 0.0D0) S = NORM
         TST1 = S
         TST2 = TST1 + DABS(H(L,L-1))
         IF (TST2 .EQ. TST1) GO TO 100
   80 CONTINUE
C     .......... FORM SHIFT ..........
  100 X = H(EN,EN)
      IF (L .EQ. EN) GO TO 270
      Y = H(NA,NA)
      W = H(EN,NA) * H(NA,EN)
      IF (L .EQ. NA) GO TO 280
      IF (ITN .EQ. 0) GO TO 1000
      IF (ITS .NE. 10 .AND. ITS .NE. 20) GO TO 130
C     .......... FORM EXCEPTIONAL SHIFT ..........
      T = T + X
C
      DO 120 I = LOW, EN
  120 H(I,I) = H(I,I) - X
C
      S = DABS(H(EN,NA)) + DABS(H(NA,ENM2))
      X = 0.75D0 * S
      Y = X
      W = -0.4375D0 * S * S
  130 ITS = ITS + 1
      ITN = ITN - 1
C     .......... LOOK FOR TWO CONSECUTIVE SMALL
C                SUB-DIAGONAL ELEMENTS.
C                FOR M=EN-2 STEP -1 UNTIL L DO -- ..........
      DO 140 MM = L, ENM2
         M = ENM2 + L - MM
         ZZ = H(M,M)
         R = X - ZZ
         S = Y - ZZ
         P = (R * S - W) / H(M+1,M) + H(M,M+1)
         Q = H(M+1,M+1) - ZZ - R - S
         R = H(M+2,M+1)
         S = DABS(P) + DABS(Q) + DABS(R)
         P = P / S
         Q = Q / S
         R = R / S
         IF (M .EQ. L) GO TO 150
         TST1 = DABS(P)*(DABS(H(M-1,M-1)) + DABS(ZZ) + DABS(H(M+1,M+1)))
         TST2 = TST1 + DABS(H(M,M-1))*(DABS(Q) + DABS(R))
         IF (TST2 .EQ. TST1) GO TO 150
  140 CONTINUE
C
  150 MP2 = M + 2
C
      DO 160 I = MP2, EN
         H(I,I-2) = 0.0D0
         IF (I .EQ. MP2) GO TO 160
         H(I,I-3) = 0.0D0
  160 CONTINUE
C     .......... DOUBLE QR STEP INVOLVING ROWS L TO EN AND
C                COLUMNS M TO EN ..........
      DO 260 K = M, NA
         NOTLAS = K .NE. NA
         IF (K .EQ. M) GO TO 170
         P = H(K,K-1)
         Q = H(K+1,K-1)
         R = 0.0D0
         IF (NOTLAS) R = H(K+2,K-1)
         X = DABS(P) + DABS(Q) + DABS(R)
         IF (X .EQ. 0.0D0) GO TO 260
         P = P / X
         Q = Q / X
         R = R / X
  170    S = DSIGN(DSQRT(P*P+Q*Q+R*R),P)
         IF (K .EQ. M) GO TO 180
         H(K,K-1) = -S * X
         GO TO 190
  180    IF (L .NE. M) H(K,K-1) = -H(K,K-1)
  190    P = P + S
         X = P / S
         Y = Q / S
         ZZ = R / S
         Q = Q / P
         R = R / P
         IF (NOTLAS) GO TO 225
C     .......... ROW MODIFICATION ..........
         DO 200 J = K, N
            P = H(K,J) + Q * H(K+1,J)
            H(K,J) = H(K,J) - P * X
            H(K+1,J) = H(K+1,J) - P * Y
  200    CONTINUE
C
         J = MIN0(EN,K+3)
C     .......... COLUMN MODIFICATION ..........
         DO 210 I = 1, J
            P = X * H(I,K) + Y * H(I,K+1)
            H(I,K) = H(I,K) - P
            H(I,K+1) = H(I,K+1) - P * Q
  210    CONTINUE
         GO TO 255
  225    CONTINUE
C     .......... ROW MODIFICATION ..........
         DO 230 J = K, N
            P = H(K,J) + Q * H(K+1,J) + R * H(K+2,J)
            H(K,J) = H(K,J) - P * X
            H(K+1,J) = H(K+1,J) - P * Y
            H(K+2,J) = H(K+2,J) - P * ZZ
  230    CONTINUE
C
         J = MIN0(EN,K+3)
C     .......... COLUMN MODIFICATION ..........
         DO 240 I = 1, J
            P = X * H(I,K) + Y * H(I,K+1) + ZZ * H(I,K+2)
            H(I,K) = H(I,K) - P
            H(I,K+1) = H(I,K+1) - P * Q
            H(I,K+2) = H(I,K+2) - P * R
  240    CONTINUE
  255    CONTINUE
C
  260 CONTINUE
C
      GO TO 70
C     .......... ONE ROOT FOUND ..........
  270 WR(EN) = X + T
      WI(EN) = 0.0D0
      EN = NA
      GO TO 60
C     .......... TWO ROOTS FOUND ..........
  280 P = (Y - X) / 2.0D0
      Q = P * P + W
      ZZ = DSQRT(DABS(Q))
      X = X + T
      IF (Q .LT. 0.0D0) GO TO 320
C     .......... REAL PAIR ..........
      ZZ = P + DSIGN(ZZ,P)
      WR(NA) = X + ZZ
      WR(EN) = WR(NA)
      IF (ZZ .NE. 0.0D0) WR(EN) = X - W / ZZ
      WI(NA) = 0.0D0
      WI(EN) = 0.0D0
      GO TO 330
C     .......... COMPLEX PAIR ..........
  320 WR(NA) = X + P
      WR(EN) = X + P
      WI(NA) = ZZ
      WI(EN) = -ZZ
  330 EN = ENM2
      GO TO 60
C     .......... SET ERROR -- ALL EIGENVALUES HAVE NOT
C                CONVERGED AFTER 30*N ITERATIONS ..........
 1000 IERR = EN
 1001 RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE HQR2(NM,N,LOW,IGH,H,WR,WI,Z,IERR)
C     THIS SUBROUTINE HAS BEEN EDITTED BY JONATHAN SHERRATT 01/07/11
C     TO REMOVE THE ARITHMETIC IF STATEMENT. THE RELEVANT LINES ARE
C     INDICATED BY COMMENT LINES BEGINNING CJAS
C
      INTEGER I,J,K,L,M,N,EN,II,JJ,LL,MM,NA,NM,NN,
     X        IGH,ITN,ITS,LOW,MP2,ENM2,IERR
      DOUBLE PRECISION H(NM,N),WR(N),WI(N),Z(NM,N)
      DOUBLE PRECISION P,Q,R,S,T,W,X,Y,RA,SA,VI,VR,ZZ,NORM,TST1,TST2
      LOGICAL NOTLAS
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE HQR2,
C     NUM. MATH. 16, 181-204(1970) BY PETERS AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971).
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS
C     OF A REAL UPPER HESSENBERG MATRIX BY THE QR METHOD.  THE
C     EIGENVECTORS OF A REAL GENERAL MATRIX CAN ALSO BE FOUND
C     IF  ELMHES  AND  ELTRAN  OR  ORTHES  AND  ORTRAN  HAVE
C     BEEN USED TO REDUCE THIS GENERAL MATRIX TO HESSENBERG FORM
C     AND TO ACCUMULATE THE SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N.
C
C        H CONTAINS THE UPPER HESSENBERG MATRIX.
C
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED BY  ELTRAN
C          AFTER THE REDUCTION BY  ELMHES, OR BY  ORTRAN  AFTER THE
C          REDUCTION BY  ORTHES, IF PERFORMED.  IF THE EIGENVECTORS
C          OF THE HESSENBERG MATRIX ARE DESIRED, Z MUST CONTAIN THE
C          IDENTITY MATRIX.
C
C     ON OUTPUT
C
C        H HAS BEEN DESTROYED.
C
C        WR AND WI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVALUES.  THE EIGENVALUES
C          ARE UNORDERED EXCEPT THAT COMPLEX CONJUGATE PAIRS
C          OF VALUES APPEAR CONSECUTIVELY WITH THE EIGENVALUE
C          HAVING THE POSITIVE IMAGINARY PART FIRST.  IF AN
C          ERROR EXIT IS MADE, THE EIGENVALUES SHOULD BE CORRECT
C          FOR INDICES IERR+1,...,N.
C
C        Z CONTAINS THE REAL AND IMAGINARY PARTS OF THE EIGENVECTORS.
C          IF THE I-TH EIGENVALUE IS REAL, THE I-TH COLUMN OF Z
C          CONTAINS ITS EIGENVECTOR.  IF THE I-TH EIGENVALUE IS COMPLEX
C          WITH POSITIVE IMAGINARY PART, THE I-TH AND (I+1)-TH
C          COLUMNS OF Z CONTAIN THE REAL AND IMAGINARY PARTS OF ITS
C          EIGENVECTOR.  THE EIGENVECTORS ARE UNNORMALIZED.  IF AN
C          ERROR EXIT IS MADE, NONE OF THE EIGENVECTORS HAS BEEN FOUND.
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE LIMIT OF 30*N ITERATIONS IS EXHAUSTED
C                     WHILE THE J-TH EIGENVALUE IS BEING SOUGHT.
C
C     CALLS CDIV FOR COMPLEX DIVISION.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IERR = 0
      NORM = 0.0D0
      K = 1
C     .......... STORE ROOTS ISOLATED BY BALANC
C                AND COMPUTE MATRIX NORM ..........
      DO 50 I = 1, N
C
         DO 40 J = K, N
   40    NORM = NORM + DABS(H(I,J))
C
         K = I
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 50
         WR(I) = H(I,I)
         WI(I) = 0.0D0
   50 CONTINUE
C
      EN = IGH
      T = 0.0D0
      ITN = 30*N
C     .......... SEARCH FOR NEXT EIGENVALUES ..........
   60 IF (EN .LT. LOW) GO TO 340
      ITS = 0
      NA = EN - 1
      ENM2 = NA - 1
C     .......... LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT
C                FOR L=EN STEP -1 UNTIL LOW DO -- ..........
   70 DO 80 LL = LOW, EN
         L = EN + LOW - LL
         IF (L .EQ. LOW) GO TO 100
         S = DABS(H(L-1,L-1)) + DABS(H(L,L))
         IF (S .EQ. 0.0D0) S = NORM
         TST1 = S
         TST2 = TST1 + DABS(H(L,L-1))
         IF (TST2 .EQ. TST1) GO TO 100
   80 CONTINUE
C     .......... FORM SHIFT ..........
  100 X = H(EN,EN)
      IF (L .EQ. EN) GO TO 270
      Y = H(NA,NA)
      W = H(EN,NA) * H(NA,EN)
      IF (L .EQ. NA) GO TO 280
      IF (ITN .EQ. 0) GO TO 1000
      IF (ITS .NE. 10 .AND. ITS .NE. 20) GO TO 130
C     .......... FORM EXCEPTIONAL SHIFT ..........
      T = T + X
C
      DO 120 I = LOW, EN
  120 H(I,I) = H(I,I) - X
C
      S = DABS(H(EN,NA)) + DABS(H(NA,ENM2))
      X = 0.75D0 * S
      Y = X
      W = -0.4375D0 * S * S
  130 ITS = ITS + 1
      ITN = ITN - 1
C     .......... LOOK FOR TWO CONSECUTIVE SMALL
C                SUB-DIAGONAL ELEMENTS.
C                FOR M=EN-2 STEP -1 UNTIL L DO -- ..........
      DO 140 MM = L, ENM2
         M = ENM2 + L - MM
         ZZ = H(M,M)
         R = X - ZZ
         S = Y - ZZ
         P = (R * S - W) / H(M+1,M) + H(M,M+1)
         Q = H(M+1,M+1) - ZZ - R - S
         R = H(M+2,M+1)
         S = DABS(P) + DABS(Q) + DABS(R)
         P = P / S
         Q = Q / S
         R = R / S
         IF (M .EQ. L) GO TO 150
         TST1 = DABS(P)*(DABS(H(M-1,M-1)) + DABS(ZZ) + DABS(H(M+1,M+1)))
         TST2 = TST1 + DABS(H(M,M-1))*(DABS(Q) + DABS(R))
         IF (TST2 .EQ. TST1) GO TO 150
  140 CONTINUE
C
  150 MP2 = M + 2
C
      DO 160 I = MP2, EN
         H(I,I-2) = 0.0D0
         IF (I .EQ. MP2) GO TO 160
         H(I,I-3) = 0.0D0
  160 CONTINUE
C     .......... DOUBLE QR STEP INVOLVING ROWS L TO EN AND
C                COLUMNS M TO EN ..........
      DO 260 K = M, NA
         NOTLAS = K .NE. NA
         IF (K .EQ. M) GO TO 170
         P = H(K,K-1)
         Q = H(K+1,K-1)
         R = 0.0D0
         IF (NOTLAS) R = H(K+2,K-1)
         X = DABS(P) + DABS(Q) + DABS(R)
         IF (X .EQ. 0.0D0) GO TO 260
         P = P / X
         Q = Q / X
         R = R / X
  170    S = DSIGN(DSQRT(P*P+Q*Q+R*R),P)
         IF (K .EQ. M) GO TO 180
         H(K,K-1) = -S * X
         GO TO 190
  180    IF (L .NE. M) H(K,K-1) = -H(K,K-1)
  190    P = P + S
         X = P / S
         Y = Q / S
         ZZ = R / S
         Q = Q / P
         R = R / P
         IF (NOTLAS) GO TO 225
C     .......... ROW MODIFICATION ..........
         DO 200 J = K, N
            P = H(K,J) + Q * H(K+1,J)
            H(K,J) = H(K,J) - P * X
            H(K+1,J) = H(K+1,J) - P * Y
  200    CONTINUE
C
         J = MIN0(EN,K+3)
C     .......... COLUMN MODIFICATION ..........
         DO 210 I = 1, J
            P = X * H(I,K) + Y * H(I,K+1)
            H(I,K) = H(I,K) - P
            H(I,K+1) = H(I,K+1) - P * Q
  210    CONTINUE
C     .......... ACCUMULATE TRANSFORMATIONS ..........
         DO 220 I = LOW, IGH
            P = X * Z(I,K) + Y * Z(I,K+1)
            Z(I,K) = Z(I,K) - P
            Z(I,K+1) = Z(I,K+1) - P * Q
  220    CONTINUE
         GO TO 255
  225    CONTINUE
C     .......... ROW MODIFICATION ..........
         DO 230 J = K, N
            P = H(K,J) + Q * H(K+1,J) + R * H(K+2,J)
            H(K,J) = H(K,J) - P * X
            H(K+1,J) = H(K+1,J) - P * Y
            H(K+2,J) = H(K+2,J) - P * ZZ
  230    CONTINUE
C
         J = MIN0(EN,K+3)
C     .......... COLUMN MODIFICATION ..........
         DO 240 I = 1, J
            P = X * H(I,K) + Y * H(I,K+1) + ZZ * H(I,K+2)
            H(I,K) = H(I,K) - P
            H(I,K+1) = H(I,K+1) - P * Q
            H(I,K+2) = H(I,K+2) - P * R
  240    CONTINUE
C     .......... ACCUMULATE TRANSFORMATIONS ..........
         DO 250 I = LOW, IGH
            P = X * Z(I,K) + Y * Z(I,K+1) + ZZ * Z(I,K+2)
            Z(I,K) = Z(I,K) - P
            Z(I,K+1) = Z(I,K+1) - P * Q
            Z(I,K+2) = Z(I,K+2) - P * R
  250    CONTINUE
  255    CONTINUE
C
  260 CONTINUE
C
      GO TO 70
C     .......... ONE ROOT FOUND ..........
  270 H(EN,EN) = X + T
      WR(EN) = H(EN,EN)
      WI(EN) = 0.0D0
      EN = NA
      GO TO 60
C     .......... TWO ROOTS FOUND ..........
  280 P = (Y - X) / 2.0D0
      Q = P * P + W
      ZZ = DSQRT(DABS(Q))
      H(EN,EN) = X + T
      X = H(EN,EN)
      H(NA,NA) = Y + T
      IF (Q .LT. 0.0D0) GO TO 320
C     .......... REAL PAIR ..........
      ZZ = P + DSIGN(ZZ,P)
      WR(NA) = X + ZZ
      WR(EN) = WR(NA)
      IF (ZZ .NE. 0.0D0) WR(EN) = X - W / ZZ
      WI(NA) = 0.0D0
      WI(EN) = 0.0D0
      X = H(EN,NA)
      S = DABS(X) + DABS(ZZ)
      P = X / S
      Q = ZZ / S
      R = DSQRT(P*P+Q*Q)
      P = P / R
      Q = Q / R
C     .......... ROW MODIFICATION ..........
      DO 290 J = NA, N
         ZZ = H(NA,J)
         H(NA,J) = Q * ZZ + P * H(EN,J)
         H(EN,J) = Q * H(EN,J) - P * ZZ
  290 CONTINUE
C     .......... COLUMN MODIFICATION ..........
      DO 300 I = 1, EN
         ZZ = H(I,NA)
         H(I,NA) = Q * ZZ + P * H(I,EN)
         H(I,EN) = Q * H(I,EN) - P * ZZ
  300 CONTINUE
C     .......... ACCUMULATE TRANSFORMATIONS ..........
      DO 310 I = LOW, IGH
         ZZ = Z(I,NA)
         Z(I,NA) = Q * ZZ + P * Z(I,EN)
         Z(I,EN) = Q * Z(I,EN) - P * ZZ
  310 CONTINUE
C
      GO TO 330
C     .......... COMPLEX PAIR ..........
  320 WR(NA) = X + P
      WR(EN) = X + P
      WI(NA) = ZZ
      WI(EN) = -ZZ
  330 EN = ENM2
      GO TO 60
C     .......... ALL ROOTS FOUND.  BACKSUBSTITUTE TO FIND
C                VECTORS OF UPPER TRIANGULAR FORM ..........
  340 IF (NORM .EQ. 0.0D0) GO TO 1001
C     .......... FOR EN=N STEP -1 UNTIL 1 DO -- ..........
      DO 800 NN = 1, N
         EN = N + 1 - NN
         P = WR(EN)
         Q = WI(EN)
         NA = EN - 1
CJAS         IF (Q) 710, 600, 800
CJAS     The above line has been replaced by the following lines
         IF (Q.LT.0.0D0) THEN
            GOTO 710
         ENDIF
         IF (Q.EQ.0.0D0) THEN
            GOTO 600
         ENDIF
         IF (Q.GT.0.0D0) THEN
            GOTO 800
         ENDIF
CJAS     End of replacement lines
C     .......... REAL VECTOR ..........
  600    M = EN
         H(EN,EN) = 1.0D0
         IF (NA .EQ. 0) GO TO 800
C     .......... FOR I=EN-1 STEP -1 UNTIL 1 DO -- ..........
         DO 700 II = 1, NA
            I = EN - II
            W = H(I,I) - P
            R = 0.0D0
C
            DO 610 J = M, EN
  610       R = R + H(I,J) * H(J,EN)
C
            IF (WI(I) .GE. 0.0D0) GO TO 630
            ZZ = W
            S = R
            GO TO 700
  630       M = I
            IF (WI(I) .NE. 0.0D0) GO TO 640
            T = W
            IF (T .NE. 0.0D0) GO TO 635
               TST1 = NORM
               T = TST1
  632          T = 0.01D0 * T
               TST2 = NORM + T
               IF (TST2 .GT. TST1) GO TO 632
  635       H(I,EN) = -R / T
            GO TO 680
C     .......... SOLVE REAL EQUATIONS ..........
  640       X = H(I,I+1)
            Y = H(I+1,I)
            Q = (WR(I) - P) * (WR(I) - P) + WI(I) * WI(I)
            T = (X * S - ZZ * R) / Q
            H(I,EN) = T
            IF (DABS(X) .LE. DABS(ZZ)) GO TO 650
            H(I+1,EN) = (-R - W * T) / X
            GO TO 680
  650       H(I+1,EN) = (-S - Y * T) / ZZ
C
C     .......... OVERFLOW CONTROL ..........
  680       T = DABS(H(I,EN))
            IF (T .EQ. 0.0D0) GO TO 700
            TST1 = T
            TST2 = TST1 + 1.0D0/TST1
            IF (TST2 .GT. TST1) GO TO 700
            DO 690 J = I, EN
               H(J,EN) = H(J,EN)/T
  690       CONTINUE
C
  700    CONTINUE
C     .......... END REAL VECTOR ..........
         GO TO 800
C     .......... COMPLEX VECTOR ..........
  710    M = NA
C     .......... LAST VECTOR COMPONENT CHOSEN IMAGINARY SO THAT
C                EIGENVECTOR MATRIX IS TRIANGULAR ..........
         IF (DABS(H(EN,NA)) .LE. DABS(H(NA,EN))) GO TO 720
         H(NA,NA) = Q / H(EN,NA)
         H(NA,EN) = -(H(EN,EN) - P) / H(EN,NA)
         GO TO 730
  720    CALL CDIV(0.0D0,-H(NA,EN),H(NA,NA)-P,Q,H(NA,NA),H(NA,EN))
  730    H(EN,NA) = 0.0D0
         H(EN,EN) = 1.0D0
         ENM2 = NA - 1
         IF (ENM2 .EQ. 0) GO TO 800
C     .......... FOR I=EN-2 STEP -1 UNTIL 1 DO -- ..........
         DO 795 II = 1, ENM2
            I = NA - II
            W = H(I,I) - P
            RA = 0.0D0
            SA = 0.0D0
C
            DO 760 J = M, EN
               RA = RA + H(I,J) * H(J,NA)
               SA = SA + H(I,J) * H(J,EN)
  760       CONTINUE
C
            IF (WI(I) .GE. 0.0D0) GO TO 770
            ZZ = W
            R = RA
            S = SA
            GO TO 795
  770       M = I
            IF (WI(I) .NE. 0.0D0) GO TO 780
            CALL CDIV(-RA,-SA,W,Q,H(I,NA),H(I,EN))
            GO TO 790
C     .......... SOLVE COMPLEX EQUATIONS ..........
  780       X = H(I,I+1)
            Y = H(I+1,I)
            VR = (WR(I) - P) * (WR(I) - P) + WI(I) * WI(I) - Q * Q
            VI = (WR(I) - P) * 2.0D0 * Q
            IF (VR .NE. 0.0D0 .OR. VI .NE. 0.0D0) GO TO 784
               TST1 = NORM * (DABS(W) + DABS(Q) + DABS(X)
     X                      + DABS(Y) + DABS(ZZ))
               VR = TST1
  783          VR = 0.01D0 * VR
               TST2 = TST1 + VR
               IF (TST2 .GT. TST1) GO TO 783
  784       CALL CDIV(X*R-ZZ*RA+Q*SA,X*S-ZZ*SA-Q*RA,VR,VI,
     X                H(I,NA),H(I,EN))
            IF (DABS(X) .LE. DABS(ZZ) + DABS(Q)) GO TO 785
            H(I+1,NA) = (-RA - W * H(I,NA) + Q * H(I,EN)) / X
            H(I+1,EN) = (-SA - W * H(I,EN) - Q * H(I,NA)) / X
            GO TO 790
  785       CALL CDIV(-R-Y*H(I,NA),-S-Y*H(I,EN),ZZ,Q,
     X                H(I+1,NA),H(I+1,EN))
C
C     .......... OVERFLOW CONTROL ..........
  790       T = DMAX1(DABS(H(I,NA)), DABS(H(I,EN)))
            IF (T .EQ. 0.0D0) GO TO 795
            TST1 = T
            TST2 = TST1 + 1.0D0/TST1
            IF (TST2 .GT. TST1) GO TO 795
            DO 792 J = I, EN
               H(J,NA) = H(J,NA)/T
               H(J,EN) = H(J,EN)/T
  792       CONTINUE
C
  795    CONTINUE
C     .......... END COMPLEX VECTOR ..........
  800 CONTINUE
C     .......... END BACK SUBSTITUTION.
C                VECTORS OF ISOLATED ROOTS ..........
      DO 840 I = 1, N
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 840
C
         DO 820 J = I, N
  820    Z(I,J) = H(I,J)
C
  840 CONTINUE
C     .......... MULTIPLY BY TRANSFORMATION MATRIX TO GIVE
C                VECTORS OF ORIGINAL FULL MATRIX.
C                FOR J=N STEP -1 UNTIL LOW DO -- ..........
      DO 880 JJ = LOW, N
         J = N + LOW - JJ
         M = MIN0(J,IGH)
C
         DO 880 I = LOW, IGH
            ZZ = 0.0D0
C
            DO 860 K = LOW, M
  860       ZZ = ZZ + Z(I,K) * H(K,J)
C
            Z(I,J) = ZZ
  880 CONTINUE
C
      GO TO 1001
C     .......... SET ERROR -- ALL EIGENVALUES HAVE NOT
C                CONVERGED AFTER 30*N ITERATIONS ..........
 1000 IERR = EN
 1001 RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE CDIV(AR,AI,BR,BI,CR,CI)
      DOUBLE PRECISION AR,AI,BR,BI,CR,CI
C
C     COMPLEX DIVISION, (CR,CI) = (AR,AI)/(BR,BI)
C
      DOUBLE PRECISION S,ARS,AIS,BRS,BIS
      S = DABS(BR) + DABS(BI)
      ARS = AR/S
      AIS = AI/S
      BRS = BR/S
      BIS = BI/S
      S = BRS**2 + BIS**2
      CR = (ARS*BRS + AIS*BIS)/S
      CI = (AIS*BRS - ARS*BIS)/S
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE BALANC(NM,N,A,LOW,IGH,SCALE)
C
      INTEGER I,J,K,L,M,N,JJ,NM,IGH,LOW,IEXC
      DOUBLE PRECISION A(NM,N),SCALE(N)
      DOUBLE PRECISION C,F,G,R,S,B2,RADIX
      LOGICAL NOCONV
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE BALANCE,
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971).
C
C     THIS SUBROUTINE BALANCES A REAL MATRIX AND ISOLATES
C     EIGENVALUES WHENEVER POSSIBLE.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        A CONTAINS THE INPUT MATRIX TO BE BALANCED.
C
C     ON OUTPUT
C
C        A CONTAINS THE BALANCED MATRIX.
C
C        LOW AND IGH ARE TWO INTEGERS SUCH THAT A(I,J)
C          IS EQUAL TO ZERO IF
C           (1) I IS GREATER THAN J AND
C           (2) J=1,...,LOW-1 OR I=IGH+1,...,N.
C
C        SCALE CONTAINS INFORMATION DETERMINING THE
C           PERMUTATIONS AND SCALING FACTORS USED.
C
C     SUPPOSE THAT THE PRINCIPAL SUBMATRIX IN ROWS LOW THROUGH IGH
C     HAS BEEN BALANCED, THAT P(J) DENOTES THE INDEX INTERCHANGED
C     WITH J DURING THE PERMUTATION STEP, AND THAT THE ELEMENTS
C     OF THE DIAGONAL MATRIX USED ARE DENOTED BY D(I,J).  THEN
C        SCALE(J) = P(J),    FOR J = 1,...,LOW-1
C                 = D(J,J),      J = LOW,...,IGH
C                 = P(J)         J = IGH+1,...,N.
C     THE ORDER IN WHICH THE INTERCHANGES ARE MADE IS N TO IGH+1,
C     THEN 1 TO LOW-1.
C
C     NOTE THAT 1 IS RETURNED FOR IGH IF IGH IS ZERO FORMALLY.
C
C     THE ALGOL PROCEDURE EXC CONTAINED IN BALANCE APPEARS IN
C     BALANC  IN LINE.  (NOTE THAT THE ALGOL ROLES OF IDENTIFIERS
C     K,L HAVE BEEN REVERSED.)
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      RADIX = 16.0D0
C
      B2 = RADIX * RADIX
      K = 1
      L = N
      GO TO 100
C     .......... IN-LINE PROCEDURE FOR ROW AND
C                COLUMN EXCHANGE ..........
   20 SCALE(M) = J
      IF (J .EQ. M) GO TO 50
C
      DO 30 I = 1, L
         F = A(I,J)
         A(I,J) = A(I,M)
         A(I,M) = F
   30 CONTINUE
C
      DO 40 I = K, N
         F = A(J,I)
         A(J,I) = A(M,I)
         A(M,I) = F
   40 CONTINUE
C
   50 GO TO (80,130), IEXC
C     .......... SEARCH FOR ROWS ISOLATING AN EIGENVALUE
C                AND PUSH THEM DOWN ..........
   80 IF (L .EQ. 1) GO TO 280
      L = L - 1
C     .......... FOR J=L STEP -1 UNTIL 1 DO -- ..........
  100 DO 120 JJ = 1, L
         J = L + 1 - JJ
C
         DO 110 I = 1, L
            IF (I .EQ. J) GO TO 110
            IF (A(J,I) .NE. 0.0D0) GO TO 120
  110    CONTINUE
C
         M = L
         IEXC = 1
         GO TO 20
  120 CONTINUE
C
      GO TO 140
C     .......... SEARCH FOR COLUMNS ISOLATING AN EIGENVALUE
C                AND PUSH THEM LEFT ..........
  130 K = K + 1
C
  140 DO 170 J = K, L
C
         DO 150 I = K, L
            IF (I .EQ. J) GO TO 150
            IF (A(I,J) .NE. 0.0D0) GO TO 170
  150    CONTINUE
C
         M = K
         IEXC = 2
         GO TO 20
  170 CONTINUE
C     .......... NOW BALANCE THE SUBMATRIX IN ROWS K TO L ..........
      DO 180 I = K, L
  180 SCALE(I) = 1.0D0
C     .......... ITERATIVE LOOP FOR NORM REDUCTION ..........
  190 NOCONV = .FALSE.
C
      DO 270 I = K, L
         C = 0.0D0
         R = 0.0D0
C
         DO 200 J = K, L
            IF (J .EQ. I) GO TO 200
            C = C + DABS(A(J,I))
            R = R + DABS(A(I,J))
  200    CONTINUE
C     .......... GUARD AGAINST ZERO C OR R DUE TO UNDERFLOW ..........
         IF (C .EQ. 0.0D0 .OR. R .EQ. 0.0D0) GO TO 270
         G = R / RADIX
         F = 1.0D0
         S = C + R
  210    IF (C .GE. G) GO TO 220
         F = F * RADIX
         C = C * B2
         GO TO 210
  220    G = R * RADIX
  230    IF (C .LT. G) GO TO 240
         F = F / RADIX
         C = C / B2
         GO TO 230
C     .......... NOW BALANCE ..........
  240    IF ((C + R) / F .GE. 0.95D0 * S) GO TO 270
         G = 1.0D0 / F
         SCALE(I) = SCALE(I) * F
         NOCONV = .TRUE.
C
         DO 250 J = K, N
  250    A(I,J) = A(I,J) * G
C
         DO 260 J = 1, L
  260    A(J,I) = A(J,I) * F
C
  270 CONTINUE
C
      IF (NOCONV) GO TO 190
C
  280 LOW = K
      IGH = L
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE BALBAK(NM,N,LOW,IGH,SCALE,M,Z)
C
      INTEGER I,J,K,M,N,II,NM,IGH,LOW
      DOUBLE PRECISION SCALE(N),Z(NM,M)
      DOUBLE PRECISION S
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE BALBAK,
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971).
C
C     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A REAL GENERAL
C     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING
C     BALANCED MATRIX DETERMINED BY  BALANC.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY  BALANC.
C
C        SCALE CONTAINS INFORMATION DETERMINING THE PERMUTATIONS
C          AND SCALING FACTORS USED BY  BALANC.
C
C        M IS THE NUMBER OF COLUMNS OF Z TO BE BACK TRANSFORMED.
C
C        Z CONTAINS THE REAL AND IMAGINARY PARTS OF THE EIGEN-
C          VECTORS TO BE BACK TRANSFORMED IN ITS FIRST M COLUMNS.
C
C     ON OUTPUT
C
C        Z CONTAINS THE REAL AND IMAGINARY PARTS OF THE
C          TRANSFORMED EIGENVECTORS IN ITS FIRST M COLUMNS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IF (M .EQ. 0) GO TO 200
      IF (IGH .EQ. LOW) GO TO 120
C
      DO 110 I = LOW, IGH
         S = SCALE(I)
C     .......... LEFT HAND EIGENVECTORS ARE BACK TRANSFORMED
C                IF THE FOREGOING STATEMENT IS REPLACED BY
C                S=1.0D0/SCALE(I). ..........
         DO 100 J = 1, M
  100    Z(I,J) = Z(I,J) * S
C
  110 CONTINUE
C     ......... FOR I=LOW-1 STEP -1 UNTIL 1,
C               IGH+1 STEP 1 UNTIL N DO -- ..........
  120 DO 140 II = 1, N
         I = II
         IF (I .GE. LOW .AND. I .LE. IGH) GO TO 140
         IF (I .LT. LOW) I = LOW - II
         K = SCALE(I)
         IF (K .EQ. I) GO TO 140
C
         DO 130 J = 1, M
            S = Z(I,J)
            Z(I,J) = Z(K,J)
            Z(K,J) = S
  130    CONTINUE
C
  140 CONTINUE
C
  200 RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE ELMHES(NM,N,LOW,IGH,A,INT)
C
      INTEGER I,J,M,N,LA,NM,IGH,KP1,LOW,MM1,MP1
      DOUBLE PRECISION A(NM,N)
      DOUBLE PRECISION X,Y
      INTEGER INT(IGH)
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE ELMHES,
C     NUM. MATH. 12, 349-368(1968) BY MARTIN AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 339-358(1971).
C
C     GIVEN A REAL GENERAL MATRIX, THIS SUBROUTINE
C     REDUCES A SUBMATRIX SITUATED IN ROWS AND COLUMNS
C     LOW THROUGH IGH TO UPPER HESSENBERG FORM BY
C     STABILIZED ELEMENTARY SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N.
C
C        A CONTAINS THE INPUT MATRIX.
C
C     ON OUTPUT
C
C        A CONTAINS THE HESSENBERG MATRIX.  THE MULTIPLIERS
C          WHICH WERE USED IN THE REDUCTION ARE STORED IN THE
C          REMAINING TRIANGLE UNDER THE HESSENBERG MATRIX.
C
C        INT CONTAINS INFORMATION ON THE ROWS AND COLUMNS
C          INTERCHANGED IN THE REDUCTION.
C          ONLY ELEMENTS LOW THROUGH IGH ARE USED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      LA = IGH - 1
      KP1 = LOW + 1
      IF (LA .LT. KP1) GO TO 200
C
      DO 180 M = KP1, LA
         MM1 = M - 1
         X = 0.0D0
         I = M
C
         DO 100 J = M, IGH
            IF (DABS(A(J,MM1)) .LE. DABS(X)) GO TO 100
            X = A(J,MM1)
            I = J
  100    CONTINUE
C
         INT(M) = I
         IF (I .EQ. M) GO TO 130
C     .......... INTERCHANGE ROWS AND COLUMNS OF A ..........
         DO 110 J = MM1, N
            Y = A(I,J)
            A(I,J) = A(M,J)
            A(M,J) = Y
  110    CONTINUE
C
         DO 120 J = 1, IGH
            Y = A(J,I)
            A(J,I) = A(J,M)
            A(J,M) = Y
  120    CONTINUE
C     .......... END INTERCHANGE ..........
  130    IF (X .EQ. 0.0D0) GO TO 180
         MP1 = M + 1
C
         DO 160 I = MP1, IGH
            Y = A(I,MM1)
            IF (Y .EQ. 0.0D0) GO TO 160
            Y = Y / X
            A(I,MM1) = Y
C
            DO 140 J = M, N
  140       A(I,J) = A(I,J) - Y * A(M,J)
C
            DO 150 J = 1, IGH
  150       A(J,M) = A(J,M) + Y * A(J,I)
C
  160    CONTINUE
C
  180 CONTINUE
C
  200 RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE ELTRAN(NM,N,LOW,IGH,A,INT,Z)
C
      INTEGER I,J,N,KL,MM,MP,NM,IGH,LOW,MP1
      DOUBLE PRECISION A(NM,IGH),Z(NM,N)
      INTEGER INT(IGH)
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE ELMTRANS,
C     NUM. MATH. 16, 181-204(1970) BY PETERS AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971).
C
C     THIS SUBROUTINE ACCUMULATES THE STABILIZED ELEMENTARY
C     SIMILARITY TRANSFORMATIONS USED IN THE REDUCTION OF A
C     REAL GENERAL MATRIX TO UPPER HESSENBERG FORM BY  ELMHES.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N.
C
C        A CONTAINS THE MULTIPLIERS WHICH WERE USED IN THE
C          REDUCTION BY  ELMHES  IN ITS LOWER TRIANGLE
C          BELOW THE SUBDIAGONAL.
C
C        INT CONTAINS INFORMATION ON THE ROWS AND COLUMNS
C          INTERCHANGED IN THE REDUCTION BY  ELMHES.
C          ONLY ELEMENTS LOW THROUGH IGH ARE USED.
C
C     ON OUTPUT
C
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
C          REDUCTION BY  ELMHES.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
C     .......... INITIALIZE Z TO IDENTITY MATRIX ..........
      DO 80 J = 1, N
C
         DO 60 I = 1, N
   60    Z(I,J) = 0.0D0
C
         Z(J,J) = 1.0D0
   80 CONTINUE
C
      KL = IGH - LOW - 1
      IF (KL .LT. 1) GO TO 200
C     .......... FOR MP=IGH-1 STEP -1 UNTIL LOW+1 DO -- ..........
      DO 140 MM = 1, KL
         MP = IGH - MM
         MP1 = MP + 1
C
         DO 100 I = MP1, IGH
  100    Z(I,MP) = A(I,MP-1)
C
         I = INT(MP)
         IF (I .EQ. MP) GO TO 140
C
         DO 130 J = MP, IGH
            Z(MP,J) = Z(I,J)
            Z(I,J) = 0.0D0
  130    CONTINUE
C
         Z(I,MP) = 1.0D0
  140 CONTINUE
C
  200 RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C  EISPACK routines needed in the computation of Floquet multipliers
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE QZHES(NM,N,A,B,MATZ,Z)
C
      INTEGER I,J,K,L,N,LB,L1,NM,NK1,NM1,NM2
      DOUBLE PRECISION A(NM,N),B(NM,N),Z(NM,N)
      DOUBLE PRECISION R,S,T,U1,U2,V1,V2,RHO
      LOGICAL MATZ
C
C     THIS SUBROUTINE IS THE FIRST STEP OF THE QZ ALGORITHM
C     FOR SOLVING GENERALIZED MATRIX EIGENVALUE PROBLEMS,
C     SIAM J. NUMER. ANAL. 10, 241-256(1973) BY MOLER AND STEWART.
C
C     THIS SUBROUTINE ACCEPTS A PAIR OF REAL GENERAL MATRICES AND
C     REDUCES ONE OF THEM TO UPPER HESSENBERG FORM AND THE OTHER
C     TO UPPER TRIANGULAR FORM USING ORTHOGONAL TRANSFORMATIONS.
C     IT IS USUALLY FOLLOWED BY  QZIT,  QZVAL  AND, POSSIBLY,  QZVEC.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRICES.
C
C        A CONTAINS A REAL GENERAL MATRIX.
C
C        B CONTAINS A REAL GENERAL MATRIX.
C
C        MATZ SHOULD BE SET TO .TRUE. IF THE RIGHT HAND TRANSFORMATIONS
C          ARE TO BE ACCUMULATED FOR LATER USE IN COMPUTING
C          EIGENVECTORS, AND TO .FALSE. OTHERWISE.
C
C     ON OUTPUT
C
C        A HAS BEEN REDUCED TO UPPER HESSENBERG FORM.  THE ELEMENTS
C          BELOW THE FIRST SUBDIAGONAL HAVE BEEN SET TO ZERO.
C
C        B HAS BEEN REDUCED TO UPPER TRIANGULAR FORM.  THE ELEMENTS
C          BELOW THE MAIN DIAGONAL HAVE BEEN SET TO ZERO.
C
C        Z CONTAINS THE PRODUCT OF THE RIGHT HAND TRANSFORMATIONS IF
C          MATZ HAS BEEN SET TO .TRUE.  OTHERWISE, Z IS NOT REFERENCED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
C     .......... INITIALIZE Z ..........
      IF (.NOT. MATZ) GO TO 10
C
      DO 3 J = 1, N
C
         DO 2 I = 1, N
            Z(I,J) = 0.0D0
    2    CONTINUE
C
         Z(J,J) = 1.0D0
    3 CONTINUE
C     .......... REDUCE B TO UPPER TRIANGULAR FORM ..........
   10 IF (N .LE. 1) GO TO 170
      NM1 = N - 1
C
      DO 100 L = 1, NM1
         L1 = L + 1
         S = 0.0D0
C
         DO 20 I = L1, N
            S = S + DABS(B(I,L))
   20    CONTINUE
C
         IF (S .EQ. 0.0D0) GO TO 100
         S = S + DABS(B(L,L))
         R = 0.0D0
C
         DO 25 I = L, N
            B(I,L) = B(I,L) / S
            R = R + B(I,L)**2
   25    CONTINUE
C
         R = DSIGN(DSQRT(R),B(L,L))
         B(L,L) = B(L,L) + R
         RHO = R * B(L,L)
C
         DO 50 J = L1, N
            T = 0.0D0
C
            DO 30 I = L, N
               T = T + B(I,L) * B(I,J)
   30       CONTINUE
C
            T = -T / RHO
C
            DO 40 I = L, N
               B(I,J) = B(I,J) + T * B(I,L)
   40       CONTINUE
C
   50    CONTINUE
C
         DO 80 J = 1, N
            T = 0.0D0
C
            DO 60 I = L, N
               T = T + B(I,L) * A(I,J)
   60       CONTINUE
C
            T = -T / RHO
C
            DO 70 I = L, N
               A(I,J) = A(I,J) + T * B(I,L)
   70       CONTINUE
C
   80    CONTINUE
C
         B(L,L) = -S * R
C
         DO 90 I = L1, N
            B(I,L) = 0.0D0
   90    CONTINUE
C
  100 CONTINUE
C     .......... REDUCE A TO UPPER HESSENBERG FORM, WHILE
C                KEEPING B TRIANGULAR ..........
      IF (N .EQ. 2) GO TO 170
      NM2 = N - 2
C
      DO 160 K = 1, NM2
         NK1 = NM1 - K
C     .......... FOR L=N-1 STEP -1 UNTIL K+1 DO -- ..........
         DO 150 LB = 1, NK1
            L = N - LB
            L1 = L + 1
C     .......... ZERO A(L+1,K) ..........
            S = DABS(A(L,K)) + DABS(A(L1,K))
            IF (S .EQ. 0.0D0) GO TO 150
            U1 = A(L,K) / S
            U2 = A(L1,K) / S
            R = DSIGN(DSQRT(U1*U1+U2*U2),U1)
            V1 =  -(U1 + R) / R
            V2 = -U2 / R
            U2 = V2 / V1
C
            DO 110 J = K, N
               T = A(L,J) + U2 * A(L1,J)
               A(L,J) = A(L,J) + T * V1
               A(L1,J) = A(L1,J) + T * V2
  110       CONTINUE
C
            A(L1,K) = 0.0D0
C
            DO 120 J = L, N
               T = B(L,J) + U2 * B(L1,J)
               B(L,J) = B(L,J) + T * V1
               B(L1,J) = B(L1,J) + T * V2
  120       CONTINUE
C     .......... ZERO B(L+1,L) ..........
            S = DABS(B(L1,L1)) + DABS(B(L1,L))
            IF (S .EQ. 0.0D0) GO TO 150
            U1 = B(L1,L1) / S
            U2 = B(L1,L) / S
            R = DSIGN(DSQRT(U1*U1+U2*U2),U1)
            V1 =  -(U1 + R) / R
            V2 = -U2 / R
            U2 = V2 / V1
C
            DO 130 I = 1, L1
               T = B(I,L1) + U2 * B(I,L)
               B(I,L1) = B(I,L1) + T * V1
               B(I,L) = B(I,L) + T * V2
  130       CONTINUE
C
            B(L1,L) = 0.0D0
C
            DO 140 I = 1, N
               T = A(I,L1) + U2 * A(I,L)
               A(I,L1) = A(I,L1) + T * V1
               A(I,L) = A(I,L) + T * V2
  140       CONTINUE
C
            IF (.NOT. MATZ) GO TO 150
C
            DO 145 I = 1, N
               T = Z(I,L1) + U2 * Z(I,L)
               Z(I,L1) = Z(I,L1) + T * V1
               Z(I,L) = Z(I,L) + T * V2
  145       CONTINUE
C
  150    CONTINUE
C
  160 CONTINUE
C
  170 RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE QZIT(NM,N,A,B,EPS1,MATZ,Z,IERR)
C
      INTEGER I,J,K,L,N,EN,K1,K2,LD,LL,L1,NA,NM,ISH,ITN,ITS,KM1,LM1,
     X        ENM2,IERR,LOR1,ENORN
      DOUBLE PRECISION A(NM,N),B(NM,N),Z(NM,N)
      DOUBLE PRECISION R,S,T,A1,A2,A3,EP,SH,U1,U2,U3,V1,V2,V3,ANI,A11,
     X       A12,A21,A22,A33,A34,A43,A44,BNI,B11,B12,B22,B33,B34,
     X       B44,EPSA,EPSB,EPS1,ANORM,BNORM,EPSLON
      LOGICAL MATZ,NOTLAS
C
C     THIS SUBROUTINE IS THE SECOND STEP OF THE QZ ALGORITHM
C     FOR SOLVING GENERALIZED MATRIX EIGENVALUE PROBLEMS,
C     SIAM J. NUMER. ANAL. 10, 241-256(1973) BY MOLER AND STEWART,
C     AS MODIFIED IN TECHNICAL NOTE NASA TN D-7305(1973) BY WARD.
C
C     THIS SUBROUTINE ACCEPTS A PAIR OF REAL MATRICES, ONE OF THEM
C     IN UPPER HESSENBERG FORM AND THE OTHER IN UPPER TRIANGULAR FORM.
C     IT REDUCES THE HESSENBERG MATRIX TO QUASI-TRIANGULAR FORM USING
C     ORTHOGONAL TRANSFORMATIONS WHILE MAINTAINING THE TRIANGULAR FORM
C     OF THE OTHER MATRIX.  IT IS USUALLY PRECEDED BY  QZHES  AND
C     FOLLOWED BY  QZVAL  AND, POSSIBLY,  QZVEC.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRICES.
C
C        A CONTAINS A REAL UPPER HESSENBERG MATRIX.
C
C        B CONTAINS A REAL UPPER TRIANGULAR MATRIX.
C
C        EPS1 IS A TOLERANCE USED TO DETERMINE NEGLIGIBLE ELEMENTS.
C          EPS1 = 0.0 (OR NEGATIVE) MAY BE INPUT, IN WHICH CASE AN
C          ELEMENT WILL BE NEGLECTED ONLY IF IT IS LESS THAN ROUNDOFF
C          ERROR TIMES THE NORM OF ITS MATRIX.  IF THE INPUT EPS1 IS
C          POSITIVE, THEN AN ELEMENT WILL BE CONSIDERED NEGLIGIBLE
C          IF IT IS LESS THAN EPS1 TIMES THE NORM OF ITS MATRIX.  A
C          POSITIVE VALUE OF EPS1 MAY RESULT IN FASTER EXECUTION,
C          BUT LESS ACCURATE RESULTS.
C
C        MATZ SHOULD BE SET TO .TRUE. IF THE RIGHT HAND TRANSFORMATIONS
C          ARE TO BE ACCUMULATED FOR LATER USE IN COMPUTING
C          EIGENVECTORS, AND TO .FALSE. OTHERWISE.
C
C        Z CONTAINS, IF MATZ HAS BEEN SET TO .TRUE., THE
C          TRANSFORMATION MATRIX PRODUCED IN THE REDUCTION
C          BY  QZHES, IF PERFORMED, OR ELSE THE IDENTITY MATRIX.
C          IF MATZ HAS BEEN SET TO .FALSE., Z IS NOT REFERENCED.
C
C     ON OUTPUT
C
C        A HAS BEEN REDUCED TO QUASI-TRIANGULAR FORM.  THE ELEMENTS
C          BELOW THE FIRST SUBDIAGONAL ARE STILL ZERO AND NO TWO
C          CONSECUTIVE SUBDIAGONAL ELEMENTS ARE NONZERO.
C
C        B IS STILL IN UPPER TRIANGULAR FORM, ALTHOUGH ITS ELEMENTS
C          HAVE BEEN ALTERED.  THE LOCATION B(N,1) IS USED TO STORE
C          EPS1 TIMES THE NORM OF B FOR LATER USE BY  QZVAL  AND  QZVEC.
C
C        Z CONTAINS THE PRODUCT OF THE RIGHT HAND TRANSFORMATIONS
C          (FOR BOTH STEPS) IF MATZ HAS BEEN SET TO .TRUE..
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE LIMIT OF 30*N ITERATIONS IS EXHAUSTED
C                     WHILE THE J-TH EIGENVALUE IS BEING SOUGHT.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IERR = 0
C     .......... COMPUTE EPSA,EPSB ..........
      ANORM = 0.0D0
      BNORM = 0.0D0
C
      DO 30 I = 1, N
         ANI = 0.0D0
         IF (I .NE. 1) ANI = DABS(A(I,I-1))
         BNI = 0.0D0
C
         DO 20 J = I, N
            ANI = ANI + DABS(A(I,J))
            BNI = BNI + DABS(B(I,J))
   20    CONTINUE
C
         IF (ANI .GT. ANORM) ANORM = ANI
         IF (BNI .GT. BNORM) BNORM = BNI
   30 CONTINUE
C
      IF (ANORM .EQ. 0.0D0) ANORM = 1.0D0
      IF (BNORM .EQ. 0.0D0) BNORM = 1.0D0
      EP = EPS1
      IF (EP .GT. 0.0D0) GO TO 50
C     .......... USE ROUNDOFF LEVEL IF EPS1 IS ZERO ..........
      EP = EPSLON(1.0D0)
   50 EPSA = EP * ANORM
      EPSB = EP * BNORM
C     .......... REDUCE A TO QUASI-TRIANGULAR FORM, WHILE
C                KEEPING B TRIANGULAR ..........
      LOR1 = 1
      ENORN = N
      EN = N
      ITN = 30*N
C     .......... BEGIN QZ STEP ..........
   60 IF (EN .LE. 2) GO TO 1001
      IF (.NOT. MATZ) ENORN = EN
      ITS = 0
      NA = EN - 1
      ENM2 = NA - 1
   70 ISH = 2
C     .......... CHECK FOR CONVERGENCE OR REDUCIBILITY.
C                FOR L=EN STEP -1 UNTIL 1 DO -- ..........
      DO 80 LL = 1, EN
         LM1 = EN - LL
         L = LM1 + 1
         IF (L .EQ. 1) GO TO 95
         IF (DABS(A(L,LM1)) .LE. EPSA) GO TO 90
   80 CONTINUE
C
   90 A(L,LM1) = 0.0D0
      IF (L .LT. NA) GO TO 95
C     .......... 1-BY-1 OR 2-BY-2 BLOCK ISOLATED ..........
      EN = LM1
      GO TO 60
C     .......... CHECK FOR SMALL TOP OF B ..........
   95 LD = L
  100 L1 = L + 1
      B11 = B(L,L)
      IF (DABS(B11) .GT. EPSB) GO TO 120
      B(L,L) = 0.0D0
      S = DABS(A(L,L)) + DABS(A(L1,L))
      U1 = A(L,L) / S
      U2 = A(L1,L) / S
      R = DSIGN(DSQRT(U1*U1+U2*U2),U1)
      V1 = -(U1 + R) / R
      V2 = -U2 / R
      U2 = V2 / V1
C
      DO 110 J = L, ENORN
         T = A(L,J) + U2 * A(L1,J)
         A(L,J) = A(L,J) + T * V1
         A(L1,J) = A(L1,J) + T * V2
         T = B(L,J) + U2 * B(L1,J)
         B(L,J) = B(L,J) + T * V1
         B(L1,J) = B(L1,J) + T * V2
  110 CONTINUE
C
      IF (L .NE. 1) A(L,LM1) = -A(L,LM1)
      LM1 = L
      L = L1
      GO TO 90
  120 A11 = A(L,L) / B11
      A21 = A(L1,L) / B11
      IF (ISH .EQ. 1) GO TO 140
C     .......... ITERATION STRATEGY ..........
      IF (ITN .EQ. 0) GO TO 1000
      IF (ITS .EQ. 10) GO TO 155
C     .......... DETERMINE TYPE OF SHIFT ..........
      B22 = B(L1,L1)
      IF (DABS(B22) .LT. EPSB) B22 = EPSB
      B33 = B(NA,NA)
      IF (DABS(B33) .LT. EPSB) B33 = EPSB
      B44 = B(EN,EN)
      IF (DABS(B44) .LT. EPSB) B44 = EPSB
      A33 = A(NA,NA) / B33
      A34 = A(NA,EN) / B44
      A43 = A(EN,NA) / B33
      A44 = A(EN,EN) / B44
      B34 = B(NA,EN) / B44
      T = 0.5D0 * (A43 * B34 - A33 - A44)
      R = T * T + A34 * A43 - A33 * A44
      IF (R .LT. 0.0D0) GO TO 150
C     .......... DETERMINE SINGLE SHIFT ZEROTH COLUMN OF A ..........
      ISH = 1
      R = DSQRT(R)
      SH = -T + R
      S = -T - R
      IF (DABS(S-A44) .LT. DABS(SH-A44)) SH = S
C     .......... LOOK FOR TWO CONSECUTIVE SMALL
C                SUB-DIAGONAL ELEMENTS OF A.
C                FOR L=EN-2 STEP -1 UNTIL LD DO -- ..........
      DO 130 LL = LD, ENM2
         L = ENM2 + LD - LL
         IF (L .EQ. LD) GO TO 140
         LM1 = L - 1
         L1 = L + 1
         T = A(L,L)
         IF (DABS(B(L,L)) .GT. EPSB) T = T - SH * B(L,L)
         IF (DABS(A(L,LM1)) .LE. DABS(T/A(L1,L)) * EPSA) GO TO 100
  130 CONTINUE
C
  140 A1 = A11 - SH
      A2 = A21
      IF (L .NE. LD) A(L,LM1) = -A(L,LM1)
      GO TO 160
C     .......... DETERMINE DOUBLE SHIFT ZEROTH COLUMN OF A ..........
  150 A12 = A(L,L1) / B22
      A22 = A(L1,L1) / B22
      B12 = B(L,L1) / B22
      A1 = ((A33 - A11) * (A44 - A11) - A34 * A43 + A43 * B34 * A11)
     X     / A21 + A12 - A11 * B12
      A2 = (A22 - A11) - A21 * B12 - (A33 - A11) - (A44 - A11)
     X     + A43 * B34
      A3 = A(L1+1,L1) / B22
      GO TO 160
C     .......... AD HOC SHIFT ..........
  155 A1 = 0.0D0
      A2 = 1.0D0
      A3 = 1.1605D0
  160 ITS = ITS + 1
      ITN = ITN - 1
      IF (.NOT. MATZ) LOR1 = LD
C     .......... MAIN LOOP ..........
      DO 260 K = L, NA
         NOTLAS = K .NE. NA .AND. ISH .EQ. 2
         K1 = K + 1
         K2 = K + 2
         KM1 = MAX0(K-1,L)
         LL = MIN0(EN,K1+ISH)
         IF (NOTLAS) GO TO 190
C     .......... ZERO A(K+1,K-1) ..........
         IF (K .EQ. L) GO TO 170
         A1 = A(K,KM1)
         A2 = A(K1,KM1)
  170    S = DABS(A1) + DABS(A2)
         IF (S .EQ. 0.0D0) GO TO 70
         U1 = A1 / S
         U2 = A2 / S
         R = DSIGN(DSQRT(U1*U1+U2*U2),U1)
         V1 = -(U1 + R) / R
         V2 = -U2 / R
         U2 = V2 / V1
C
         DO 180 J = KM1, ENORN
            T = A(K,J) + U2 * A(K1,J)
            A(K,J) = A(K,J) + T * V1
            A(K1,J) = A(K1,J) + T * V2
            T = B(K,J) + U2 * B(K1,J)
            B(K,J) = B(K,J) + T * V1
            B(K1,J) = B(K1,J) + T * V2
  180    CONTINUE
C
         IF (K .NE. L) A(K1,KM1) = 0.0D0
         GO TO 240
C     .......... ZERO A(K+1,K-1) AND A(K+2,K-1) ..........
  190    IF (K .EQ. L) GO TO 200
         A1 = A(K,KM1)
         A2 = A(K1,KM1)
         A3 = A(K2,KM1)
  200    S = DABS(A1) + DABS(A2) + DABS(A3)
         IF (S .EQ. 0.0D0) GO TO 260
         U1 = A1 / S
         U2 = A2 / S
         U3 = A3 / S
         R = DSIGN(DSQRT(U1*U1+U2*U2+U3*U3),U1)
         V1 = -(U1 + R) / R
         V2 = -U2 / R
         V3 = -U3 / R
         U2 = V2 / V1
         U3 = V3 / V1
C
         DO 210 J = KM1, ENORN
            T = A(K,J) + U2 * A(K1,J) + U3 * A(K2,J)
            A(K,J) = A(K,J) + T * V1
            A(K1,J) = A(K1,J) + T * V2
            A(K2,J) = A(K2,J) + T * V3
            T = B(K,J) + U2 * B(K1,J) + U3 * B(K2,J)
            B(K,J) = B(K,J) + T * V1
            B(K1,J) = B(K1,J) + T * V2
            B(K2,J) = B(K2,J) + T * V3
  210    CONTINUE
C
         IF (K .EQ. L) GO TO 220
         A(K1,KM1) = 0.0D0
         A(K2,KM1) = 0.0D0
C     .......... ZERO B(K+2,K+1) AND B(K+2,K) ..........
  220    S = DABS(B(K2,K2)) + DABS(B(K2,K1)) + DABS(B(K2,K))
         IF (S .EQ. 0.0D0) GO TO 240
         U1 = B(K2,K2) / S
         U2 = B(K2,K1) / S
         U3 = B(K2,K) / S
         R = DSIGN(DSQRT(U1*U1+U2*U2+U3*U3),U1)
         V1 = -(U1 + R) / R
         V2 = -U2 / R
         V3 = -U3 / R
         U2 = V2 / V1
         U3 = V3 / V1
C
         DO 230 I = LOR1, LL
            T = A(I,K2) + U2 * A(I,K1) + U3 * A(I,K)
            A(I,K2) = A(I,K2) + T * V1
            A(I,K1) = A(I,K1) + T * V2
            A(I,K) = A(I,K) + T * V3
            T = B(I,K2) + U2 * B(I,K1) + U3 * B(I,K)
            B(I,K2) = B(I,K2) + T * V1
            B(I,K1) = B(I,K1) + T * V2
            B(I,K) = B(I,K) + T * V3
  230    CONTINUE
C
         B(K2,K) = 0.0D0
         B(K2,K1) = 0.0D0
         IF (.NOT. MATZ) GO TO 240
C
         DO 235 I = 1, N
            T = Z(I,K2) + U2 * Z(I,K1) + U3 * Z(I,K)
            Z(I,K2) = Z(I,K2) + T * V1
            Z(I,K1) = Z(I,K1) + T * V2
            Z(I,K) = Z(I,K) + T * V3
  235    CONTINUE
C     .......... ZERO B(K+1,K) ..........
  240    S = DABS(B(K1,K1)) + DABS(B(K1,K))
         IF (S .EQ. 0.0D0) GO TO 260
         U1 = B(K1,K1) / S
         U2 = B(K1,K) / S
         R = DSIGN(DSQRT(U1*U1+U2*U2),U1)
         V1 = -(U1 + R) / R
         V2 = -U2 / R
         U2 = V2 / V1
C
         DO 250 I = LOR1, LL
            T = A(I,K1) + U2 * A(I,K)
            A(I,K1) = A(I,K1) + T * V1
            A(I,K) = A(I,K) + T * V2
            T = B(I,K1) + U2 * B(I,K)
            B(I,K1) = B(I,K1) + T * V1
            B(I,K) = B(I,K) + T * V2
  250    CONTINUE
C
         B(K1,K) = 0.0D0
         IF (.NOT. MATZ) GO TO 260
C
         DO 255 I = 1, N
            T = Z(I,K1) + U2 * Z(I,K)
            Z(I,K1) = Z(I,K1) + T * V1
            Z(I,K) = Z(I,K) + T * V2
  255    CONTINUE
C
  260 CONTINUE
C     .......... END QZ STEP ..........
      GO TO 70
C     .......... SET ERROR -- ALL EIGENVALUES HAVE NOT
C                CONVERGED AFTER 30*N ITERATIONS ..........
 1000 IERR = EN
C     .......... SAVE EPSB FOR USE BY QZVAL AND QZVEC ..........
 1001 IF (N .GT. 1) B(N,1) = EPSB
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE QZVAL(NM,N,A,B,ALFR,ALFI,BETA,MATZ,Z)
C
      INTEGER I,J,N,EN,NA,NM,NN,ISW
      DOUBLE PRECISION A(NM,N),B(NM,N),ALFR(N),ALFI(N),BETA(N),Z(NM,N)
      DOUBLE PRECISION C,D,E,R,S,T,AN,A1,A2,BN,CQ,CZ,DI,DR,EI,TI,TR,U1,
     X       U2,V1,V2,A1I,A11,A12,A2I,A21,A22,B11,B12,B22,SQI,SQR,
     X       SSI,SSR,SZI,SZR,A11I,A11R,A12I,A12R,A22I,A22R,EPSB
      LOGICAL MATZ
C
C     THIS SUBROUTINE IS THE THIRD STEP OF THE QZ ALGORITHM
C     FOR SOLVING GENERALIZED MATRIX EIGENVALUE PROBLEMS,
C     SIAM J. NUMER. ANAL. 10, 241-256(1973) BY MOLER AND STEWART.
C
C     THIS SUBROUTINE ACCEPTS A PAIR OF REAL MATRICES, ONE OF THEM
C     IN QUASI-TRIANGULAR FORM AND THE OTHER IN UPPER TRIANGULAR FORM.
C     IT REDUCES THE QUASI-TRIANGULAR MATRIX FURTHER, SO THAT ANY
C     REMAINING 2-BY-2 BLOCKS CORRESPOND TO PAIRS OF COMPLEX
C     EIGENVALUES, AND RETURNS QUANTITIES WHOSE RATIOS GIVE THE
C     GENERALIZED EIGENVALUES.  IT IS USUALLY PRECEDED BY  QZHES
C     AND  QZIT  AND MAY BE FOLLOWED BY  QZVEC.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRICES.
C
C        A CONTAINS A REAL UPPER QUASI-TRIANGULAR MATRIX.
C
C        B CONTAINS A REAL UPPER TRIANGULAR MATRIX.  IN ADDITION,
C          LOCATION B(N,1) CONTAINS THE TOLERANCE QUANTITY (EPSB)
C          COMPUTED AND SAVED IN  QZIT.
C
C        MATZ SHOULD BE SET TO .TRUE. IF THE RIGHT HAND TRANSFORMATIONS
C          ARE TO BE ACCUMULATED FOR LATER USE IN COMPUTING
C          EIGENVECTORS, AND TO .FALSE. OTHERWISE.
C
C        Z CONTAINS, IF MATZ HAS BEEN SET TO .TRUE., THE
C          TRANSFORMATION MATRIX PRODUCED IN THE REDUCTIONS BY QZHES
C          AND QZIT, IF PERFORMED, OR ELSE THE IDENTITY MATRIX.
C          IF MATZ HAS BEEN SET TO .FALSE., Z IS NOT REFERENCED.
C
C     ON OUTPUT
C
C        A HAS BEEN REDUCED FURTHER TO A QUASI-TRIANGULAR MATRIX
C          IN WHICH ALL NONZERO SUBDIAGONAL ELEMENTS CORRESPOND TO
C          PAIRS OF COMPLEX EIGENVALUES.
C
C        B IS STILL IN UPPER TRIANGULAR FORM, ALTHOUGH ITS ELEMENTS
C          HAVE BEEN ALTERED.  B(N,1) IS UNALTERED.
C
C        ALFR AND ALFI CONTAIN THE REAL AND IMAGINARY PARTS OF THE
C          DIAGONAL ELEMENTS OF THE TRIANGULAR MATRIX THAT WOULD BE
C          OBTAINED IF A WERE REDUCED COMPLETELY TO TRIANGULAR FORM
C          BY UNITARY TRANSFORMATIONS.  NON-ZERO VALUES OF ALFI OCCUR
C          IN PAIRS, THE FIRST MEMBER POSITIVE AND THE SECOND NEGATIVE.
C
C        BETA CONTAINS THE DIAGONAL ELEMENTS OF THE CORRESPONDING B,
C          NORMALIZED TO BE REAL AND NON-NEGATIVE.  THE GENERALIZED
C          EIGENVALUES ARE THEN THE RATIOS ((ALFR+I*ALFI)/BETA).
C
C        Z CONTAINS THE PRODUCT OF THE RIGHT HAND TRANSFORMATIONS
C          (FOR ALL THREE STEPS) IF MATZ HAS BEEN SET TO .TRUE.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      EPSB = B(N,1)
      ISW = 1
C     .......... FIND EIGENVALUES OF QUASI-TRIANGULAR MATRICES.
C                FOR EN=N STEP -1 UNTIL 1 DO -- ..........
      DO 510 NN = 1, N
         EN = N + 1 - NN
         NA = EN - 1
         IF (ISW .EQ. 2) GO TO 505
         IF (EN .EQ. 1) GO TO 410
         IF (A(EN,NA) .NE. 0.0D0) GO TO 420
C     .......... 1-BY-1 BLOCK, ONE REAL ROOT ..........
  410    ALFR(EN) = A(EN,EN)
         IF (B(EN,EN) .LT. 0.0D0) ALFR(EN) = -ALFR(EN)
         BETA(EN) = DABS(B(EN,EN))
         ALFI(EN) = 0.0D0
         GO TO 510
C     .......... 2-BY-2 BLOCK ..........
  420    IF (DABS(B(NA,NA)) .LE. EPSB) GO TO 455
         IF (DABS(B(EN,EN)) .GT. EPSB) GO TO 430
         A1 = A(EN,EN)
         A2 = A(EN,NA)
         BN = 0.0D0
         GO TO 435
  430    AN = DABS(A(NA,NA)) + DABS(A(NA,EN)) + DABS(A(EN,NA))
     X      + DABS(A(EN,EN))
         BN = DABS(B(NA,NA)) + DABS(B(NA,EN)) + DABS(B(EN,EN))
         A11 = A(NA,NA) / AN
         A12 = A(NA,EN) / AN
         A21 = A(EN,NA) / AN
         A22 = A(EN,EN) / AN
         B11 = B(NA,NA) / BN
         B12 = B(NA,EN) / BN
         B22 = B(EN,EN) / BN
         E = A11 / B11
         EI = A22 / B22
         S = A21 / (B11 * B22)
         T = (A22 - E * B22) / B22
         IF (DABS(E) .LE. DABS(EI)) GO TO 431
         E = EI
         T = (A11 - E * B11) / B11
  431    C = 0.5D0 * (T - S * B12)
         D = C * C + S * (A12 - E * B12)
         IF (D .LT. 0.0D0) GO TO 480
C     .......... TWO REAL ROOTS.
C                ZERO BOTH A(EN,NA) AND B(EN,NA) ..........
         E = E + (C + DSIGN(DSQRT(D),C))
         A11 = A11 - E * B11
         A12 = A12 - E * B12
         A22 = A22 - E * B22
         IF (DABS(A11) + DABS(A12) .LT.
     X       DABS(A21) + DABS(A22)) GO TO 432
         A1 = A12
         A2 = A11
         GO TO 435
  432    A1 = A22
         A2 = A21
C     .......... CHOOSE AND APPLY REAL Z ..........
  435    S = DABS(A1) + DABS(A2)
         U1 = A1 / S
         U2 = A2 / S
         R = DSIGN(DSQRT(U1*U1+U2*U2),U1)
         V1 = -(U1 + R) / R
         V2 = -U2 / R
         U2 = V2 / V1
C
         DO 440 I = 1, EN
            T = A(I,EN) + U2 * A(I,NA)
            A(I,EN) = A(I,EN) + T * V1
            A(I,NA) = A(I,NA) + T * V2
            T = B(I,EN) + U2 * B(I,NA)
            B(I,EN) = B(I,EN) + T * V1
            B(I,NA) = B(I,NA) + T * V2
  440    CONTINUE
C
         IF (.NOT. MATZ) GO TO 450
C
         DO 445 I = 1, N
            T = Z(I,EN) + U2 * Z(I,NA)
            Z(I,EN) = Z(I,EN) + T * V1
            Z(I,NA) = Z(I,NA) + T * V2
  445    CONTINUE
C
  450    IF (BN .EQ. 0.0D0) GO TO 475
         IF (AN .LT. DABS(E) * BN) GO TO 455
         A1 = B(NA,NA)
         A2 = B(EN,NA)
         GO TO 460
  455    A1 = A(NA,NA)
         A2 = A(EN,NA)
C     .......... CHOOSE AND APPLY REAL Q ..........
  460    S = DABS(A1) + DABS(A2)
         IF (S .EQ. 0.0D0) GO TO 475
         U1 = A1 / S
         U2 = A2 / S
         R = DSIGN(DSQRT(U1*U1+U2*U2),U1)
         V1 = -(U1 + R) / R
         V2 = -U2 / R
         U2 = V2 / V1
C
         DO 470 J = NA, N
            T = A(NA,J) + U2 * A(EN,J)
            A(NA,J) = A(NA,J) + T * V1
            A(EN,J) = A(EN,J) + T * V2
            T = B(NA,J) + U2 * B(EN,J)
            B(NA,J) = B(NA,J) + T * V1
            B(EN,J) = B(EN,J) + T * V2
  470    CONTINUE
C
  475    A(EN,NA) = 0.0D0
         B(EN,NA) = 0.0D0
         ALFR(NA) = A(NA,NA)
         ALFR(EN) = A(EN,EN)
         IF (B(NA,NA) .LT. 0.0D0) ALFR(NA) = -ALFR(NA)
         IF (B(EN,EN) .LT. 0.0D0) ALFR(EN) = -ALFR(EN)
         BETA(NA) = DABS(B(NA,NA))
         BETA(EN) = DABS(B(EN,EN))
         ALFI(EN) = 0.0D0
         ALFI(NA) = 0.0D0
         GO TO 505
C     .......... TWO COMPLEX ROOTS ..........
  480    E = E + C
         EI = DSQRT(-D)
         A11R = A11 - E * B11
         A11I = EI * B11
         A12R = A12 - E * B12
         A12I = EI * B12
         A22R = A22 - E * B22
         A22I = EI * B22
         IF (DABS(A11R) + DABS(A11I) + DABS(A12R) + DABS(A12I) .LT.
     X       DABS(A21) + DABS(A22R) + DABS(A22I)) GO TO 482
         A1 = A12R
         A1I = A12I
         A2 = -A11R
         A2I = -A11I
         GO TO 485
  482    A1 = A22R
         A1I = A22I
         A2 = -A21
         A2I = 0.0D0
C     .......... CHOOSE COMPLEX Z ..........
  485    CZ = DSQRT(A1*A1+A1I*A1I)
         IF (CZ .EQ. 0.0D0) GO TO 487
         SZR = (A1 * A2 + A1I * A2I) / CZ
         SZI = (A1 * A2I - A1I * A2) / CZ
         R = DSQRT(CZ*CZ+SZR*SZR+SZI*SZI)
         CZ = CZ / R
         SZR = SZR / R
         SZI = SZI / R
         GO TO 490
  487    SZR = 1.0D0
         SZI = 0.0D0
  490    IF (AN .LT. (DABS(E) + EI) * BN) GO TO 492
         A1 = CZ * B11 + SZR * B12
         A1I = SZI * B12
         A2 = SZR * B22
         A2I = SZI * B22
         GO TO 495
  492    A1 = CZ * A11 + SZR * A12
         A1I = SZI * A12
         A2 = CZ * A21 + SZR * A22
         A2I = SZI * A22
C     .......... CHOOSE COMPLEX Q ..........
  495    CQ = DSQRT(A1*A1+A1I*A1I)
         IF (CQ .EQ. 0.0D0) GO TO 497
         SQR = (A1 * A2 + A1I * A2I) / CQ
         SQI = (A1 * A2I - A1I * A2) / CQ
         R = DSQRT(CQ*CQ+SQR*SQR+SQI*SQI)
         CQ = CQ / R
         SQR = SQR / R
         SQI = SQI / R
         GO TO 500
  497    SQR = 1.0D0
         SQI = 0.0D0
C     .......... COMPUTE DIAGONAL ELEMENTS THAT WOULD RESULT
C                IF TRANSFORMATIONS WERE APPLIED ..........
  500    SSR = SQR * SZR + SQI * SZI
         SSI = SQR * SZI - SQI * SZR
         I = 1
         TR = CQ * CZ * A11 + CQ * SZR * A12 + SQR * CZ * A21
     X      + SSR * A22
         TI = CQ * SZI * A12 - SQI * CZ * A21 + SSI * A22
         DR = CQ * CZ * B11 + CQ * SZR * B12 + SSR * B22
         DI = CQ * SZI * B12 + SSI * B22
         GO TO 503
  502    I = 2
         TR = SSR * A11 - SQR * CZ * A12 - CQ * SZR * A21
     X      + CQ * CZ * A22
         TI = -SSI * A11 - SQI * CZ * A12 + CQ * SZI * A21
         DR = SSR * B11 - SQR * CZ * B12 + CQ * CZ * B22
         DI = -SSI * B11 - SQI * CZ * B12
  503    T = TI * DR - TR * DI
         J = NA
         IF (T .LT. 0.0D0) J = EN
         R = DSQRT(DR*DR+DI*DI)
         BETA(J) = BN * R
         ALFR(J) = AN * (TR * DR + TI * DI) / R
         ALFI(J) = AN * T / R
         IF (I .EQ. 1) GO TO 502
  505    ISW = 3 - ISW
  510 CONTINUE
      B(N,1) = EPSB
C
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      DOUBLE PRECISION FUNCTION EPSLON (X)
      DOUBLE PRECISION X
C
C     ESTIMATE UNIT ROUNDOFF IN QUANTITIES OF SIZE X.
C
      DOUBLE PRECISION A,B,C,EPS
C
C     THIS PROGRAM SHOULD FUNCTION PROPERLY ON ALL SYSTEMS
C     SATISFYING THE FOLLOWING TWO ASSUMPTIONS,
C        1.  THE BASE USED IN REPRESENTING FLOATING POINT
C            NUMBERS IS NOT A POWER OF THREE.
C        2.  THE QUANTITY  A  IN STATEMENT 10 IS REPRESENTED TO
C            THE ACCURACY USED IN FLOATING POINT VARIABLES
C            THAT ARE STORED IN MEMORY.
C     THE STATEMENT NUMBER 10 AND THE GO TO 10 ARE INTENDED TO
C     FORCE OPTIMIZING COMPILERS TO GENERATE CODE SATISFYING
C     ASSUMPTION 2.
C     UNDER THESE ASSUMPTIONS, IT SHOULD BE TRUE THAT,
C            A  IS NOT EXACTLY EQUAL TO FOUR-THIRDS,
C            B  HAS A ZERO FOR ITS LAST BIT OR DIGIT,
C            C  IS NOT EXACTLY EQUAL TO ONE,
C            EPS  MEASURES THE SEPARATION OF 1.0 FROM
C                 THE NEXT LARGER FLOATING POINT NUMBER.
C     THE DEVELOPERS OF EISPACK WOULD APPRECIATE BEING INFORMED
C     ABOUT ANY SYSTEMS WHERE THESE ASSUMPTIONS DO NOT HOLD.
C
C     THIS VERSION DATED 4/6/83.
C
      A = 4.0D0/3.0D0
   10 B = A - 1.0D0
      C = B + B + B
      EPS = DABS(C-1.0D0)
      IF (EPS .EQ. 0.0D0) GO TO 10
      EPSLON = EPS*DABS(X)
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C  BLAS-1 routines needed in the computation of Floquet multipliers
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      DOUBLE PRECISION FUNCTION DNRM2 ( N, DX, INCX)
C     NOTE: THIS ROUTINE HAS BEEN ALTERED SLIGHTLY BY JONATHAN SHERRATT
C     01/07/11 TO AVOID THE USE OF THE ASSIGN STATEMENT AND CORRESPONDING
C     GOTO STATEMENTS WITH LISTS, SINCE THESE ARE NOT RECOGNISED BY SOME
C     FORTRAN95 COMPILERS THAT CAN OTHERWISE COPE WITH FORTRAN77 CODE
      INTEGER          NEXT
      DOUBLE PRECISION   DX(1), CUTLO, CUTHI, HITEST, SUM, XMAX,ZERO,ONE
      DATA   ZERO, ONE /0.0D0, 1.0D0/
C
C     EUCLIDEAN NORM OF THE N-VECTOR STORED IN DX() WITH STORAGE
C     INCREMENT INCX .
C     IF    N .LE. 0 RETURN WITH RESULT = 0.
C     IF N .GE. 1 THEN INCX MUST BE .GE. 1
C
C           C.L.LAWSON, 1978 JAN 08
C
C     FOUR PHASE METHOD     USING TWO BUILT-IN CONSTANTS THAT ARE
C     HOPEFULLY APPLICABLE TO ALL MACHINES.
C         CUTLO = MAXIMUM OF  DSQRT(U/EPS)  OVER ALL KNOWN MACHINES.
C         CUTHI = MINIMUM OF  DSQRT(V)      OVER ALL KNOWN MACHINES.
C     WHERE
C         EPS = SMALLEST NO. SUCH THAT EPS + 1. .GT. 1.
C         U   = SMALLEST POSITIVE NO.   (UNDERFLOW LIMIT)
C         V   = LARGEST  NO.            (OVERFLOW  LIMIT)
C
C     BRIEF OUTLINE OF ALGORITHM..
C
C     PHASE 1    SCANS ZERO COMPONENTS.
C     MOVE TO PHASE 2 WHEN A COMPONENT IS NONZERO AND .LE. CUTLO
C     MOVE TO PHASE 3 WHEN A COMPONENT IS .GT. CUTLO
C     MOVE TO PHASE 4 WHEN A COMPONENT IS .GE. CUTHI/M
C     WHERE M = N FOR X() REAL AND M = 2*N FOR COMPLEX.
C
C     VALUES FOR CUTLO AND CUTHI..
C     FROM THE ENVIRONMENTAL PARAMETERS LISTED IN THE IMSL CONVERTER
C     DOCUMENT THE LIMITING VALUES ARE AS FOLLOWS..
C     CUTLO, S.P.   U/EPS = 2**(-102) FOR  HONEYWELL.  CLOSE SECONDS ARE
C                   UNIVAC AND DEC AT 2**(-103)
C                   THUS CUTLO = 2**(-51) = 4.44089E-16
C     CUTHI, S.P.   V = 2**127 FOR UNIVAC, HONEYWELL, AND DEC.
C                   THUS CUTHI = 2**(63.5) = 1.30438E19
C     CUTLO, D.P.   U/EPS = 2**(-67) FOR HONEYWELL AND DEC.
C                   THUS CUTLO = 2**(-33.5) = 8.23181D-11
C     CUTHI, D.P.   SAME AS S.P.  CUTHI = 1.30438D19
C     DATA CUTLO, CUTHI / 8.232D-11,  1.304D19 /
C     DATA CUTLO, CUTHI / 4.441E-16,  1.304E19 /
      DATA CUTLO, CUTHI / 8.232D-11,  1.304D19 /
C
      IF(N .GT. 0) GO TO 10
         DNRM2  = ZERO
         GO TO 300
C
CJAS   10 ASSIGN 30 TO NEXT
CJAS      The above line was replaced by the following line by
CJAS      Jonathan Sherratt 01/07/11
 10      NEXT=30
      SUM = ZERO
      NN = N * INCX
C                                                 BEGIN MAIN LOOP
      I = 1
CJAS   20    GO TO NEXT,(30, 50, 70, 110)
CJAS      The above line was replaced by the following lines by
CJAS      Jonathan Sherratt 01/07/11
 20   IF (NEXT.EQ.30) THEN
         GOTO 30
      ENDIF
      IF (NEXT.EQ.50) THEN
         GOTO 50
      ENDIF
      IF (NEXT.EQ.70) THEN
         GOTO 70
      ENDIF
      IF (NEXT.EQ.110) THEN
         GOTO 110
      ENDIF
CJAS  End of replacement lines 
   30 IF( DABS(DX(I)) .GT. CUTLO) GO TO 85
CJAS      ASSIGN 50 TO NEXT
CJAS      The above line was replaced by the following line by
CJAS      Jonathan Sherratt 01/07/11
      NEXT=50
      XMAX = ZERO
C
C                        PHASE 1.  SUM IS ZERO
C
   50 IF( DX(I) .EQ. ZERO) GO TO 200
      IF( DABS(DX(I)) .GT. CUTLO) GO TO 85
C
C                                PREPARE FOR PHASE 2.
CJAS      ASSIGN 70 TO NEXT
CJAS      The above line was replaced by the following line by
CJAS      Jonathan Sherratt 01/07/11
      NEXT=70
      GO TO 105
C
C                                PREPARE FOR PHASE 4.
C
  100 I = J
CJAS      ASSIGN 110 TO NEXT
CJAS      The above line was replaced by the following line by
CJAS      Jonathan Sherratt 01/07/11
      NEXT=110
      SUM = (SUM / DX(I)) / DX(I)
  105 XMAX = DABS(DX(I))
      GO TO 115
C
C                   PHASE 2.  SUM IS SMALL.
C                             SCALE TO AVOID DESTRUCTIVE UNDERFLOW.
C
   70 IF( DABS(DX(I)) .GT. CUTLO ) GO TO 75
C
C                     COMMON CODE FOR PHASES 2 AND 4.
C                     IN PHASE 4 SUM IS LARGE.  SCALE TO AVOID OVERFLOW.
C
  110 IF( DABS(DX(I)) .LE. XMAX ) GO TO 115
         SUM = ONE + SUM * (XMAX / DX(I))**2
         XMAX = DABS(DX(I))
         GO TO 200
C
  115 SUM = SUM + (DX(I)/XMAX)**2
      GO TO 200
C
C
C                  PREPARE FOR PHASE 3.
C
   75 SUM = (SUM * XMAX) * XMAX
C
C
C     FOR REAL OR D.P. SET HITEST = CUTHI/N
C     FOR COMPLEX      SET HITEST = CUTHI/(2*N)
C
   85 HITEST = CUTHI/FLOAT( N )
C
C                   PHASE 3.  SUM IS MID-RANGE.  NO SCALING.
C
      DO 95 J =I,NN,INCX
      IF(DABS(DX(J)) .GE. HITEST) GO TO 100
   95    SUM = SUM + DX(J)**2
      DNRM2 = DSQRT( SUM )
      GO TO 300
C
  200 CONTINUE
      I = I + INCX
      IF ( I .LE. NN ) GO TO 20
C
C              END OF MAIN LOOP.
C
C              COMPUTE SQUARE ROOT AND ADJUST FOR SCALING.
C
      DNRM2 = XMAX * DSQRT(SUM)
  300 CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      DOUBLE PRECISION FUNCTION DDOT(N,DX,INCX,DY,INCY)
C
C     FORMS THE DOT PRODUCT OF TWO VECTORS.
C     USES UNROLLED LOOPS FOR INCREMENTS EQUAL TO ONE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DX(1),DY(1),DTEMP
      INTEGER I,INCX,INCY,IX,IY,M,MP1,N
C
      DDOT = 0.0D0
      DTEMP = 0.0D0
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1.AND.INCY.EQ.1)GO TO 20
C
C        CODE FOR UNEQUAL INCREMENTS OR EQUAL INCREMENTS
C          NOT EQUAL TO 1
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        DTEMP = DTEMP + DX(IX)*DY(IY)
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      DDOT = DTEMP
      RETURN
C
C        CODE FOR BOTH INCREMENTS EQUAL TO 1
C
C
C        CLEAN-UP LOOP
C
   20 M = MOD(N,5)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DTEMP = DTEMP + DX(I)*DY(I)
   30 CONTINUE
      IF( N .LT. 5 ) GO TO 60
   40 MP1 = M + 1
      DO 50 I = MP1,N,5
        DTEMP = DTEMP + DX(I)*DY(I) + DX(I + 1)*DY(I + 1) +
     *   DX(I + 2)*DY(I + 2) + DX(I + 3)*DY(I + 3) + DX(I + 4)*DY(I + 4)
   50 CONTINUE
   60 DDOT = DTEMP
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE  DSCAL(N,DA,DX,INCX)
C
C     SCALES A VECTOR BY A CONSTANT.
C     USES UNROLLED LOOPS FOR INCREMENT EQUAL TO ONE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DA,DX(1)
      INTEGER I,INCX,M,MP1,N,NINCX
C
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1)GO TO 20
C
C        CODE FOR INCREMENT NOT EQUAL TO 1
C
      NINCX = N*INCX
      DO 10 I = 1,NINCX,INCX
        DX(I) = DA*DX(I)
   10 CONTINUE
      RETURN
C
C        CODE FOR INCREMENT EQUAL TO 1
C
C
C        CLEAN-UP LOOP
C
   20 M = MOD(N,5)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DX(I) = DA*DX(I)
   30 CONTINUE
      IF( N .LT. 5 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,5
        DX(I) = DA*DX(I)
        DX(I + 1) = DA*DX(I + 1)
        DX(I + 2) = DA*DX(I + 2)
        DX(I + 3) = DA*DX(I + 3)
        DX(I + 4) = DA*DX(I + 4)
   50 CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      INTEGER FUNCTION IDAMAX(N,DX,INCX)
C
C     FINDS THE INDEX OF ELEMENT HAVING MAX. ABSOLUTE VALUE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DX(1),DMAX
      INTEGER I,INCX,IX,N
C
      IDAMAX = 0
      IF( N .LT. 1 ) RETURN
      IDAMAX = 1
      IF(N.EQ.1)RETURN
      IF(INCX.EQ.1)GO TO 20
C
C        CODE FOR INCREMENT NOT EQUAL TO 1
C
      IX = 1
      DMAX = DABS(DX(1))
      IX = IX + INCX
      DO 10 I = 2,N
         IF(DABS(DX(IX)).LE.DMAX) GO TO 5
         IDAMAX = I
         DMAX = DABS(DX(IX))
    5    IX = IX + INCX
   10 CONTINUE
      RETURN
C
C        CODE FOR INCREMENT EQUAL TO 1
C
   20 DMAX = DABS(DX(1))
      DO 30 I = 2,N
         IF(DABS(DX(I)).LE.DMAX) GO TO 30
         IDAMAX = I
         DMAX = DABS(DX(I))
   30 CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE DAXPY(N,DA,DX,INCX,DY,INCY)
C
C     CONSTANT TIMES A VECTOR PLUS A VECTOR.
C     USES UNROLLED LOOPS FOR INCREMENTS EQUAL TO ONE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DX(1),DY(1),DA
      INTEGER I,INCX,INCY,M,MP1,N
C
      IF(N.LE.0)RETURN
      IF (DA .EQ. 0.0D0) RETURN
      IF(INCX.EQ.1.AND.INCY.EQ.1)GO TO 20
C
C        CODE FOR UNEQUAL INCREMENTS OR EQUAL INCREMENTS
C          NOT EQUAL TO 1
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        DY(IY) = DY(IY) + DA*DX(IX)
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      RETURN
C
C        CODE FOR BOTH INCREMENTS EQUAL TO 1
C
C
C        CLEAN-UP LOOP
C
   20 M = MOD(N,4)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DY(I) = DY(I) + DA*DX(I)
   30 CONTINUE
      IF( N .LT. 4 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,4
        DY(I) = DY(I) + DA*DX(I)
        DY(I + 1) = DY(I + 1) + DA*DX(I + 1)
        DY(I + 2) = DY(I + 2) + DA*DX(I + 2)
        DY(I + 3) = DY(I + 3) + DA*DX(I + 3)
   50 CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE  DROT (N,DX,INCX,DY,INCY,C,S)
C
C     APPLIES A PLANE ROTATION.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DX(1),DY(1),DTEMP,C,S
      INTEGER I,INCX,INCY,IX,IY,N
C
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1.AND.INCY.EQ.1)GO TO 20
C
C       CODE FOR UNEQUAL INCREMENTS OR EQUAL INCREMENTS NOT EQUAL
C         TO 1
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        DTEMP = C*DX(IX) + S*DY(IY)
        DY(IY) = C*DY(IY) - S*DX(IX)
        DX(IX) = DTEMP
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      RETURN
C
C       CODE FOR BOTH INCREMENTS EQUAL TO 1
C
   20 DO 30 I = 1,N
        DTEMP = C*DX(I) + S*DY(I)
        DY(I) = C*DY(I) - S*DX(I)
        DX(I) = DTEMP
   30 CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE  DSWAP (N,DX,INCX,DY,INCY)
C
C     INTERCHANGES TWO VECTORS.
C     USES UNROLLED LOOPS FOR INCREMENTS EQUAL ONE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
      DOUBLE PRECISION DX(1),DY(1),DTEMP
      INTEGER I,INCX,INCY,IX,IY,M,MP1,N
C
      IF(N.LE.0)RETURN
      IF(INCX.EQ.1.AND.INCY.EQ.1)GO TO 20
C
C       CODE FOR UNEQUAL INCREMENTS OR EQUAL INCREMENTS NOT EQUAL
C         TO 1
C
      IX = 1
      IY = 1
      IF(INCX.LT.0)IX = (-N+1)*INCX + 1
      IF(INCY.LT.0)IY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        DTEMP = DX(IX)
        DX(IX) = DY(IY)
        DY(IY) = DTEMP
        IX = IX + INCX
        IY = IY + INCY
   10 CONTINUE
      RETURN
C
C       CODE FOR BOTH INCREMENTS EQUAL TO 1
C
C
C       CLEAN-UP LOOP
C
   20 M = MOD(N,3)
      IF( M .EQ. 0 ) GO TO 40
      DO 30 I = 1,M
        DTEMP = DX(I)
        DX(I) = DY(I)
        DY(I) = DTEMP
   30 CONTINUE
      IF( N .LT. 3 ) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,3
        DTEMP = DX(I)
        DX(I) = DY(I)
        DY(I) = DTEMP
        DTEMP = DX(I + 1)
        DX(I + 1) = DY(I + 1)
        DY(I + 1) = DTEMP
        DTEMP = DX(I + 2)
        DX(I + 2) = DY(I + 2)
        DY(I + 2) = DTEMP
   50 CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE  DGEMC (M, N, A, LDA, B, LDB, TRANS)
C
      IMPLICIT LOGICAL (A-Z)
C
C  This subroutine copies a double precision real 
C  M by N matrix stored in A to double precision real B.
C  If TRANS is true, B is assigned A transpose.
C
      INTEGER           M, N, LDA, LDB
      DOUBLE PRECISION  A(LDA, *), B(LDB, *)
      LOGICAL           TRANS
C
      INTEGER           I, J, MM, MMP1, MOD
      INTRINSIC         MOD
C
      IF (TRANS) THEN
          DO 100 J = 1, N
C
C         USES UNROLLED LOOPS 
C         from JACK DONGARRA, LINPACK, 3/11/78.
C
              MM = MOD(M,7)
              IF( MM .EQ. 0 ) GO TO 80
              DO 70 I = 1,MM
                  B(J,I) = A(I,J)
   70         CONTINUE
              IF( M .LT. 7 ) GOTO 99
   80         MMP1 = MM + 1
              DO 90 I = MMP1,M,7
                B(J,I) = A(I,J)
                B(J, I + 1) = A(I + 1,J)
                B(J, I + 2) = A(I + 2,J)
                B(J, I + 3) = A(I + 3,J)
                B(J, I + 4) = A(I + 4,J)
                B(J, I + 5) = A(I + 5,J)
                B(J, I + 6) = A(I + 6,J)
   90         CONTINUE
   99       CONTINUE
  100     CONTINUE
      ELSE
          DO 200 J = 1, N
C
C         USES UNROLLED LOOPS 
C         from JACK DONGARRA, LINPACK, 3/11/78.
C
              MM = MOD(M,7)
              IF( MM .EQ. 0 ) GO TO 180
              DO 170 I = 1,MM
                  B(I,J) = A(I,J)
  170         CONTINUE
              IF( M .LT. 7 ) GOTO 199
  180         MMP1 = MM + 1
              DO 190 I = MMP1,M,7
                B(I,J) = A(I,J)
                B(I + 1,J) = A(I + 1,J)
                B(I + 2,J) = A(I + 2,J)
                B(I + 3,J) = A(I + 3,J)
                B(I + 4,J) = A(I + 4,J)
                B(I + 5,J) = A(I + 5,J)
                B(I + 6,J) = A(I + 6,J)
  190         CONTINUE
  199       CONTINUE
  200     CONTINUE
      ENDIF 
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C  BLAS-2 routines needed in the computation of Floquet multipliers
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE XERBLA ( SRNAME, INFO )
C     ..    Scalar Arguments ..
      INTEGER            INFO
      CHARACTER*6        SRNAME
C  Purpose
C  XERBLA  is an error handler for the Level 2 BLAS routines.
C  It is called by the Level 2 BLAS routines if an input parameter is
C  invalid.
C  Installers should consider modifying the STOP statement in order to
C  call system-specific exception-handling facilities.
C  Parameters
C  SRNAME - CHARACTER*6.
C           On entry, SRNAME specifies the name of the routine which
C           called XERBLA.
C  INFO   - INTEGER.
C           On entry, INFO specifies the position of the invalid
C           parameter in the parameter-list of the calling routine.
C  Auxiliary routine for Level 2 Blas.
C  Written on 20-July-1986.
C     .. Executable Statements ..
      WRITE (*,99999) SRNAME, INFO
      STOP
99999 FORMAT ( ' ** ON ENTRY TO ', A6, ' PARAMETER NUMBER ', I2,
     $         ' HAD AN ILLEGAL VALUE' )
C     End of XERBLA.
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      LOGICAL FUNCTION LSAME ( CA, CB )
C     .. Scalar Arguments ..
      CHARACTER*1            CA, CB
C  Purpose
C  LSAME  tests if CA is the same letter as CB regardless of case.
C  CB is assumed to be an upper case letter. LSAME returns .TRUE. if
C  CA is either the same as CB or the equivalent lower case letter.
C  N.B. This version of the routine is only correct for ASCII code.
C       Installers must modify the routine for other character-codes.
C       For EBCDIC systems the constant IOFF must be changed to -64.
C       For CDC systems using 6-12 bit representations, the system-
C       specific code in comments must be activated.
C  Parameters
C  CA     - CHARACTER*1
C  CB     - CHARACTER*1
C           On entry, CA and CB specify characters to be compared.
C           Unchanged on exit.
C  Auxiliary routine for Level 2 Blas.
C  -- Written on 20-July-1986
C     Richard Hanson, Sandia National Labs.
C     Jeremy Du Croz, Nag Central Office.
C     .. Parameters ..
      INTEGER                IOFF
      PARAMETER            ( IOFF=32 )
C     .. Intrinsic Functions ..
      INTRINSIC              ICHAR
C     .. Executable Statements ..
C     Test if the characters are equal
      LSAME = CA .EQ. CB
C     Now test for equivalence
      IF ( .NOT.LSAME ) THEN
         LSAME = ICHAR(CA) - IOFF .EQ. ICHAR(CB)
      END IF
      RETURN
C  The following comments contain code for CDC systems using 6-12 bit
C  representations.
C     .. Parameters ..
C     INTEGER                ICIRFX
C     PARAMETER            ( ICIRFX=62 )
C     .. Scalar Arguments ..
C     CHARACTER*1            CB
C     .. Array Arguments ..
C     CHARACTER*1            CA(*)
C     .. Local Scalars ..
C     INTEGER                IVAL
C     .. Intrinsic Functions ..
C     INTRINSIC              ICHAR, CHAR
C     .. Executable Statements ..
C     See if the first character in string CA equals string CB.
C     LSAME = CA(1) .EQ. CB .AND. CA(1) .NE. CHAR(ICIRFX)
C     IF (LSAME) RETURN
C     The characters are not identical. Now check them for equivalence.
C     Look for the 'escape' character, circumflex, followed by the
C     letter.
C     IVAL = ICHAR(CA(2))
C     IF (IVAL.GE.ICHAR('A') .AND. IVAL.LE.ICHAR('Z')) THEN
C        LSAME = CA(1) .EQ. CHAR(ICIRFX) .AND. CA(2) .EQ. CB
C     END IF
C     RETURN
C     End of LSAME.
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C  BLAS-3 routines needed in the computation of Floquet multipliers
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE DGEMM ( TRANSA, TRANSB, M, N, K, ALPHA, A, LDA, B, LDB,
     $                   BETA, C, LDC )
C     .. Scalar Arguments ..
      CHARACTER*1        TRANSA, TRANSB
      INTEGER            M, N, K, LDA, LDB, LDC
      DOUBLE PRECISION   ALPHA, BETA
C     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), B( LDB, * ), C( LDC, * )
C  Purpose
C  DGEMM  performs one of the matrix-matrix operations
C     C := alpha*op( A )*op( B ) + beta*C,
C  where  op( X ) is one of
C     op( X ) = X   or   op( X ) = X',
C  alpha and beta are scalars, and A, B and C are matrices, with op( A )
C  an m by k matrix,  op( B )  a  k by n matrix and  C an m by n matrix.
C  Parameters
C  TRANSA - CHARACTER*1.
C           On entry, TRANSA specifies the form of op( A ) to be used in
C           the matrix multiplication as follows:
C              TRANSA = 'N' or 'n',  op( A ) = A.
C              TRANSA = 'T' or 't',  op( A ) = A'.
C              TRANSA = 'C' or 'c',  op( A ) = A'.
C           Unchanged on exit.
C  TRANSB - CHARACTER*1.
C           On entry, TRANSB specifies the form of op( B ) to be used in
C           the matrix multiplication as follows:
C              TRANSB = 'N' or 'n',  op( B ) = B.
C              TRANSB = 'T' or 't',  op( B ) = B'.
C              TRANSB = 'C' or 'c',  op( B ) = B'.
C           Unchanged on exit.
C  M      - INTEGER.
C           On entry,  M  specifies  the number  of rows  of the  matrix
C           op( A )  and of the  matrix  C.  M  must  be at least  zero.
C           Unchanged on exit.
C  N      - INTEGER.
C           On entry,  N  specifies the number  of columns of the matrix
C           op( B ) and the number of columns of the matrix C. N must be
C           at least zero.
C           Unchanged on exit.
C  K      - INTEGER.
C           On entry,  K  specifies  the number of columns of the matrix
C           op( A ) and the number of rows of the matrix op( B ). K must
C           be at least  zero.
C           Unchanged on exit.
C  ALPHA  - DOUBLE PRECISION.
C           On entry, ALPHA specifies the scalar alpha.
C           Unchanged on exit.
C  A      - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
C           k  when  TRANSA = 'N' or 'n',  and is  m  otherwise.
C           Before entry with  TRANSA = 'N' or 'n',  the leading  m by k
C           part of the array  A  must contain the matrix  A,  otherwise
C           the leading  k by m  part of the array  A  must contain  the
C           matrix A.
C           Unchanged on exit.
C  LDA    - INTEGER.
C           On entry, LDA specifies the first dimension of A as declared
C           in the calling (sub) program. When  TRANSA = 'N' or 'n' then
C           LDA must be at least  max( 1, m ), otherwise  LDA must be at
C           least  max( 1, k ).
C           Unchanged on exit.
C  B      - DOUBLE PRECISION array of DIMENSION ( LDB, kb ), where kb is
C           n  when  TRANSB = 'N' or 'n',  and is  k  otherwise.
C           Before entry with  TRANSB = 'N' or 'n',  the leading  k by n
C           part of the array  B  must contain the matrix  B,  otherwise
C           the leading  n by k  part of the array  B  must contain  the
C           matrix B.
C           Unchanged on exit.
C  LDB    - INTEGER.
C           On entry, LDB specifies the first dimension of B as declared
C           in the calling (sub) program. When  TRANSB = 'N' or 'n' then
C           LDB must be at least  max( 1, k ), otherwise  LDB must be at
C           least  max( 1, n ).
C           Unchanged on exit.
C  BETA   - DOUBLE PRECISION.
C           On entry,  BETA  specifies the scalar  beta.  When  BETA  is
C           supplied as zero then C need not be set on input.
C           Unchanged on exit.
C  C      - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
C           Before entry, the leading  m by n  part of the array  C must
C           contain the matrix  C,  except when  beta  is zero, in which
C           case C need not be set on entry.
C           On exit, the array  C  is overwritten by the  m by n  matrix
C           ( alpha*op( A )*op( B ) + beta*C ).
C  LDC    - INTEGER.
C           On entry, LDC specifies the first dimension of C as declared
C           in  the  calling  (sub)  program.   LDC  must  be  at  least
C           max( 1, m ).
C           Unchanged on exit.
C  Level 3 Blas routine.
C  -- Written on 8-February-1989.
C     Jack Dongarra, Argonne National Laboratory.
C     Iain Duff, AERE Harwell.
C     Jeremy Du Croz, Numerical Algorithms Group Ltd.
C     Sven Hammarling, Numerical Algorithms Group Ltd.
C     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
C     .. External Subroutines ..
      EXTERNAL           XERBLA
C     .. Intrinsic Functions ..
      INTRINSIC          MAX
C     .. Local Scalars ..
      LOGICAL            NOTA, NOTB
      INTEGER            I, INFO, J, L, NCOLA, NROWA, NROWB
      DOUBLE PRECISION   TEMP
C     .. Parameters ..
      DOUBLE PRECISION   ONE         , ZERO
      PARAMETER        ( ONE = 1.0D+0, ZERO = 0.0D+0 )
C     .. Executable Statements ..
C     Set  NOTA  and  NOTB  as  true if  A  and  B  respectively are not
C     transposed and set  NROWA, NCOLA and  NROWB  as the number of rows
C     and  columns of  A  and the  number of  rows  of  B  respectively.
      NOTA  = LSAME( TRANSA, 'N' )
      NOTB  = LSAME( TRANSB, 'N' )
      IF( NOTA )THEN
         NROWA = M
         NCOLA = K
      ELSE
         NROWA = K
         NCOLA = M
      END IF
      IF( NOTB )THEN
         NROWB = K
      ELSE
         NROWB = N
      END IF
C     Test the input parameters.
      INFO = 0
      IF(      ( .NOT.NOTA                 ).AND.
     $         ( .NOT.LSAME( TRANSA, 'C' ) ).AND.
     $         ( .NOT.LSAME( TRANSA, 'T' ) )      )THEN
         INFO = 1
      ELSE IF( ( .NOT.NOTB                 ).AND.
     $         ( .NOT.LSAME( TRANSB, 'C' ) ).AND.
     $         ( .NOT.LSAME( TRANSB, 'T' ) )      )THEN
         INFO = 2
      ELSE IF( M  .LT.0               )THEN
         INFO = 3
      ELSE IF( N  .LT.0               )THEN
         INFO = 4
      ELSE IF( K  .LT.0               )THEN
         INFO = 5
      ELSE IF( LDA.LT.MAX( 1, NROWA ) )THEN
         INFO = 8
      ELSE IF( LDB.LT.MAX( 1, NROWB ) )THEN
         INFO = 10
      ELSE IF( LDC.LT.MAX( 1, M     ) )THEN
         INFO = 13
      END IF
      IF( INFO.NE.0 )THEN
         CALL XERBLA( 'DGEMM ', INFO )
         RETURN
      END IF
C     Quick return if possible.
      IF( ( M.EQ.0 ).OR.( N.EQ.0 ).OR.
     $    ( ( ( ALPHA.EQ.ZERO ).OR.( K.EQ.0 ) ).AND.( BETA.EQ.ONE ) ) )
     $   RETURN
C     And if  alpha.eq.zero.
      IF( ALPHA.EQ.ZERO )THEN
         IF( BETA.EQ.ZERO )THEN
            DO 20, J = 1, N
               DO 10, I = 1, M
                  C( I, J ) = ZERO
   10          CONTINUE
   20       CONTINUE
         ELSE
            DO 40, J = 1, N
               DO 30, I = 1, M
                  C( I, J ) = BETA*C( I, J )
   30          CONTINUE
   40       CONTINUE
         END IF
         RETURN
      END IF
C     Start the operations.
      IF( NOTB )THEN
         IF( NOTA )THEN
C           Form  C := alpha*A*B + beta*C.
            DO 90, J = 1, N
               IF( BETA.EQ.ZERO )THEN
                  DO 50, I = 1, M
                     C( I, J ) = ZERO
   50             CONTINUE
               ELSE IF( BETA.NE.ONE )THEN
                  DO 60, I = 1, M
                     C( I, J ) = BETA*C( I, J )
   60             CONTINUE
               END IF
               DO 80, L = 1, K
                  IF( B( L, J ).NE.ZERO )THEN
                     TEMP = ALPHA*B( L, J )
                     DO 70, I = 1, M
                        C( I, J ) = C( I, J ) + TEMP*A( I, L )
   70                CONTINUE
                  END IF
   80          CONTINUE
   90       CONTINUE
         ELSE
C           Form  C := alpha*A'*B + beta*C
            DO 120, J = 1, N
               DO 110, I = 1, M
                  TEMP = ZERO
                  DO 100, L = 1, K
                     TEMP = TEMP + A( L, I )*B( L, J )
  100             CONTINUE
                  IF( BETA.EQ.ZERO )THEN
                     C( I, J ) = ALPHA*TEMP
                  ELSE
                     C( I, J ) = ALPHA*TEMP + BETA*C( I, J )
                  END IF
  110          CONTINUE
  120       CONTINUE
         END IF
      ELSE
         IF( NOTA )THEN
C           Form  C := alpha*A*B' + beta*C
            DO 170, J = 1, N
               IF( BETA.EQ.ZERO )THEN
                  DO 130, I = 1, M
                     C( I, J ) = ZERO
  130             CONTINUE
               ELSE IF( BETA.NE.ONE )THEN
                  DO 140, I = 1, M
                     C( I, J ) = BETA*C( I, J )
  140             CONTINUE
               END IF
               DO 160, L = 1, K
                  IF( B( J, L ).NE.ZERO )THEN
                     TEMP = ALPHA*B( J, L )
                     DO 150, I = 1, M
                        C( I, J ) = C( I, J ) + TEMP*A( I, L )
  150                CONTINUE
                  END IF
  160          CONTINUE
  170       CONTINUE
         ELSE
C           Form  C := alpha*A'*B' + beta*C
            DO 200, J = 1, N
               DO 190, I = 1, M
                  TEMP = ZERO
                  DO 180, L = 1, K
                     TEMP = TEMP + A( L, I )*B( J, L )
  180             CONTINUE
                  IF( BETA.EQ.ZERO )THEN
                     C( I, J ) = ALPHA*TEMP
                  ELSE
                     C( I, J ) = ALPHA*TEMP + BETA*C( I, J )
                  END IF
  190          CONTINUE
  200       CONTINUE
         END IF
      END IF
      RETURN
C     End of DGEMM .
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C Demmel-Kahan SVD routines needed for computing the Floquet multipliers
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE EZSVD(X,LDX,N,P,S,E,U,LDU,V,LDV,WORK,JOB,INFO,TOL)
C
      INTEGER LDX,N,P,LDU,LDV,JOB,INFO,SKIP
      DOUBLE PRECISION X(LDX,1),S(1),E(1),U(LDU,1),V(LDV,1),WORK(1)
      DOUBLE PRECISION MAXSIN,TOL
      INTEGER IDBG, IFULL, IIDIR, MAXITR, LIMSHF, KOUNT,
     &        KOUNT1, KOUNT2
C
C     new svd by J. Demmel, W. Kahan
C     finds singular values of bidiagonal matrices with guaranteed high
C     relative precision
C
C     easy to use version of ndsvd ("hard to use" version, below)
C     with defaults for some ndsvd parameters
C
C     all parameters same as linpack dsvdc except for tol:
C
C     tol  = if positive, desired relative precision in singular values
C            if negative, desired absolute precision in singular values
C               (expressed as abs(tol) * sigma-max)
C            (in both cases, abs(tol) should be less than 1 and 
C             greater than macheps)
C
C        I have tested this software on a SUN 3 in double precision
C        IEEE arithmetic with macheps about 2.2e-16 and tol=1e-14; 
C        In general I recommend tol 10-100 times larger than macheps.
C
C        On the average it appears to be as fast or faster than dsvdc.
C        I have seen it go 3.5 times faster and 2 times slower at the 
C        extremes.
C
C     defaults for ndsvd parameters (see ndsvd for more description of
C     these parameters) are:
C
C     set to no debug output
      IDBG = 0
C     use zero-shift normally
      IFULL = 0     
C     use normal bidiagonalization code
      SKIP = 0     
C     choose chase direction normally
      IIDIR = 0
C     maximum 30 QR sweeps per singular value
      MAXITR = 30
C
      CALL NDSVD(X,LDX,N,P,S,E,U,LDU,V,LDV,WORK,JOB,INFO,MAXITR,
     *                  TOL,IDBG,IFULL,KOUNT,KOUNT1,KOUNT2,SKIP,LIMSHF,
     *                  MAXSIN,IIDIR)
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE NDROTG(F,G,CS,SN)
C     faster version of drotg, except g unchanged on return
C     cs, sn returned so that -sn*f+cs*g = 0
C     and returned f = cs*f + sn*g
C    
C     if g=0, then cs=1 and sn=0 (in case svd adds extra zero row
C         to bidiagonal, this makes sure last row rotation is trivial)
C
C     if f=0 and g.ne.0, then cs=0 and sn=1 without floating point work
C         (in case s(i)=0 in svd so that bidiagonal deflates, this
C          computes rotation without any floating point operations)
C
      DOUBLE PRECISION F, G, CS, SN, T, TT
      IF (F.EQ.0.0D0) THEN
        IF (G.EQ.0.0D0) THEN
C         this case needed in case extra zero row added in svd, so
C         bottom rotation always trivial
          CS = 1.0D0
          SN = 0.0D0
        ELSE
C         this case needed for s(i)=0 in svd to compute rotation
C         cheaply
          CS = 0.0D0
          SN = 1.0D0
          F = G
        ENDIF
      ELSE
        IF (ABS(F) .GT. ABS(G)) THEN
          T = G/F
          TT = DSQRT(1.0D0 + T*T )
          CS = 1.0D0/TT
          SN = T*CS
          F = F*TT
        ELSE
          T = F/G
          TT = DSQRT(1.0D0 + T*T )
          SN = 1.0D0/TT
          CS = T*SN
          F = G*TT
        ENDIF
      ENDIF
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE NDSVD(X,LDX,N,P,S,E,U,LDU,V,LDV,WORK,JOB,INFO,MAXITR,
     *                  TOL,IDBG,IFULL,KOUNT,KOUNT1,KOUNT2,SKIP,LIMSHF,
     *                  MAXSIN,IIDIR)
C
C     LINPACK SVD modified by:
C     James Demmel                      W. Kahan
C     Courant Institute                 Computer Science Dept.
C     demmel@acf8.nyu.edu               U.C. Berkeley
C
C     modified version designed to guarantee relative accuracy of
C     all singular values of intermediate bidiagonal form
C
C     extra input/output parameters in addition to those from LINPACK SVD:
C
C     extra input paramters:
C
C     tol  = if positive, desired relative precision in singular values
C            if negative, desired absolute precision in singular values
C               (expressed as abs(tol) * sigma-max)
C            (abs(tol) should be less than 1 and greater than macheps)
C
C     idbg = 0 for no debug output (normal setting)
C          = 1 convergence, shift decisions (written to standard output)
C          = 2 for above plus before, after qr
C
C     ifull= 0 if decision to use zero-shift set normally (normal setting)
C          = 1 if always set to nonzero-shift
C          = 2 if always set to zero-shift
C
C     skip =-1 means standard code but do all work of bidiagonalization
C              (even if input bidiagonal)
C            0 means standard code (normal setting)
C            1 means assume x is bidiagonal, and skip bidiagonalization
C              entirely
C          (skip used for timing tests)
C
C     iidir = 0 if idir (chase direction) chosen normally
C             1 if idir=1 (chase top to bottom) always
C             2 if idir=2 (chase bottom to top) always
C
C     extra output parameters:
C
C     kount =number of qr sweeps taken
C
C     kount1=number of passes through inner loop of full qr
C
C     kount2=number of passes through inner loop of zero-shift qr
C
C     limshf = number of times the shift was greater than its threshold
C              (nct*smin) and had to be decreased
C
C     maxsin = maximum sin in inner loop of zero-shift
C
      INTEGER LDX,N,P,LDU,LDV,JOB,INFO,SKIP
      INTEGER IDBG, IFULL, IIDIR, MAXITR, LIMSHF, KOUNT,
     &        KOUNT1, KOUNT2
      DOUBLE PRECISION X(LDX,1),S(1),E(1),U(LDU,1),V(LDV,1),WORK(1)
      DOUBLE PRECISION MAXSIN
C
C     new version designed to be robust with respect to over/underflow
C     have fast inner loop when shift is zero,
C     guarantee relative accuracy of all singular values
C
C     dsvdc is a subroutine to reduce a double precision nxp matrix x
C     by orthogonal transformations u and v to diagonal form.  the
C     diagonal elements s(i) are the singular values of x.  the
C     columns of u are the corresponding left singular vectors,
C     and the columns of v the right singular vectors.
C
C     on entry
C
C         x         double precision(ldx,p), where ldx.ge.n.
C                   x contains the matrix whose singular value
C                   decomposition is to be computed.  x is
C                   destroyed by dsvdc.
C
C         ldx       integer.
C                   ldx is the leading dimension of the array x.
C
C         n         integer.
C                   n is the number of rows of the matrix x.
C
C         p         integer.
C                   p is the number of columns of the matrix x.
C
C         ldu       integer.
C                   ldu is the leading dimension of the array u.
C                   (see below).
C
C         ldv       integer.
C                   ldv is the leading dimension of the array v.
C                   (see below).
C
C         work      double precision(n).
C                   work is a scratch array.
C
C         job       integer.
C                   job controls the computation of the singular
C                   vectors.  it has the decimal expansion ab
C                   with the following meaning
C
C                        a.eq.0    do not compute the left singular
C                                  vectors.
C                        a.eq.1    return the n left singular vectors
C                                  in u.
C                        a.ge.2    return the first min(n,p) singular
C                                  vectors in u.
C                        b.eq.0    do not compute the right singular
C                                  vectors.
C                        b.eq.1    return the right singular vectors
C                                  in v.
C
C     on return
C
C         s         double precision(mm), where mm=min(n+1,p).
C                   the first min(n,p) entries of s contain the
C                   singular values of x arranged in descending
C                   order of magnitude.
C
C         e         double precision(p), 
C                   e ordinarily contains zeros.  however see the
C                   discussion of info for exceptions.
C
C         u         double precision(ldu,k), where ldu.ge.n.  if
C                                   joba.eq.1 then k.eq.n, if joba.ge.2
C                                   then k.eq.min(n,p).
C                   u contains the matrix of left singular vectors.
C                   u is not referenced if joba.eq.0.  if n.le.p
C                   or if joba.eq.2, then u may be identified with x
C                   in the subroutine call.
C
C         v         double precision(ldv,p), where ldv.ge.p.
C                   v contains the matrix of right singular vectors.
C                   v is not referenced if job.eq.0.  if p.le.n,
C                   then v may be identified with x in the
C                   subroutine call.
C
C         info      integer.
C                   the singular values (and their corresponding
C                   singular vectors) s(info+1),s(info+2),...,s(m)
C                   are correct (here m=min(n,p)).  thus if
C                   info.eq.0, all the singular values and their
C                   vectors are correct.  in any event, the matrix
C                   b = trans(u)*x*v is the bidiagonal matrix
C                   with the elements of s on its diagonal and the
C                   elements of e on its super-diagonal (trans(u)
C                   is the transpose of u).  thus the singular
C                   values of x and b are the same.
C
C     linpack. this version dated 08/14/78 .
C              correction made to shift 2/84.
C     g.w. stewart, university of maryland, argonne national lab.
C
C     dsvdc uses the following functions and subprograms.
C
C     external drot
C     blas daxpy,ddot,dscal,dswap,dnrm2,drotg
C     fortran dabs,dmax1,max0,min0,mod,dsqrt,dsign
C     prse,ndrotg,sig22,sndrtg,sigmin
C
C     internal variables
C
      INTEGER I,ITER,J,JOBU,L,LL,LM1,LP1,LU,M,MAXIT,
     *        NCT,NCTP1,NCU,NRT,NRTP1
      DOUBLE PRECISION DDOT,T
      DOUBLE PRECISION CS,F,G,DNRM2,SHIFT,SN,
     *                 TEST
      LOGICAL WANTU,WANTV
C
C     new variables
      DOUBLE PRECISION TOL,SMIN,SMAX,ABSS,ABSE,TEMP,SIGMIN,SMINL,GG
      DOUBLE PRECISION SMM1,SM,EMM1,SLL,OLDCS,OLDSN,THRESH
C     double precision sg1,sg2
      DOUBLE PRECISION SIGMN,SIGMX,SINR,COSR,SINL,COSL
      DOUBLE PRECISION MU,LAMBDA
      INTEGER OLDLL,OLDM,OLDACC, MINNP, MM, LLL, IDIR, K, IISUB
C
C
C     set the maximum number of iterations.
C
C     maxit = 30
      KOUNT=0
      KOUNT1=0
      KOUNT2=0
      LIMSHF=0
      MAXSIN=0.
C
C     determine what is to be computed.
C
      WANTU = .FALSE.
      WANTV = .FALSE.
      JOBU = MOD(JOB,100)/10
      NCU = N
      IF (JOBU .GT. 1) NCU = MIN0(N,P)
      IF (JOBU .NE. 0) WANTU = .TRUE.
      IF (MOD(JOB,10) .NE. 0) WANTV = .TRUE.
C
C     reduce x to bidiagonal form, storing the diagonal elements
C     in s and the super-diagonal elements in e.
C
      INFO = 0
      NCT = MIN0(N-1,P)
      NRT = MAX0(0,MIN0(P-2,N))
      LU = MAX0(NCT,NRT)
      IF (SKIP.LE.0) THEN
      IF (LU .LT. 1) GO TO 170
      DO 160 L = 1, LU
         LP1 = L + 1
         IF (L .GT. NCT) GO TO 20
C
C           compute the transformation for the l-th column and
C           place the l-th diagonal in s(l).
C
            S(L) = DNRM2(N-L+1,X(L,L),1)
            IF (S(L) .EQ. 0.0D0 .AND. SKIP.EQ.0) GO TO 10
               IF (X(L,L) .NE. 0.0D0) S(L) = DSIGN(S(L),X(L,L))
               CALL DSCAL(N-L+1,1.0D0/S(L),X(L,L),1)
               X(L,L) = 1.0D0 + X(L,L)
   10       CONTINUE
            S(L) = -S(L)
   20    CONTINUE
         IF (P .LT. LP1) GO TO 50
         DO 40 J = LP1, P
            IF (L .GT. NCT) GO TO 30
            IF (S(L) .EQ. 0.0D0 .AND. SKIP.EQ.0) GO TO 30
C
C              apply the transformation.
C
               T = -DDOT(N-L+1,X(L,L),1,X(L,J),1)/X(L,L)
               CALL DAXPY(N-L+1,T,X(L,L),1,X(L,J),1)
   30       CONTINUE
C
C           place the l-th row of x into  e for the
C           subsequent calculation of the row transformation.
C
            E(J) = X(L,J)
   40    CONTINUE
   50    CONTINUE
         IF (.NOT.WANTU .OR. L .GT. NCT) GO TO 70
C
C           place the transformation in u for subsequent back
C           multiplication.
C
            DO 60 I = L, N
               U(I,L) = X(I,L)
   60       CONTINUE
   70    CONTINUE
         IF (L .GT. NRT) GO TO 150
C
C           compute the l-th row transformation and place the
C           l-th super-diagonal in e(l).
C
            E(L) = DNRM2(P-L,E(LP1),1)
            IF (E(L) .EQ. 0.0D0 .AND. SKIP.EQ.0) GO TO 80
               IF (E(LP1) .NE. 0.0D0) E(L) = DSIGN(E(L),E(LP1))
               CALL DSCAL(P-L,1.0D0/E(L),E(LP1),1)
               E(LP1) = 1.0D0 + E(LP1)
   80       CONTINUE
            E(L) = -E(L)
            IF (LP1 .GT. N .OR. (E(L) .EQ. 0.0D0 .AND. SKIP.EQ.0)) 
     +          GO TO 120
C
C              apply the transformation.
C
               DO 90 I = LP1, N
                  WORK(I) = 0.0D0
   90          CONTINUE
               DO 100 J = LP1, P
                  CALL DAXPY(N-L,E(J),X(LP1,J),1,WORK(LP1),1)
  100          CONTINUE
               DO 110 J = LP1, P
                  CALL DAXPY(N-L,-E(J)/E(LP1),WORK(LP1),1,X(LP1,J),1)
  110          CONTINUE
  120       CONTINUE
            IF (.NOT.WANTV) GO TO 140
C
C              place the transformation in v for subsequent
C              back multiplication.
C
               DO 130 I = LP1, P
                  V(I,L) = E(I)
  130          CONTINUE
  140       CONTINUE
  150    CONTINUE
  160 CONTINUE
  170 CONTINUE
      ENDIF
C
C     set up the final bidiagonal matrix or order m.
C
      M = MIN0(P,N+1)
      NCTP1 = NCT + 1
      NRTP1 = NRT + 1
      IF (SKIP.LE.0) THEN
      IF (NCT .LT. P) S(NCTP1) = X(NCTP1,NCTP1)
      IF (N .LT. M) S(M) = 0.0D0
      IF (NRTP1 .LT. M) E(NRTP1) = X(NRTP1,M)
      E(M) = 0.0D0
C
C     if required, generate u.
C
      IF (.NOT.WANTU) GO TO 300
         IF (NCU .LT. NCTP1) GO TO 200
         DO 190 J = NCTP1, NCU
            DO 180 I = 1, N
               U(I,J) = 0.0D0
  180       CONTINUE
            U(J,J) = 1.0D0
  190    CONTINUE
  200    CONTINUE
         IF (NCT .LT. 1) GO TO 290
         DO 280 LL = 1, NCT
            L = NCT - LL + 1
            IF (S(L) .EQ. 0.0D0) GO TO 250
               LP1 = L + 1
               IF (NCU .LT. LP1) GO TO 220
               DO 210 J = LP1, NCU
                  T = -DDOT(N-L+1,U(L,L),1,U(L,J),1)/U(L,L)
                  CALL DAXPY(N-L+1,T,U(L,L),1,U(L,J),1)
  210          CONTINUE
  220          CONTINUE
               CALL DSCAL(N-L+1,-1.0D0,U(L,L),1)
               U(L,L) = 1.0D0 + U(L,L)
               LM1 = L - 1
               IF (LM1 .LT. 1) GO TO 240
               DO 230 I = 1, LM1
                  U(I,L) = 0.0D0
  230          CONTINUE
  240          CONTINUE
            GO TO 270
  250       CONTINUE
               DO 260 I = 1, N
                  U(I,L) = 0.0D0
  260          CONTINUE
               U(L,L) = 1.0D0
  270       CONTINUE
  280    CONTINUE
  290    CONTINUE
  300 CONTINUE
C
C     if it is required, generate v.
C
      IF (.NOT.WANTV) GO TO 350
         DO 340 LL = 1, P
            L = P - LL + 1
            LP1 = L + 1
            IF (L .GT. NRT) GO TO 320
            IF (E(L) .EQ. 0.0D0) GO TO 320
               DO 310 J = LP1, P
                  T = -DDOT(P-L,V(LP1,L),1,V(LP1,J),1)/V(LP1,L)
                  CALL DAXPY(P-L,T,V(LP1,L),1,V(LP1,J),1)
  310          CONTINUE
  320       CONTINUE
            DO 330 I = 1, P
               V(I,L) = 0.0D0
  330       CONTINUE
            V(L,L) = 1.0D0
  340    CONTINUE
  350 CONTINUE
      ENDIF
C
C
      IF (SKIP.EQ.1) THEN
C       set up s,e,u,v assuming x bidiagonal on input
        MINNP=MIN(N,P)
        DO 351 I=1,MINNP
          S(I)=X(I,I)
          IF (I.LT.P) E(I)=X(I,I+1)
351     CONTINUE
        IF (N.LT.P) S(N+1)=0.
        E(M)=0
        IF (WANTU) THEN
          DO 352 J=1,NCU
            DO 353 I=1,N
              U(I,J)=0.
353         CONTINUE
            U(J,J)=1.D0
352       CONTINUE
        ENDIF
        IF (WANTV) THEN
          DO 354 J=1,P
            DO 355 I=1,P
              V(I,J)=0.
355         CONTINUE
            V(J,J)=1.D0
354       CONTINUE
        ENDIF
      ENDIF
C
C     main iteration loop for the singular values.
C
C     convert maxit to bound on total number of passes through
C     inner loops of qr iteration (half number of rotations)
      MAXIT = MAXITR*M*M/2
      ITER=0
      OLDLL = -1
      OLDM = -1
      OLDACC = -1
      IF (TOL.GT.0.0) THEN
C       relative accuracy desired
        THRESH = 0.0D0
      ELSE
C       absolute accuracy desired
        SMAX = ABS(S(M))
        DO 1111 I=1,M-1
          SMAX = MAX(SMAX,ABS(S(I)),ABS(E(I)))
1111    CONTINUE
        THRESH = ABS(TOL)*SMAX
      ENDIF
      MM=M
C
C     begin loop
999   CONTINUE
      IF (IDBG.GT.0) THEN
        PRINT *,'TOP OF LOOP'
        PRINT *,'OLDLL,OLDM,OLDACC,M,ITER,MAXIT,IFULL,THRESH=',
     *  OLDLL,OLDM,OLDACC,M,ITER,MAXIT,IFULL,THRESH
        CALL PRSE(1,MM,N,P,S,E)
      ENDIF
C
C     check for being done
      IF (M.EQ.1) GOTO 998
C
C     check number of iterations
      IF (ITER.GE.MAXIT) GOTO 997
C
C     compute minimum s(i) and max of all s(i),e(i)
      IF (TOL.LE.0. .AND. ABS(S(M)).LE.THRESH) S(M) = 0.0
      SMAX = ABS(S(M))
      SMIN = SMAX
C
C     reset convergence threshold if starting new part of matrix
      IF (M.LE.OLDLL .AND. TOL.GT.0.0) THRESH = 0.0D0
      IF (IDBG.GT.0) PRINT *,'THRESH=',THRESH
      DO 1001 LLL=1,M
        LL=M-LLL
        IF (LL.EQ.0) GOTO 1003
        IF (TOL.LE.0. .AND. ABS(S(LL)).LE.THRESH) S(LL) = 0.0
        IF (ABS(E(LL)).LE.THRESH) GOTO 1002
        ABSS = ABS(S(LL))
        ABSE = ABS(E(LL))
        SMIN = MIN(SMIN,ABSS)
        SMAX = MAX(SMAX,ABSS,ABSE)
1001  CONTINUE
1002  CONTINUE
      E(LL) = 0.0D0
C
C     matrix splits since e(ll)=0
      IF (LL.EQ.M-1) THEN
C       convergence of bottom singular values
        M=M-1
        IF (IDBG.GT.0) PRINT *, 'CONVERGENCE'
        GOTO 999
      ENDIF
1003  CONTINUE
      LL=LL+1
C     e(ll) ... e(m-1) are nonzero
      IF (IDBG.GT.0) THEN
        PRINT *,'WORK ON BLOCK LL,M=',LL,M
        PRINT *,'SMIN=',SMIN
        PRINT *,'SMAX=',SMAX
      ENDIF
C
C     2 by 2 block - handle specially to guarantee convergence
      IF (LL.EQ.M-1) THEN
C       after one step
        KOUNT1 = KOUNT1 +1
C       shift = sigmin(s(m-1),e(m-1),s(m))
C       rotate, setting e(m-1)=0 and s(m)=+-shift
C       if (s(ll).eq.0.0d0) then
C         f = 0.0d0
C       else
C         f = (abs(s(ll)) - shift)*(dsign(1.0d0,s(ll))+shift/s(ll))
C       endif
C       g=e(ll)
C       call ndrotg(f,g,cs,sn)
C       sg1=dsign(1.0d0,s(m))
C       sg2=dsign(1.0d0,cs)
C       f = cs*s(ll) + sn*e(ll)
C       g = sn*s(m)
C       if (idbg.gt.0) then
C         abss = cs*s(m)
C         abse = -sn*s(ll) + cs*e(ll)
C       endif
C       if (wantv) call drot(p,v(1,ll),1,v(1,m),1,cs,sn)
C       call ndrotg(f,g,cs,sn)
C       s(ll)=f
C       if (wantu.and.ll.lt.n) call drot(n,u(1,ll),1,u(1,m),1,cs,sn)
C       e(ll) = 0.0d0
C       s(m) = shift * dsign(1.0d0,cs) * sg1 * sg2
C       if (idbg.gt.0) then
C         print *,'2 by 2 block'
C         print *,'shift=',shift
C         print *,'check shift=',-sn*abse+cs*abss
C         print *,'check zero=',cs*abse+sn*abss
C       endif
        CALL SIG22(S(M-1),E(M-1),S(M),SIGMN,SIGMX,SINR,COSR,SINL,COSL)
        S(M-1)=SIGMX
        E(M-1)=0.
        S(M)=SIGMN
        IF (WANTV) CALL DROT(P,V(1,LL),1,V(1,M),1,COSR,SINR)
C       if wantu and ll.eq.n, then rotation trivial
        IF (WANTU.AND.LL.LT.N) CALL DROT(N,U(1,LL),1,U(1,M),1,COSL,SINL)
        GOTO 999
      ENDIF
C
C     choose shift direction if new submatrix
C     if (ll.ne.oldll .or. m.ne.oldm) then
C     choose shift direction if working on entirely new submatrix
      IF (LL.GT.OLDM .OR. M.LT.OLDLL) THEN
        IF ((ABS(S(LL)).GE.ABS(S(M)) .AND. IIDIR.EQ.0) .OR. IIDIR.EQ.1) 
     *    THEN
C         chase bulge from top (big end) to bottom (small end)
C         if m=n+1, chase from top to bottom even if s(ll)=0
          IDIR=1
        ELSE
C         chase bulge from bottom (big end) to top (small end)
          IDIR=2
        ENDIF
      ENDIF
      IF (IDBG.GT.0) PRINT *,'IDIR=',IDIR
C
C     compute lower bound on smallest singular value
C     if old lower bound still good, do not recompute it
C     if (ll.ne.oldll .or. m.ne.oldm .or. oldacc.ne.1) then
C       compute lower bound
C       sminl = smin
C       oldacc = 1
C       if (sminl.gt.0.0d0) then
C         if (idir.eq.1) then
C           do 1004 lll=ll,m-1
C             abse = abs(e(lll))
C             abss = abs(s(lll))
C             if (abss.lt.abse) then
C               sminl = sminl * (abss/abse)
C               oldacc = -1
C             endif
1004        CONTINUE
C         else
C           do 1005 lll=ll,m-1
C             abse = abs(e(lll))
C             abss = abs(s(lll+1))
C             if (abss.lt.abse) then
C               sminl = sminl * (abss/abse)
C               oldacc = -1
C             endif
1005        CONTINUE
C         endif
C       endif
C       oldll = ll
C       oldm = m
C       sminl is lower bound on smallest singular value
C       within a factor of sqrt(m*(m+1)/2)
C       if oldacc = 1 as well, sminl is also upper bound
C       note that smin is always an upper bound
C
C       compute convergence threshold
C       thresh = tol*sminl
C     endif
C     if (idbg.gt.0) then
C       print *,'oldll,oldm,oldacc=',oldll,oldm,oldacc
C       print *,'sminl=',sminl
C       print *,'thresh=',thresh
C     endif
C
C     test again for convergence using new thresh
C     iconv = 0
C     do 1014 lll=ll,m-1
C       if (dabs(e(lll)).le.thresh) then
C         e(lll) = 0.0d0
C         iconv = 1
C       endif
1014  CONTINUE
C     if (iconv.eq.1) goto 999
C   
C     Kahan's convergence test
      SMINL = 0.
      IF (TOL.GT.0.0) THEN
        IF (IDIR.EQ.1) THEN
C         forward direction
C         apply test on bottom 2 by 2 only
          IF (DABS(E(M-1)).LE.TOL*DABS(S(M))) THEN
C           convergence of bottom element
            E(M-1) = 0
            GOTO 999
          ENDIF
C         apply test in forward direction
          MU = DABS(S(LL))
          SMINL = MU
          DO 3330 LLL=LL,M-1
            IF (DABS(E(LLL)).LE.TOL*MU) THEN
C             test for negligibility satisfied
              IF (IDBG.GE.1) PRINT *,'KNEW: E(LLL),MU=',E(LLL),MU
              E(LLL) = 0
              GOTO 999
            ELSE
              MU = DABS(S(LLL+1))*(MU/(MU+DABS(E(LLL))))
            ENDIF
            SMINL = MIN(SMINL,MU)
3330      CONTINUE
        ELSE
C         idir=2,  backwards direction
C         apply test on top 2 by 2 only
          IF (DABS(E(LL)).LE.TOL*DABS(S(LL))) THEN
C           convergence of top element
            E(LL) = 0
            GOTO 999
          ENDIF
C         apply test in backward direction
          LAMBDA = DABS(S(M))
          SMINL = LAMBDA
          DO 3331 LLL=M-1,LL,-1
            IF (DABS(E(LLL)).LE.TOL*LAMBDA) THEN
C             test for negligibility satisfied
              IF (IDBG.GE.1) PRINT *,'KNEW: E(LLL),LAMBDA=',E(LLL),
     +                               LAMBDA
              E(LLL) = 0
              GOTO 999
            ELSE
              LAMBDA = DABS(S(LLL))*(LAMBDA/(LAMBDA+DABS(E(LLL))))
            ENDIF
            SMINL = MIN(SMINL,LAMBDA)
3331      CONTINUE
        ENDIF
        THRESH = TOL*SMINL
C       thresh = 0
      ENDIF
      OLDLL = LL
      OLDM = M
C
C     test for zero shift
      TEST = 1.0D0 + NCT*TOL*(SMINL/SMAX)
      IF ((TEST.EQ.1.0D0 .AND. IFULL.NE.1 .AND. TOL.GT.0.0) 
     +     .OR. IFULL.EQ.2) THEN
C       do a zero shift so that roundoff does not contaminate
C       smallest singular value
        SHIFT = 0.0D0
        IF (IDBG.GT.0) PRINT *,'SMINL TEST FOR SHIFT IS ZERO'
      ELSE
C     
C       compute shift from 2 by 2 block at end of matrix
        IF (IDIR.EQ.1) THEN
          SMM1 = S(M-1)
          EMM1 = E(M-1)
          SM = S(M)
          SLL = S(LL)
        ELSE
          SMM1 = S(LL+1)
          EMM1 = E(LL)
          SM = S(LL)
          SLL = S(M)
        ENDIF
        IF (IDBG.GT.0) PRINT *,'SMM1,EMM1,SM=',SMM1,EMM1,SM
        SHIFT = SIGMIN(SMM1,EMM1,SM)
        IF (IDBG.GT.0) PRINT *,'SIGMA-MIN OF 2 BY 2 CORNER=',SHIFT
        IF (TOL.GT.0.0) THEN
          IF (SHIFT.GT.NCT*SMIN) THEN
            LIMSHF = LIMSHF+1
            SHIFT = NCT*SMIN
            IF (IDBG.GT.0) PRINT *,'SHIFT LIMITED'
          ENDIF
          IF (IDBG.GT.0) PRINT *,'SHIFT=',SHIFT
          TEMP = SHIFT/SLL
          IF (IDBG.GT.0) PRINT *,'TEMP=',TEMP
          TEST = 1.0D0 - TEMP**2
C         test to see if shift negligible
          IF (IFULL.NE.1 .AND. TEST.EQ.1.0D0) SHIFT = 0.0D0
        ELSE
C         if shift much larger than s(ll), first rotation could be 0,
C         leading to infinite loop; avoid by doing 0 shift in this case
          IF (SHIFT.GT.ABS(S(LL))) THEN
            TEST = S(LL)/SHIFT
            IF (1.0D0 + TEST .EQ. 1.0D0) THEN
              LIMSHF = LIMSHF+1
              IF (IFULL.NE.1) SHIFT = 0.
              IF (IDBG.GT.0 .AND. IFULL.NE.1) PRINT *,'SHIFT LIMITED'
            ENDIF
          ENDIF
          TEST = SMAX+SMIN
          IF (TEST.EQ.SMAX .AND. IFULL.NE.1) SHIFT = 0.
        ENDIF
        IF (IDBG.GT.0) PRINT *,'TEST,SHIFT=',TEST,SHIFT
      ENDIF
C
C     increment iteration counter
      ITER = ITER + M-LL
      KOUNT = KOUNT + 1
      IF (IDBG.GT.1) THEN
        PRINT *,'S,E BEFORE QR'
        CALL PRSE(LL,M,N,P,S,E)
      ENDIF
C
C     if shift = 0, do simplified qr iteration
      IF (SHIFT.EQ.0.0D0) THEN
        KOUNT2 = KOUNT2 + M-LL
C
C       if idir=1, chase bulge from top to bottom
        IF (IDIR.EQ.1) THEN
          IF (IDBG.GT.2) PRINT *,'QR WITH ZERO SHIFT, TOP TO BOTTOM'
          OLDCS=1.0D0
          F=S(LL)
          G=E(LL)
          DO 1006 K=LL,M-1
C           if (idbg.gt.2) print *,'qr inner loop, k=',k
C           if (idbg.gt.3) print *,'f,g=',f,g
            CALL NDROTG(F,G,CS,SN)
            MAXSIN=MAX(MAXSIN,ABS(SN))
C           if (idbg.gt.3) print *,'f,cs,sn=',f,cs,sn
            IF (WANTV) CALL DROT(P,V(1,K),1,V(1,K+1),1,CS,SN)
            IF (K.NE.LL) E(K-1) = OLDSN*F
C           if (k.ne.ll .and. idbg.gt.3) print *,'e(k-1)=',e(k-1)
            F = OLDCS*F
C           if (idbg.gt.3) print *,'f=',f
            TEMP = S(K+1)
C           if (idbg.gt.3) print *,'temp=',temp
            G = TEMP*SN
C           if (idbg.gt.3) print *,'g=',g
            GG = TEMP*CS
C           if (idbg.gt.3) print *,'gg=',gg
            CALL NDROTG(F,G,CS,SN)
            MAXSIN=MAX(MAXSIN,ABS(SN))
C           if (idbg.gt.3) print *,'f,cs,sn=',f,cs,sn
C           if wantu and k.eq.n, then s(k+1)=0 so g=0 so cs=1 and sn=0
            IF (WANTU .AND. K .LT. N)
     *            CALL DROT(N,U(1,K),1,U(1,K+1),1,CS,SN)
            S(K) = F
C           if (idbg.gt.3) print *,'s(k)=',s(k)
            F = GG
C           if (idbg.gt.3) print *,'f=',f
            G = E(K+1)
C           if (idbg.gt.3) print *,'g=',g
            OLDCS = CS
C           if (idbg.gt.3) print *,'oldcs=',oldcs
            OLDSN = SN
C           if (idbg.gt.3) print *,'oldsn=',oldsn
C           if (idbg.gt.2) call prse(ll,m,n,p,s,e)
1006      CONTINUE
          E(M-1) = GG*SN
C         if (idbg.gt.3) print *,'e(m-1)=',e(m-1)
          S(M) = GG*CS
C         if (idbg.gt.3) print *,'s(m)=',s(m)
C
C         test convergence
          IF (IDBG.GT.0) THEN
            PRINT *,'CONVERGENCE DECISION FOR ZERO SHIFT TOP TO BOTTOM'
            PRINT *,'E(M-1), THRESHOLD=',E(M-1),THRESH
            IF (ABS(E(M-1)).LE.THRESH) PRINT *,'***CONVERGED***'
          ENDIF
          IF (ABS(E(M-1)).LE.THRESH) E(M-1) = 0.0D0
        ELSE
C       (idir=2, so chase bulge from bottom to top)
          IF (IDBG.GT.2) PRINT *,'QR WITH ZERO SHIFT, BOTTOM TO TOP'
          OLDCS=1.0D0
          F = S(M)
          G = E(M-1)
          DO 1007 K=M,LL+1,-1
C           if (idbg.gt.2) print *,'qr inner loop, k=',k
C           if (idbg.gt.3) print *,'f,g=',f,g
            CALL NDROTG(F,G,CS,SN)
            MAXSIN=MAX(MAXSIN,ABS(SN))
C           if (idbg.gt.3) print *,'f,cs,sn=',f,cs,sn
C           if m=n+1, always chase from top to bottom so no test for
C           k.lt.n necessary
            IF (WANTU) CALL DROT(N,U(1,K-1),1,U(1,K),1,CS,-SN)
            IF (K.NE.M) E(K) = OLDSN*F
C           if (k.ne.m .and. idbg.gt.3) print *,'e(k)=',e(k)
            F = OLDCS*F
C           if (idbg.gt.3) print *,'f=',f
            TEMP = S(K-1)
C           if (idbg.gt.3) print *,'temp=',temp
            G = SN*TEMP
C           if (idbg.gt.3) print *,'g=',g
            GG = CS*TEMP
C           if (idbg.gt.3) print *,'gg=',gg
            CALL NDROTG(F,G,CS,SN)
            MAXSIN=MAX(MAXSIN,ABS(SN))
C           if (idbg.gt.3) print *,'f,cs,sn=',f,cs,sn
            IF (WANTV) CALL DROT(P,V(1,K-1),1,V(1,K),1,CS,-SN)
            S(K) = F
C           if (idbg.gt.3) print *,'s(k)=',s(k)
            F = GG
C           if (idbg.gt.3) print *,'f=',f
            IF (K.NE.LL+1) G = E(K-2)
C           if (k.ne.ll+1 .and. idbg.gt.3) print *,'g=',g
            OLDCS = CS
C           if (idbg.gt.3) print *,'oldcs=',oldcs
            OLDSN = SN
C           if (idbg.gt.3) print *,'oldsn=',oldsn
C           if (idbg.gt.2) call prse(ll,m,n,p,s,e)
1007      CONTINUE
          E(LL) = GG*SN
C         if (idbg.gt.3) print *,'e(ll)=',e(ll)
          S(LL) = GG*CS
C         if (idbg.gt.3) print *,'s(ll)=',s(ll)
C
C         test convergence
          IF (IDBG.GT.0) THEN
            PRINT *,'CONVERGENCE DECISION FOR ZERO SHIFT BOTTOM TO TOP'
            PRINT *,'E(LL), THRESHOLD=',E(LL),THRESH
            IF (ABS(E(LL)).LE.THRESH) PRINT *,'***CONVERGED***'
          ENDIF
          IF (ABS(E(LL)).LE.THRESH) E(LL) = 0.0D0
        ENDIF
      ELSE
C     (shift.ne.0, so do standard qr iteration)
        KOUNT1 = KOUNT1 + M-LL
C
C       if idir=1, chase bulge from top to bottom
        IF (IDIR.EQ.1) THEN
          IF (IDBG.GT.2) PRINT *,'QR WITH NONZERO SHIFT, TOP TO BOTTOM'
          F = (ABS(S(LL)) - SHIFT)*(DSIGN(1.0D0,S(LL))+SHIFT/S(LL))
          G = E(LL)
          DO 1008 K = LL, M-1
C           if (idbg.gt.2) print *,'qr inner loop, k=',k
C           if (idbg.gt.3) print *,'f,g=',f,g
            CALL NDROTG(F,G,CS,SN)
C           if (idbg.gt.3) print *,'f,cs,sn=',f,cs,sn
            IF (K .NE. LL) E(K-1) = F
C           if (k.ne.ll .and. idbg.gt.3) print *,'e(k-1)=',e(k-1)
            F = CS*S(K) + SN*E(K)
C           if (idbg.gt.3) print *,'f=',f
            E(K) = CS*E(K) - SN*S(K)
C           if (idbg.gt.3) print *,'e(k)=',e(k)
            G = SN*S(K+1)
C           if (idbg.gt.3) print *,'g=',g
            S(K+1) = CS*S(K+1)
C           if (idbg.gt.3) print *,'s(k+1)=',s(k+1)
            IF (WANTV) CALL DROT(P,V(1,K),1,V(1,K+1),1,CS,SN)
            CALL NDROTG(F,G,CS,SN)
C           if (idbg.gt.3) print *,'f,cs,sn=',f,cs,sn
            S(K) = F
C           if (idbg.gt.3) print *,'s(k)=',s(k)
            F = CS*E(K) + SN*S(K+1)
C           if (idbg.gt.3) print *,'f=',f
            S(K+1) = -SN*E(K) + CS*S(K+1)
C           if (idbg.gt.3) print *,'s(k+1)=',s(k+1)
            G = SN*E(K+1)
C           if (idbg.gt.3) print *,'g=',g
            E(K+1) = CS*E(K+1)
C           if (idbg.gt.3) print *,'e(k+1)=',e(k+1)
C           test for k.lt.n seems unnecessary since k=n causes zero 
C           shift, so test removed from original code
            IF (WANTU) CALL DROT(N,U(1,K),1,U(1,K+1),1,CS,SN)
C           if (idbg.gt.2) call prse(ll,m,n,p,s,e)
1008      CONTINUE
          E(M-1) = F
C         if (idbg.gt.3) print *,'e(m-1)=',e(m-1)
C
C         check convergence
          IF (IDBG.GT.0) THEN
            PRINT *,'CONVERGENCE DECISION FOR SHIFT TOP TO BOTTOM'
            PRINT *,'E(M-1), THRESHOLD=',E(M-1),THRESH
            IF (ABS(E(M-1)).LE.THRESH) PRINT *,'***CONVERGED***'
          ENDIF
          IF (ABS(E(M-1)).LE.THRESH) E(M-1) = 0.0D0
        ELSE
C       (idir=2, so chase bulge from bottom to top)
          IF (IDBG.GT.2) PRINT *,'QR WITH NONZERO SHIFT, BOTTOM TO TOP'
          F = (ABS(S(M))-SHIFT)*(DSIGN(1.0D0,S(M))+SHIFT/S(M))
          G = E(M-1)
          DO 1009 K=M,LL+1,-1
C           if (idbg.gt.2) print *,'qr inner loop, k=',k
C           if (idbg.gt.3) print *,'f,g=',f,g
            CALL NDROTG(F,G,CS,SN)
C           if (idbg.gt.3) print *,'f,cs,sn=',f,cs,sn
            IF (K.NE.M) E(K)=F
C           if (k.ne.m .and. idbg.gt.3) print *,'e(k)=',e(k)
            F = CS*S(K) + SN*E(K-1)
C           if (idbg.gt.3) print *,'f=',f
            E(K-1) = -SN*S(K) + CS*E(K-1)
C           if (idbg.gt.3) print *,'e(k-1)=',e(k-1)
            G = SN*S(K-1)
C           if (idbg.gt.3) print *,'g=',g
            S(K-1) = CS*S(K-1)
C           if (idbg.gt.3) print *,'s(k-1)=',s(k-1)
            IF (WANTU .AND. K .LE. N)
     *            CALL DROT(N,U(1,K-1),1,U(1,K),1,CS,-SN)
            CALL NDROTG(F,G,CS,SN)
C           if (idbg.gt.3) print *,'f,cs,sn=',f,cs,sn
            IF (WANTV) CALL DROT(P,V(1,K-1),1,V(1,K),1,CS,-SN)
            S(K) = F
C           if (idbg.gt.3) print *,'s(k)=',s(k)
            F = SN*S(K-1) + CS*E(K-1)
C           if (idbg.gt.3) print *,'f=',f
            S(K-1) = CS*S(K-1) - SN*E(K-1)
C           if (idbg.gt.3) print *,'s(k-1)=',s(k-1)
            IF (K.NE.LL+1) THEN
              G = SN*E(K-2)
C             if (idbg.gt.3) print *,'g=',g
              E(K-2) = CS*E(K-2)
C             if (idbg.gt.3) print *,'e(k-2)=',e(k-2)
            ENDIF
C           if (idbg.gt.2) call prse(ll,m,n,p,s,e)
1009      CONTINUE
          E(LL) = F
C         if (idbg.gt.3) print *,'e(ll)=',e(ll)
C
C         test convergence
          IF (IDBG.GT.0) THEN
            PRINT *,'CONVERGENCE DECISION FOR SHIFT BOTTOM TO TOP'
            PRINT *,'E(LL), THRESHOLD=',E(LL),THRESH
            IF (ABS(E(LL)).LE.THRESH) PRINT *,'***CONVERGED***'
          ENDIF
          IF (ABS(E(LL)).LE.THRESH) E(LL) = 0.0D0
        ENDIF
      ENDIF
C
      IF (IDBG.GT.1) THEN
        PRINT *,'S,E AFTER QR'
        CALL PRSE(LL,M,N,P,S,E)
      ENDIF
C
C     qr iteration finished, go back to check convergence
      GOTO 999
C
998   CONTINUE
C
C     make singular values positive
      M=MIN(N,P)
      DO 1010 I=1,M
        IF (S(I).LT.0.0D0) THEN
          S(I)=-S(I)
          IF (WANTV) CALL DSCAL(P,-1.0D0,V(1,I),1)
        ENDIF
1010  CONTINUE
C
C     sort singular values from largest at top to smallest
C     at bottom (use insertion sort)
      DO 1011 I=1,M-1
C       scan for smallest s(j)
        IISUB = 1
        SMIN = S(1)
        DO 1012 J=2,M+1-I
          IF (S(J).LT.SMIN) THEN
            IISUB = J
            SMIN = S(J)
          ENDIF
1012    CONTINUE
        IF (IISUB.NE.M+1-I) THEN
C         swap singular values, vectors
          TEMP = S(M+1-I)
          S(M+1-I) = S(IISUB)
          S(IISUB) = TEMP
          IF (WANTV) CALL DSWAP(P,V(1,M+1-I),1,V(1,IISUB),1)
          IF (WANTU) CALL DSWAP(N,U(1,M+1-I),1,U(1,IISUB),1)
        ENDIF
1011  CONTINUE
C
C     finished, return
      RETURN
C
997   CONTINUE
C     maximum number of iterations exceeded
      DO 1013 I=M-1,1,-1
        INFO=I+1
        IF (E(I).NE.0.0D0) GOTO 996
1013  CONTINUE
996   CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE PRSE(LL,M,NROW,NCOL,S,E)
C     debug routine to print s,e
      DOUBLE PRECISION S(1),E(1)
      PRINT 100,LL,M
100   FORMAT(22X,'S(.)',22X,'E(.) FOR LL,M=',2I3)
      DO 1 I=LL,M-1
        PRINT 101,S(I),E(I)
101     FORMAT(2D26.17)
1     CONTINUE
      IF (M.GE.NCOL) PRINT 101,S(M)
      IF (M.LT.NCOL) PRINT 101,S(M),E(M)
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE SIG22(A,B,C,SIGMIN,SIGMAX,SNR,CSR,SNL,CSL)
C     compute the singular value decomposition of the 2 by 2
C     upper triangular matrix [[a,b];[0,c]]
C     inputs -
C       a,b,c - real*8 - matrix entries
C     outputs -
C       sigmin - real*8 - +-smaller singular value
C       sigmax - real*8 - +-larger singular value
C       snr, csr - real*8 - sin and cos of right rotation (see below)
C       snl, csl - real*8 - sin and cos of left rotation (see below)
C
C       [  csl  snl ]  * [ a b ] * [ csr  -snr ] = [ sigmax    0   ]
C       [ -snl  csl ]    [ 0 c ]   [ snr   csr ]   [    0   sigmin ]
C
C     barring over/underflow all output quantities are correct to 
C     within a few units in their last places
C    
C     let UF denote the underflow and OF the overflow threshold
C     let eps denote the machine precision
C
C     overflow is impossible unless the true value of sigmax exceeds 
C     OF (or does so within a few units in its last place)
C
C     underflow cannot adversely effect sigmin,sigmax unless they are 
C     less than UF/eps for conventional underflow
C     underflow cannot adversely effect sigmin,sigmax if underflow is 
C     gradual and results normalized
C
C     overflow is impossible in computing sinr, cosr, sinl, cosl
C     underflow can adversely effect results only if sigmax<UF/eps
C     or true angle of rotation < UF/eps
C
C     note: if c=0, then csl=1. and snl=0. (needed in general svd)
C
      REAL*8 A,B,C,SIGMIN,SIGMAX,SINR,COSR,SINL,COSL
      REAL*8 SNR,CSR,SNL,CSL
C     THIS LINE ADDED BY JONATHAN SHERRATT 24/SEPT/2010
C   
C     local variables:
      REAL*8 ABSA,ABSB,ABSC,ACMN,ACMX,AS,AT,AU,TEMP,TEMP1,TEMP2,TEMP3
      REAL*8 AC,CA,BAC,ABSBAC,SGNMN,SGNMX,SGNA,SGNB,SGNC
C
      ABSA = DABS(A)
      ABSB = DABS(B)
      ABSC = DABS(C)
      SGNA = DSIGN(1.0D0,A)
      SGNB = DSIGN(1.0D0,B)
      SGNC = DSIGN(1.0D0,C)
      ACMN = MIN(ABSA,ABSC)
      ACMX = MAX(ABSA,ABSC)
C     bad underflow possible if acmx<UF/eps and standard underflow
C     underflow impossible if underflow gradual
C     either at=0 or eps/2 <= at <= 1
C     if no or gradual underflow, at nearly correctly rounded
      AT = (ACMX-ACMN)
      IF (AT .NE. 0.0) AT = AT/ACMX
C
C     compute sigmin, sigmax
C
      IF ( ABSB .LT. ACMX ) THEN
C         abs(bac) <= 1, underflow possible
          IF (ABSA .LT. ABSC) THEN
              BAC = B/C
          ELSE
              BAC = B/A
          ENDIF
C         1 <= as <= 2, underflow and roundoff harmless
          AS = 1.0D0 + ACMN/ACMX
C         0 <= au <= 1, underflow possible
          AU = BAC*BAC
C         1 <= temp1 <= sqrt(5), underflow, roundoff harmless
          TEMP1 = DSQRT(AS*AS+AU)
C         0 <= temp2 <= 1, possible harmful underflow from at
          TEMP2 = DSQRT(AT*AT+AU)
C         1 <= temp <= sqrt(5) + sqrt(2)
          TEMP = TEMP1 + TEMP2
          SIGMIN = ACMN/TEMP
          SIGMIN = SIGMIN+SIGMIN
          SIGMAX = ACMX*(TEMP/2.)
      ELSE
          IF (ABSB.EQ.0.0) THEN
C             matrix identically zero
              SIGMIN = 0.
              SIGMAX = 0.
          ELSE
C             0 <= au <= 1, underflow possible
              AU = ACMX/ABSB
              IF (AU.EQ.0) THEN
C                 either au=0 exactly or underflows
C                 sigmin only underflows if true value should
C                 overflow on product acmn*acmx impossible
                  SIGMIN = (ACMX*ACMN)/ABSB
                  SIGMAX = ABSB
              ELSE
C                 1 <= as <= 2, underflow and roundoff harmless
                  AS = 1.0D0 + ACMN/ACMX
C                 2 <= temp <= sqrt(5)+sqrt(2), possible harmful 
C                 underflow from at
                  TEMP = DSQRT(1.0D0+(AS*AU)**2)+DSQRT(1.0D0+(AT*AU)**2)
C                 0 < sigmin <= 2
                  SIGMIN = AU+AU
C                 bad underflow possible only if true sigmin near UF
                  SIGMIN = SIGMIN*(ACMN/TEMP)
                  SIGMAX = ABSB*(TEMP/2.0D0)
              ENDIF
          ENDIF
      ENDIF
C
C     compute rotations
C
      IF (ABSB .LE. ACMX) THEN
          IF (AT .EQ. 0.0D0) THEN
C             assume as = 2, since otherwise underflow will have
C             contaminated at so much that we get a bad answer
C             anyway; this can only happen if sigmax < UF/eps
C             with conventional underflow; this cannot happen
C             with gradual underflow
              IF (ABSB .GT. 0.0D0) THEN
C                 0 <= absbac <= 1
                  ABSBAC = ABSB/ACMX
C                 1 <= temp3 <= 1+sqrt(5), underflow harmless
                  TEMP3 = ABSBAC + DSQRT(4.0D0+AU)
C                 1/3 <= temp3 <= (1+sqrt(10))/2
                  TEMP3 = TEMP3/(2.0D0 + ABSBAC*TEMP3)
                  SINR = DSIGN(1.0D0,B)
                  COSR = DSIGN(TEMP3,A)
                  SINL = DSIGN(TEMP3,C)
                  COSL = SINR
                  SGNMN = SGNA*SGNB*SGNC
                  SGNMX = SGNB
              ELSE
C                 matrix diagonal
                  SINR = 0.0D0
                  COSR = 1.0D0
                  SINL = 0.0D0
                  COSL = 1.0D0
                  SGNMN = SGNC
                  SGNMX = SGNA
              ENDIF
          ELSE
C             at .ne. 0, so eps/2 <= at <= 1
C             eps/2 <= temp3 <= 1 + sqrt(10)
              TEMP3 = AU + TEMP1*TEMP2
              IF (ABSA .LT. ABSC) THEN
C                 abs(ac) <= 1
                  AC = A/C
C                 eps <= sinr <= sqrt(13)+3
                  SINR = DSQRT((AS*AT+AU)**2+4.0D0*AC*AC*AU)+AS*AT+AU
C                 abs(cosr) <= 2; if underflow, true cosr<UF/eps
                  COSR = AC*BAC
                  COSR = COSR + COSR
C                 eps/(3+sqrt(10)) <= sinl <= 1
                  SINL = (AS*AT + TEMP3)/(1.0D0 + AC*AC + TEMP3)
C                 bad underflow possible only if sigmax < UF/eps
                  SINL = C*SINL
                  COSL = B
                  SGNMN = SGNA*SGNC
                  SGNMX = 1.0
              ELSE
C                 abs(ca) <= 1
                  CA = C/A
                  SINR = B
                  COSR = (AS*AT + TEMP3)/(1.0D0 + CA*CA + TEMP3)
                  COSR = A*COSR
C                 abs(sinl) <= 2; if underflow, true sinl<UF/eps
                  SINL = CA*BAC
                  SINL = SINL + SINL
C                 eps <= cosl <= sqrt(13)+3
                  COSL = DSQRT((AS*AT+AU)**2+4.0D0*CA*CA*AU)+AS*AT+AU
                  SGNMN = SGNA*SGNC
                  SGNMX = 1.0
              ENDIF
          ENDIF
      ELSE
          IF (ABSA .EQ. 0.0D0) THEN
              COSR = 0.0D0
              SINR = 1.0D0
              IA = 0
          ELSE
              SINR = B
C             sigmin <= abs(a)/sqrt(2), so no bad cancellation in
C             absa-sigmin; overflow extremely unlikely, and in any
C             event only if sigmax overflows as well
              COSR = (ABSA-SIGMIN)*(DSIGN(1.0D0,A)+SIGMIN/A)
              IA = 1
          ENDIF
          IF (ABSC .EQ. 0.0D0) THEN
              SINL = 0.0D0
              COSL = 1.0D0
              IB = 0
          ELSE
              COSL = B
C             sigmin <= abs(c)/sqrt(2), so no bad cancellation in
C             absc-sigmin; overflow extremely unlikely, and in any
C             event only if sigmax overflows as well
              SINL = (ABSC-SIGMIN)*(DSIGN(1.0D0,C)+SIGMIN/C)
              IB = 1
          ENDIF
          IF (IA.EQ.0 .AND. IB.EQ.0) THEN
              SGNMN = 1.0
              SGNMX = SGNB
          ELSEIF (IA.EQ.0 .AND. IB.EQ.1) THEN
              SGNMN = 1.0
              SGNMX = 1.0
          ELSEIF (IA.EQ.1 .AND. IB.EQ.0) THEN
              SGNMN = SGNA*SGNC
              SGNMX = 1.0
          ELSE
              SGNMN = SGNA*SGNB*SGNC
              SGNMX = SGNB
          ENDIF
      ENDIF
      SIGMIN = SGNMN*SIGMIN
      SIGMAX = SGNMX*SIGMAX
      CALL SNDRTG(COSR,SINR,CSR,SNR)
      CALL SNDRTG(COSL,SINL,CSL,SNL)
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      DOUBLE PRECISION FUNCTION SIGMIN(A,B,C)
C     compute smallest singular value of 2 by 2 matrix ((a,b);(0,c))
C     answer is accurate to a few ulps if final answer
C     exceeds (underflow_threshold/macheps)
C     overflow is impossible
      DOUBLE PRECISION A,B,C, AA,AB,AC,ACMX,ACMN,AS,AT,AU
      AA = ABS(A)
      AB = ABS(B)
      AC = ABS(C)
      ACMN = MIN(AA,AC)
      IF (ACMN.EQ.0.0D0) THEN
        SIGMIN = 0.0D0
      ELSE
        ACMX = MAX(AA,AC)
        AB = ABS(B)
        IF (AB.LT.ACMX) THEN
          AS = 1.0D0 + ACMN/ACMX
          AT = (ACMX-ACMN)/ACMX
          AU = (AB/ACMX)**2
          SIGMIN = ACMN/(DSQRT(AS*AS+AU) + DSQRT(AT*AT+AU))
          SIGMIN = SIGMIN + SIGMIN
        ELSE
          AU = ACMX/AB
          IF (AU.EQ.0.0D0) THEN
C           possible harmful underflow
C           if exponent range asymmetric, true sigmin may not 
C           underflow
            SIGMIN = (ACMN*ACMX)/AB
          ELSE
            AS = 1.0D0 + ACMN/ACMX
            AT = (ACMX-ACMN)/ACMX
            SIGMIN = ACMN/(DSQRT(1.0D0+(AS*AU)**2) +
     *                     DSQRT(1.0D0+(AT*AU)**2))
            SIGMIN = AU*SIGMIN
            SIGMIN = SIGMIN + SIGMIN
          ENDIF
        ENDIF
      ENDIF
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE SNDRTG(F,G,CS,SN)
C     version of ndrotg, in which sign(f)=sign(cs),sign(g)=sign(sn)
C     cs, sn returned so that -sn*f+cs*g = 0
C     and cs*f + sn*g = sqrt(f**2+g**2) 
      DOUBLE PRECISION F, G, CS, SN, T, TT
      IF (F.EQ.0.0D0 .AND. G.EQ.0.0D0) THEN
        CS = 1.0D0
        SN = 0.0D0
      ELSE
        IF (ABS(F) .GT. ABS(G)) THEN
          T = G/F
          TT = DSQRT(1.0D0 + T*T )
          CS = DSIGN(1.0D0/TT,F)
          SN = DSIGN(T*CS,G)
        ELSE
          T = F/G
          TT = DSQRT(1.0D0 + T*T )
          SN = DSIGN(1.0D0/TT,G)
          CS = DSIGN(T*SN,F)
        ENDIF
      ENDIF
      RETURN
      END
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C              LINPACK and LAPACK routines
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C
C  hqr3_loc.f orthes.f ortran.f   for computing triangular matrix
C	
      SUBROUTINE HQR3LC(A,V,N,NLOW,NUP,EPS,ER,EI,TYPE,NA,NV,IMFD)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER N, NA, NLOW, NUP, NV, TYPE(N),IMFD
      DOUBLE PRECISION A(NA,N), EI(N), ER(N), EPS, V(NV,N)
C HQR3 REDUCES THE UPPER HESSENBERG MATRIX A TO QUASI-
C TRIANGULAR FORM BY UNITARY SIMILARITY TRANSFORMATIONS.
C THE EIGENVALUES OF A, WHICH ARE CONTAINED IN THE 1X1
C AND 2X2 DIAGONAL BLOCKS OF THE REDUCED MATRIX, ARE
C ORDERED IN assending or DESCENDING ORDER OF ALONG THE
C DIAGONAL.  THE TRANSFORMATIONS ARE ACCUMULATED IN THE
C ARRAY V.  HQR3 REQUIRES THE SUBROUTINES EXCHNG,
C QRSTEP, AND SPLIT.  THE PARAMETERS IN THE CALLING
C SEQUENCE ARE (STARRED PARAMETERS ARE ALTERED BY THE
C SUBROUTINE)
C    *A       AN ARRAY THAT INITIALLY CONTAINS THE N X N
C             UPPER HESSENBERG MATRIX TO BE REDUCED.  ON
C             RETURN A CONTAINS THE REDUCED, QUASI-
C             TRIANGULAR MATRIX.
C    *V       AN ARRAY THAT CONTAINS A MATRIX INTO WHICH
C             THE REDUCING TRANSFORMATIONS ARE TO BE
C             MULTIPLIED.
C     N       THE ORDER OF THE MATRICES A AND V.
C     NLOW    A(NLOW,NLOW-1) AND A(NUP,+1,NUP) ARE
C     NUP     ASSUMED TO BE ZERO, AND ONLY ROWS NLOW
C             THROUGH NUP AND COLUMNS NLOW THROUGH
C             NUP ARE TRANSFORMED, RESULTING IN THE
C             CALCULATION OF EIGENVALUES NLOW
C             THROUGH NUP.
C     EPS     A CONVERGENCE CRITERION.
C    *ER      AN ARRAY THAT ON RETURN CONTAINS THE REAL
C             PARTS OF THE EIGENVALUES.
C    *EI      AN ARRAY THAT ON RETURN CONTAINS THE
C             IMAGINARY PARTS OF THE EIGENVALUES.
C    *TYPE    AN INTEGER ARRAY WHOSE I-TH ENTRY IS
C               0   IF THE I-TH EIGENVALUE IS REAL,
C               1   IF THE I-TH EIGENVALUE IS COMPLEX
C                   WITH POSITIVE IMAGINARY PART.
C               2   IF THE I-TH EIGENVALUE IS COMPLEX
C                   WITH NEGATIVE IMAGINARY PART,
C              -1   IF THE I-TH EIGENVALUE WAS NOT
C                   CALCULATED SUCCESSFULLY.
C     NA      THE FIRST DIMENSION OF THE ARRAY A.
C     NV      THE FIRST DIMENSION OF THE ARRAY V.
C
C     imfd    ascending or descending order of real part of eigenvalues
C              -1  ascending (i.e. negative eigenvalues first)
C              +1  descending (positive eigenvalues)
C
C
C THE CONVERGENCE CRITERION EPS IS USED TO DETERMINE
C WHEN A SUBDIAGONAL ELEMENT OF A IS NEGLIGIBLE.
C SPECIFICALLY A(I+1,I) IS REGARDED AS NEGLIGIBLE
C IF
C        ABS(A(I+1),I)) .LE. EPS*(ABS(A(I,I))+ABS(A(I+1,I+1))).
C THIS MEANS THAT THE FINAL MATRIX RETURNED BY THE
C PROGRAM WILL BE EXACTLY SIMILAR TO A + E WHERE E IS
C OF ORDER EPS*NORM(A), FOR ANY REASONABLY BALANCED NORM
C SUCH AS THE ROW-SUM NORM.
C INTERNAL VARIABLES
      INTEGER I, IT, L, MU, NL, NU
      DOUBLE PRECISION E1, E2, P, Q, R, S, T, W, X, Y, Z
      LOGICAL FAIL
C INITIALIZE.
      DO 10 I=NLOW,NUP
        TYPE(I) = -1
   10 CONTINUE
      T = 0.D0
C MAIN LOOP. FIND AND ORDER EIGENVALUES.
      NU = NUP
   20 IF (NU.LT.NLOW) GO TO 240
      IT = 0
C QR LOOP.  FIND NEGLIGIBLE ELEMENTS AND PERFORM
C QR STEPS.
   30 CONTINUE
C SEARCH BACK FOR NEGLIGIBLE ELEMENTS.
      L = NU
   40 CONTINUE
      IF (L.EQ.NLOW) GO TO 50
      IF (DABS(A(L,L-1)).LE.EPS*(DABS(A(L-1,L-1))+DABS(A(L,L)))) GO TO
     * 50
      L = L - 1
      GO TO 40
   50 CONTINUE
C TEST TO SEE IF AN EIGENVALUE OR A 2X2 BLOCK
C HAS BEEN FOUND.
      X = A(NU,NU)
      IF (L.EQ.NU) GO TO 160
      Y = A(NU-1,NU-1)
      W = A(NU,NU-1)*A(NU-1,NU)
      IF (L.EQ.NU-1) GO TO 100
C TEST ITERATION COUNT. IF IT IS 30 QUIT.  IF
C IT IS 10 OR 20 SET UP AN AD-HOC SHIFT.
      IF (IT.EQ.30) GO TO 240
      IF (IT.NE.10 .AND. IT.NE.20) GO TO 70
C AD-HOC SHIFT.
      T = T + X
      DO 60 I=NLOW,NU
        A(I,I) = A(I,I) - X
   60 CONTINUE
      S = DABS(A(NU,NU-1)) + DABS(A(NU-1,NU-2))
      X = 0.75D0*S
      Y = X
      W = -0.4375D0*S**2
   70 CONTINUE
      IT = IT + 1
C LOOK FOR TWO CONSECUTIVE SMALL SUB-DIAGONAL
C ELEMENTS.
      NL = NU - 2
   80 CONTINUE
      Z = A(NL,NL)
      R = X - Z
      S = Y - Z
      P = (R*S-W)/A(NL+1,NL) + A(NL,NL+1)
      Q = A(NL+1,NL+1) - Z - R - S
      R = A(NL+2,NL+1)
      S = DABS(P) + DABS(Q) + DABS(R)
      P = P/S
      Q = Q/S
      R = R/S
      IF (NL.EQ.L) GO TO 90
      IF (DABS(A(NL,NL-1))*(DABS(Q)+DABS(R)).LE.EPS*DABS(P)
     &   *(DABS(A(NL-1,
     &     NL-1))+DABS(Z)+DABS(A(NL+1,NL+1)))) GO TO 90
      NL = NL - 1
      GO TO 80
   90 CONTINUE
C PERFORM A QR STEP BETWEEN NL AND NU.
      CALL QRSTEP(A, V, P, Q, R, NL, NU, N, NA, NV)
      GO TO 30
C 2X2 BLOCK FOUND.
  100 IF (NU.NE.NLOW+1) A(NU-1,NU-2) = 0.D0
      A(NU,NU) = A(NU,NU) + T
      A(NU-1,NU-1) = A(NU-1,NU-1) + T
      TYPE(NU) = 0
      TYPE(NU-1) = 0
      MU = NU
C LOOP TO POSITION  2X2 BLOCK.
  110 CONTINUE
      NL = MU - 1
C ATTEMPT  TO SPLIT THE BLOCK INTO TWO REAL
C EIGENVALUES.
      CALL SPLIT(A, V, N, NL, E1, E2, NA, NV)
C IF THE SPLIT WAS SUCCESSFUL, GO AND ORDER THE
C REAL EIGENVALUES.
      IF (A(MU,MU-1).EQ.0.D0) GO TO 170
C TEST TO SEE IF THE BLOCK IS PROPERLY POSITIONED,
C AND IF NOT EXCHANGE IT
      IF (MU.EQ.NUP) GO TO 230
      IF (MU.EQ.NUP-1) GO TO 130
      IF (A(MU+2,MU+1).EQ.0.D0) GO TO 130
C THE NEXT BLOCK IS 2X2.
C     IF (A(MU-1,MU-1)*A(MU,MU)-A(MU-1,MU)*A(MU,MU-1).GE.A(MU+1,
C    * MU+1)*A(MU+2,MU+2)-A(MU+1,MU+2)*A(MU+2,MU+1)) GO TO 230
C
      IF (IMFD.EQ.1) THEN
        IF ((A(MU-1,MU-1)+A(MU,MU)).GE.(A(MU+1,MU+1)+A(MU+2,MU+2))) 
     *     GO TO 230
      ELSE 
        IF ((A(MU-1,MU-1)+A(MU,MU)).LE.(A(MU+1,MU+1)+A(MU+2,MU+2))) 
     *     GO TO 230
      ENDIF 
C
      CALL EXCHNG(A, V, N, NL, 2, 2, EPS, FAIL, NA, NV)
      IF (.NOT.FAIL) GO TO 120
      TYPE(NL) = -1
      TYPE(NL+1) = -1
      TYPE(NL+2) = -1
      TYPE(NL+3) = -1
      GO TO 240
  120 CONTINUE
      MU = MU + 2
      GO TO 150
  130 CONTINUE
C THE NEXT BLOCK IS 1X1.
C     IF (A(MU-1,MU-1)*A(MU,MU)-A(MU-1,MU)*A(MU,MU-1).GE.A(MU+1,
C    * MU+1)**2) GO TO 230
C
      IF (IMFD.EQ.1) THEN
        IF ((A(MU-1,MU-1)+A(MU,MU)).GE.(2.0D0*A(MU+1,MU+1))) 
     *     GO TO 230
      ELSE 
        IF ((A(MU-1,MU-1)+A(MU,MU)).LE.(2.0D0*A(MU+1,MU+1))) 
     *     GO TO 230
      ENDIF 
C
      CALL EXCHNG(A, V, N, NL, 2, 1, EPS, FAIL, NA, NV)
      IF (.NOT.FAIL) GO TO 140
      TYPE(NL) = -1
      TYPE(NL+1) = -1
      TYPE(NL+2) = -1
      GO TO 240
  140 CONTINUE
      MU = MU + 1
  150 CONTINUE
      GO TO 110
C SINGLE EIGENVALUE FOUND.
  160 NL = 0
      A(NU,NU) = A(NU,NU) + T
      IF (NU.NE.NLOW) A(NU,NU-1) = 0.D0
      TYPE(NU) = 0
      MU = NU
C LOOP TO POSITION ONE OR TWO REAL EIGENVALUES.
  170 CONTINUE
C POSITION THE EIGENVALUE LOCATED AT A(NL,NL).
  180 CONTINUE
      IF (MU.EQ.NUP) GO TO 220
      IF (MU.EQ.NUP-1) GO TO 200
      IF (A(MU+2,MU+1).EQ.0.D0) GO TO 200
C THE NEXT BLOCK IS 2X2.
C      IF (A(MU,MU)**2.GE.A(MU+1,MU+1)*A(MU+2,MU+2)-A(MU+1,MU+2)*
C    * A(MU+2,MU+1)) GO TO 220
C
      IF (IMFD.EQ.1) THEN
        IF ((2.0D0*A(MU,MU)).GE.(A(MU+1,MU+1)+A(MU+2,MU+2))) 
     *     GO TO 220
      ELSE
        IF ((2.0D0*A(MU,MU)).LE.(A(MU+1,MU+1)+A(MU+2,MU+2))) 
     *     GO TO 220
      ENDIF
C
      CALL EXCHNG(A, V, N, MU, 1, 2, EPS, FAIL, NA, NV)
      IF (.NOT.FAIL) GO TO 190
      TYPE(MU) = -1
      TYPE(MU+1) = -1
      TYPE(MU+2) = -1
      GO TO 240
  190 CONTINUE
      MU = MU + 2
      GO TO 210
  200 CONTINUE
C THE NEXT BLOCK IS 1X1.
C      IF (ABS(A(MU,MU)).GE.ABS(A(MU+1,MU+1))) GO TO 220
C
      IF (IMFD.EQ.1) THEN
        IF (A(MU,MU).GE.A(MU+1,MU+1)) GO TO 220
      ELSE
        IF (A(MU,MU).LE.A(MU+1,MU+1)) GO TO 220
      ENDIF
C
      CALL EXCHNG(A, V, N, MU, 1, 1, EPS, FAIL, NA, NV)
      MU = MU + 1
  210 CONTINUE
      GO TO 180
  220 CONTINUE
      MU = NL
      NL = 0
      IF (MU.NE.0) GO TO 170
C GO BACK AND GET THE NEXT EIGENVALUE.
  230 CONTINUE
      NU = L - 1
      GO TO 20
C ALL THE EIGENVALUES HAVE BEEN FOUND AND ORDERED.
C COMPUTE THEIR VALUES AND TYPE.
  240 IF (NU.LT.NLOW) GO TO 260
      DO 250 I=NLOW,NU
        A(I,I) = A(I,I) + T
  250 CONTINUE
  260 CONTINUE
      NU = NUP
  270 CONTINUE
      IF (TYPE(NU).NE.-1) GO TO 280
      NU = NU - 1
      GO TO 310
  280 CONTINUE
      IF (NU.EQ.NLOW) GO TO 290
      IF (A(NU,NU-1).EQ.0.D0) GO TO 290
C 2X2 BLOCK.
      CALL SPLIT(A, V, N, NU-1, E1, E2, NA, NV)
      IF (A(NU,NU-1).EQ.0.D0) GO TO 290
      ER(NU) = E1
      EI(NU-1) = E2
      ER(NU-1) = ER(NU)
      EI(NU) = -EI(NU-1)
      TYPE(NU-1) = 1
      TYPE(NU) = 2
      NU = NU - 2
      GO TO 300
  290 CONTINUE
C SINGLE ROOT.
      ER(NU) = A(NU,NU)
      EI(NU) = 0.D0
      NU = NU - 1
  300 CONTINUE
  310 CONTINUE
      IF (NU.GE.NLOW) GO TO 270
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE SPLIT(A, V, N, L, E1, E2, NA, NV)                      SPL   10
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER L, N, NA, NV
      DOUBLE PRECISION A(NA,N), V(NV,N)
C GIVEN THE UPPER HESSENBERG MATRIX A WITH A 2X2 BLOCK
C STARTING AT A(L,L), SPLIT DETERMINES IF THE
C CORRESPONDING EIGENVALUES ARE REAL OR COMPLEX. IF THEY
C ARE REAL, A ROTATION IS DETERMINED THAT REDUCES THE
C BLOCK TO UPPER TRIANGULAR FORM WITH THE EIGENVALUE
C OF LARGEST ABSOLUTE VALUE APPEARING FIRST.  THE
C ROTATION IS ACCUMULATED IN V.  THE EIGENVALUES (REAL
C OR COMPLEX) ARE RETURNED IN E1 AND E2.  THE PARAMETERS
C IN THE CALLING SEQUENCE ARE (STARRED PARAMETERS ARE
C ALTERED BY THE SUBROUTINE)
C    *A       THE UPPER HESSENVERG MATRIX WHOSE 2X2
C             BLOCK IS TO BE SPLIT.
C    *V       THE ARRAY IN WHICH THE SPLITTING TRANS-
C             FORMATION IS TO BE ACCUMULATED.
C     N       THE ORDER OF THE MATRIX A.
C     L       THE POSITION OF THE 2X2 BLOCK.
C    *E1      ON RETURN IF THE EIGENVALUES ARE COMPLEX
C    *E2      E1 CONTAINS THEIR COMMON REAL PART AND
C             E2 CONTAINS THE POSITIVE IMAGINARY PART.
C             IF THE EIGENVALUES ARE REAL, E1 CONTAINS
C             THE ONE LARGEST IN ABSOLUTE VALUE AND E2
C             CONTAINS THE OTHER ONE.
C     NA      THE FIRST DIMENSION OF THE ARRAY A.
C     NV      THE FIRST DIMENSION OF THE ARRAY V.
C INTERNAL VARIABLES
      INTEGER I, J, L1
      DOUBLE PRECISION P, Q, R, T, U, W, X, Y, Z
      X = A(L+1,L+1)
      Y = A(L,L)
      W = A(L,L+1)*A(L+1,L)
      P = (Y-X)/2.D0
      Q = P**2 + W
      IF (Q.GE.0.D0) GO TO 10
C COMPLEX EIGENVALUE.
      E1 = P + X
      E2 = DSQRT(-Q)
      RETURN
   10 CONTINUE
C TWO REAL EIGENVALUES.  SET UP TRANSFORMATION.
      Z = DSQRT(Q)
      IF (P.LT.0.D0) GO TO 20
      Z = P + Z
      GO TO 30
   20 CONTINUE
      Z = P - Z
   30 CONTINUE
      IF (Z.EQ.0.D0) GO TO 40
      R = -W/Z
      GO TO 50
   40 CONTINUE
      R = 0.D0
   50 CONTINUE
      IF (DABS(X+Z).GE.DABS(X+R)) Z = R
      Y = Y - X - Z
      X = -Z
      T = A(L,L+1)
      U = A(L+1,L)
      IF (DABS(Y)+DABS(U).LE.DABS(T)+DABS(X)) GO TO 60
      Q = U
      P = Y
      GO TO 70
   60 CONTINUE
      Q = X
      P = T
   70 CONTINUE
      R = DSQRT(P**2+Q**2)
      IF (R.GT.0.D0) GO TO 80
      E1 = A(L,L)
      E2 = A(L+1,L+1)
      A(L+1,L) = 0.D0
      RETURN
   80 CONTINUE
      P = P/R
      Q = Q/R
C PREMULTIPLY.
      DO 90 J=L,N
        Z = A(L,J)
        A(L,J) = P*Z + Q*A(L+1,J)
        A(L+1,J) = P*A(L+1,J) - Q*Z
   90 CONTINUE
C POSTMULTIPLY.
      L1 = L + 1
      DO 100 I=1,L1
        Z = A(I,L)
        A(I,L) = P*Z + Q*A(I,L+1)
        A(I,L+1) = P*A(I,L+1) - Q*Z
  100 CONTINUE
C ACCUMULATE THE TRANSFORMATION IN V.
      DO 110 I=1,N
        Z = V(I,L)
        V(I,L) = P*Z + Q*V(I,L+1)
        V(I,L+1) = P*V(I,L+1) - Q*Z
  110 CONTINUE
      A(L+1,L) = 0.D0
      E1 = A(L,L)
      E2 = A(L+1,L+1)
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE EXCHNG(A, V, N, L, B1, B2, EPS, FAIL, NA, NV)          EXC   10
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER B1, B2, L, NA, NV
      DOUBLE PRECISION A(NA,N), EPS, V(NV,N)
      LOGICAL FAIL
C GIVEN THE UPPER HESSENBERG MATRIX A WITH CONSECUTIVE
C B1XB1 AND B2XB2 DIAGONAL BLOCKS (B1,B2 .LE. 2)
C STARTING AT A(L,L), EXCHNG PRODUCES A UNITARY
C SIMILARITY TRANSFORMATION THAT EXCHANGES THE BLOCKS
C ALONG WITH THEIR EIGENVALUES.  THE TRANSFORMATION
C IS ACCUMULATED IN V.  EXCHNG REQUIRES THE SUBROUTINE
C QRSTEP.  THE PARAMETERS IN THE CALLING SEQUENCE ARE
C %STARRED PARAMETERS ARE ALTERED BY THE SUBROUTINE)
C    *A       THE MATRIX WHOSE BLOCKS ARE TO BE
C             INTERCHANGED.
C    *V       THE ARRAY INTO WHICH THE TRANSFORMATIONS
C             ARE TO BE ACCUMULATED.
C     N       THE ORDER OF THE MATRIX A.
C     L       THE POSITION OF THE BLOCKS.
C     B1      AN INTEGER CONTAINING THE SIZE OF THE
C             FIRST BLOCK.
C     B2      AN INTEGER CONTAINING THE SIZE OF THE
C             SECOND BLOCK.
C     EPS     A CONVERGENCE CRITERION (CF. HQR3).
C    *FAIL    A LOGICAL VARIABLE WHICH IS FALSE ON A
C             NORMAL RETURN.  IF THIRTY ITERATIONS WERE
C             PERFORMED WITHOUT CONVERGENCE, FAIL IS SET
C             TO TRUE AND THE ELEMENT
C             A(L+B2,L+B2-1) CANNOT BE ASSUMED ZERO.
C     NA      THE FIRST DIMENSION OF THE ARRAY A.
C     NV      THE FIRST DIMENSION OF THE ARRAY V.
C INTERNAL VARIABLES.
      INTEGER I, IT, J, L1, M
      DOUBLE PRECISION P, Q, R, S, W, X, Y, Z
      FAIL = .FALSE.
      IF (B1.EQ.2) GO TO 70
      IF (B2.EQ.2) GO TO 40
C INTERCHANGE 1X1 AND 1X1 BLOCKS.
      L1 = L + 1
      Q = A(L+1,L+1) - A(L,L)
      P = A(L,L+1)
      R = DMAX1(DABS(P),DABS(Q))
      IF (R.EQ.0.D0) RETURN
      P = P/R
      Q = Q/R
      R = DSQRT(P**2+Q**2)
      P = P/R
      Q = Q/R
      DO 10 J=L,N
        S = P*A(L,J) + Q*A(L+1,J)
        A(L+1,J) = P*A(L+1,J) - Q*A(L,J)
        A(L,J) = S
   10 CONTINUE
      DO 20 I=1,L1
        S = P*A(I,L) + Q*A(I,L+1)
        A(I,L+1) = P*A(I,L+1) - Q*A(I,L)
        A(I,L) = S
   20 CONTINUE
      DO 30 I=1,N
        S = P*V(I,L) + Q*V(I,L+1)
        V(I,L+1) = P*V(I,L+1) - Q*V(I,L)
        V(I,L) = S
   30 CONTINUE
      A(L+1,L) = 0.D0
      RETURN
   40 CONTINUE
C INTERCHANGE 1X1 AND 2X2 BLOCKS.
      X = A(L,L)
      P = 1.D0
      Q = 1.D0
      R = 1.D0
      CALL QRSTEP(A, V, P, Q, R, L, L+2, N, NA, NV)
      IT = 0
   50 IT = IT + 1
      IF (IT.LE.30) GO TO 60
      FAIL = .TRUE.
      RETURN
   60 CONTINUE
      P = A(L,L) - X
      Q = A(L+1,L)
      R = 0.D0
      CALL QRSTEP(A, V, P, Q, R, L, L+2, N, NA, NV)
      IF (DABS(A(L+2,L+1)).GT.EPS*(DABS(A(L+1,L+1))+DABS(A(L+2,L+2))))
     * GO TO 50
      A(L+2,L+1) = 0.D0
      RETURN
   70 CONTINUE
C INTERCHANGE 2X2 AND B2XB2 BLOCKS.
      M = L + 2
      IF (B2.EQ.2) M = M + 1
      X = A(L+1,L+1)
      Y = A(L,L)
      W = A(L+1,L)*A(L,L+1)
      P = 1.D0
      Q = 1.D0
      R = 1.D0
      CALL QRSTEP(A, V, P, Q, R, L, M, N, NA, NV)
      IT = 0
   80 IT = IT + 1
      IF (IT.LE.30) GO TO 90
      FAIL = .TRUE.
      RETURN
   90 CONTINUE
      Z = A(L,L)
      R = X - Z
      S = Y - Z
      P = (R*S-W)/A(L+1,L) + A(L,L+1)
      Q = A(L+1,L+1) - Z - R - S
      R = A(L+2,L+1)
      S = DABS(P) + DABS(Q) + DABS(R)
      P = P/S
      Q = Q/S
      R = R/S
      CALL QRSTEP(A, V, P, Q, R, L, M, N, NA, NV)
      IF (DABS(A(M-1,M-2)).GT.EPS*(DABS(A(M-1,M-1))+DABS(A(M-2,M-2))))
     * GO TO 80
      A(M-1,M-2) = 0.D0
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE QRSTEP(A, V, P, Q, R, NL, NU, N, NA, NV)               QRS   10
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER N, NA, NL, NU, NV
      DOUBLE PRECISION A(NA,N), P, Q, R, V(NV,N)
C QRSTEP PERFORMS ONE IMPLICIT QR STEP ON THE
C UPPER HESSENBERG MATRIX A.  THE SHIFT IS DETERMINED
C BY THE NUMBERS P,Q, AND R, AND THE STEP IS APPLIED TO
C ROWS AND COLUMNS NL THROUGH NU.  THE TRANSFORMATIONS
C ARE ACCUMULATED IN V.  THE PARAMETERS IN THE CALLING
C SEQUENCE ARE (STARRED APRAMETERS ARE ALTERED BY THE
C SUBROUTINE)
C    *A       THE UPPER HESSENBERG MATRIX ON WHICH THE
C             QR STEP IS TO BE PERFORMED.
C    *V       THE ARRAY IN WHICH THE TRANSFORMATIONS
C             ARE TO BE ACCUMULATED
C    *P       PARAMETERS THAT DETERMINE THE SHIFT.
C    *Q
C    *R
C     NL      THE LOWER LIMIT OF THE STEP.
C     NU      THE UPPER LIMIT OF THE STEP.
C     N       THE ORDER OF THE MATRIX A.
C     NA      THE FIRST DIMENSION OF THE ARRAY A.
C     NV      THE FIRST DIMENSION OF THE ARRAY V.
C INTERNAL VARIABLES.
      INTEGER I, J, K, NL2, NL3, NUM1
      DOUBLE PRECISION S, X, Y, Z
      LOGICAL LAST
      NL2 = NL + 2
      DO 10 I=NL2,NU
        A(I,I-2) = 0.D0
   10 CONTINUE
      IF (NL2.EQ.NU) GO TO 30
      NL3 = NL + 3
      DO 20 I=NL3,NU
        A(I,I-3) = 0.D0
   20 CONTINUE
   30 CONTINUE
      NUM1 = NU - 1
      DO 130 K=NL,NUM1
C DETERMINE THE TRANSFORMATION.
        LAST = K.EQ.NUM1
        IF (K.EQ.NL) GO TO 40
        P = A(K,K-1)
        Q = A(K+1,K-1)
        R = 0.D0
        IF (.NOT.LAST) R = A(K+2,K-1)
        X = DABS(P) + DABS(Q) + DABS(R)
        IF (X.EQ.0.D0) GO TO 130
        P = P/X
        Q = Q/X
        R = R/X
   40   CONTINUE
        S = DSQRT(P**2+Q**2+R**2)
        IF (P.LT.0.D0) S = -S
        IF (K.EQ.NL) GO TO 50
        A(K,K-1) = -S*X
        GO TO 60
   50   CONTINUE
        IF (NL.NE.1) A(K,K-1) = -A(K,K-1)
   60   CONTINUE
        P = P + S
        X = P/S
        Y = Q/S
        Z = R/S
        Q = Q/P
        R = R/P
C PREMULTIPLY.
        DO 80 J=K,N
          P = A(K,J) + Q*A(K+1,J)
          IF (LAST) GO TO 70
          P = P + R*A(K+2,J)
          A(K+2,J) = A(K+2,J) - P*Z
   70     CONTINUE
          A(K+1,J) = A(K+1,J) - P*Y
          A(K,J) = A(K,J) - P*X
   80   CONTINUE
C POSTMULTIPLY.
        J = MIN0(K+3,NU)
        DO 100 I=1,J
          P = X*A(I,K) + Y*A(I,K+1)
          IF (LAST) GO TO 90
          P = P + Z*A(I,K+2)
          A(I,K+2) = A(I,K+2) - P*R
   90     CONTINUE
          A(I,K+1) = A(I,K+1) - P*Q
          A(I,K) = A(I,K) - P
  100   CONTINUE
C ACCUMULATE THE TRANSFORMATION IN V.
        DO 120 I=1,N
          P = X*V(I,K) + Y*V(I,K+1)
          IF (LAST) GO TO 110
          P = P + Z*V(I,K+2)
          V(I,K+2) = V(I,K+2) - P*R
  110     CONTINUE
          V(I,K+1) = V(I,K+1) - P*Q
          V(I,K) = V(I,K) - P
  120   CONTINUE
  130 CONTINUE
      RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE ORTHES(NM,N,LOW,IGH,A,ORT)
C
      INTEGER I,J,M,N,II,JJ,LA,MP,NM,IGH,KP1,LOW
      DOUBLE PRECISION A(NM,N),ORT(IGH)
      DOUBLE PRECISION F,G,H,SCALE
C
C     this subroutine is a translation of the algol procedure orthes,
C     num. math. 12, 349-368(1968) by martin and wilkinson.
C     handbook for auto. comp., vol.ii-linear algebra, 339-358(1971).
C
C     given a real general matrix, this subroutine
C     reduces a submatrix situated in rows and columns
C     low through igh to upper hessenberg form by
C     orthogonal similarity transformations.
C
C     on input
C
C        nm must be set to the row dimension of two-dimensional
C          array parameters as declared in the calling program
C          dimension statement.
C
C        n is the order of the matrix.
C
C        low and igh are integers determined by the balancing
C          subroutine  balanc.  if  balanc  has not been used,
C          set low=1, igh=n.
C
C        a contains the input matrix.
C
C     on output
C
C        a contains the hessenberg matrix.  information about
C          the orthogonal transformations used in the reduction
C          is stored in the remaining triangle under the
C          hessenberg matrix.
C
C        ort contains further information about the transformations.
C          only elements low through igh are used.
C
C     questions and comments should be directed to burton s. garbow,
C     mathematics and computer science div, argonne national laboratory
C
C     this version dated august 1983.
C
C     ------------------------------------------------------------------
C
      LA = IGH - 1
      KP1 = LOW + 1
      IF (LA .LT. KP1) GO TO 200
C
      DO 180 M = KP1, LA
         H = 0.0D0
         ORT(M) = 0.0D0
         SCALE = 0.0D0
C     .......... scale column (algol tol then not needed) ..........
         DO 90 I = M, IGH
   90    SCALE = SCALE + DABS(A(I,M-1))
C
         IF (SCALE .EQ. 0.0D0) GO TO 180
         MP = M + IGH
C     .......... for i=igh step -1 until m do -- ..........
         DO 100 II = M, IGH
            I = MP - II
            ORT(I) = A(I,M-1) / SCALE
            H = H + ORT(I) * ORT(I)
  100    CONTINUE
C
         G = -DSIGN(DSQRT(H),ORT(M))
         H = H - ORT(M) * G
         ORT(M) = ORT(M) - G
C     .......... form (i-(u*ut)/h) * a ..........
         DO 130 J = M, N
            F = 0.0D0
C     .......... for i=igh step -1 until m do -- ..........
            DO 110 II = M, IGH
               I = MP - II
               F = F + ORT(I) * A(I,J)
  110       CONTINUE
C
            F = F / H
C
            DO 120 I = M, IGH
  120       A(I,J) = A(I,J) - F * ORT(I)
C
  130    CONTINUE
C     .......... form (i-(u*ut)/h)*a*(i-(u*ut)/h) ..........
         DO 160 I = 1, IGH
            F = 0.0D0
C     .......... for j=igh step -1 until m do -- ..........
            DO 140 JJ = M, IGH
               J = MP - JJ
               F = F + ORT(J) * A(I,J)
  140       CONTINUE
C
            F = F / H
C
            DO 150 J = M, IGH
  150       A(I,J) = A(I,J) - F * ORT(J)
C
  160    CONTINUE
C
         ORT(M) = SCALE * ORT(M)
         A(M,M-1) = SCALE * G
  180 CONTINUE
C
  200 RETURN
      END
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      SUBROUTINE ORTRAN(NM,N,LOW,IGH,A,ORT,Z)
C
      INTEGER I,J,N,KL,MM,MP,NM,IGH,LOW,MP1
      DOUBLE PRECISION A(NM,IGH),ORT(IGH),Z(NM,N)
      DOUBLE PRECISION G
C
C     this subroutine is a translation of the algol procedure ortrans,
C     num. math. 16, 181-204(1970) by peters and wilkinson.
C     handbook for auto. comp., vol.ii-linear algebra, 372-395(1971).
C
C     this subroutine accumulates the orthogonal similarity
C     transformations used in the reduction of a real general
C     matrix to upper hessenberg form by  orthes.
C
C     on input
C
C        nm must be set to the row dimension of two-dimensional
C          array parameters as declared in the calling program
C          dimension statement.
C
C        n is the order of the matrix.
C
C        low and igh are integers determined by the balancing
C          subroutine  balanc.  if  balanc  has not been used,
C          set low=1, igh=n.
C
C        a contains information about the orthogonal trans-
C          formations used in the reduction by  orthes
C          in its strict lower triangle.
C
C        ort contains further information about the trans-
C          formations used in the reduction by  orthes.
C          only elements low through igh are used.
C
C     on output
C
C        z contains the transformation matrix produced in the
C          reduction by  orthes.
C
C        ort has been altered.
C
C     questions and comments should be directed to burton s. garbow,
C     mathematics and computer science div, argonne national laboratory
C
C     this version dated august 1983.
C
C     ------------------------------------------------------------------
C
C     .......... initialize z to identity matrix ..........
      DO 80 J = 1, N
C
         DO 60 I = 1, N
   60    Z(I,J) = 0.0D0
C
         Z(J,J) = 1.0D0
   80 CONTINUE
C
      KL = IGH - LOW - 1
      IF (KL .LT. 1) GO TO 200
C     .......... for mp=igh-1 step -1 until low+1 do -- ..........
      DO 140 MM = 1, KL
         MP = IGH - MM
         IF (A(MP,MP-1) .EQ. 0.0D0) GO TO 140
         MP1 = MP + 1
C
         DO 100 I = MP1, IGH
  100    ORT(I) = A(I,MP-1)
C
         DO 130 J = MP, IGH
            G = 0.0D0
C
            DO 110 I = MP, IGH
  110       G = G + ORT(I) * Z(I,J)
C     .......... divisor below is negative of h formed in orthes.
C                double division avoids possible underflow ..........
            G = (G / ORT(MP)) / A(MP,MP-1)
C
            DO 120 I = MP, IGH
  120       Z(I,J) = Z(I,J) + G * ORT(I)
C
  130    CONTINUE
C
  140 CONTINUE
C
  200 RETURN
      END
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
